@@ -2,10 +2,10 @@
 
 // * Miscellaneous settings
 
-if ($hassiteconfig) { // speedup for non-admins, add all caps used on this page
+// Experimental settings page.
+$ADMIN->add('development', new admin_category('experimental', new lang_string('experimental', 'admin')));
 
-    // Experimental settings page
-    $ADMIN->add('development', new admin_category('experimental', new lang_string('experimental','admin')));
+if ($hassiteconfig) { // Speedup for non-admins, add all caps used on this page.
 
     $temp = new admin_settingpage('experimentalsettings', new lang_string('experimentalsettings', 'admin'));
     //TODO: Re-enable cc-import once re-implemented in 2.0.x
@@ -109,7 +109,16 @@ if ($hassiteconfig) { // speedup for non-admins, add all caps used on this page
     // Add the 'profiling' page to admin block.
     $ADMIN->add('development', $temp);
 
-     // Web service test clients DO NOT COMMIT : THE EXTERNAL WEB PAGE IS NOT AN ADMIN PAGE !!!!!
+    $ADMIN->add(
+        parentname: 'development',
+        something: new admin_externalpage(
+            name: 'swaggerui',
+            visiblename: new lang_string('swaggerui', 'admin'),
+            url: "$CFG->wwwroot/admin/swaggerui.php",
+        ),
+    );
+
+    // Web service test clients DO NOT COMMIT : THE EXTERNAL WEB PAGE IS NOT AN ADMIN PAGE !!!!!
     $ADMIN->add('development', new admin_externalpage('testclient', new lang_string('testclient', 'webservice'), "$CFG->wwwroot/$CFG->admin/webservice/testclient.php"));
 
 
