@@ -18,16 +18,25 @@
  * Hook callbacks for Moodle app tools
  *
  * @package    tool_mobile
- * @copyright  2024 Juan Leyva
+ * @copyright  2023 Marina Glancy
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
 
 defined('MOODLE_INTERNAL') || die();
 
 $callbacks = [
+    [
+        'hook' => \core\hook\output\before_standard_head_html_generation::class,
+        'callback' => [\tool_mobile\hook_callbacks::class, 'before_standard_head_html_generation'],
+    ],
+    [
+        'hook' => \core\hook\output\before_standard_footer_html_generation::class,
+        'callback' => [\tool_mobile\hook_callbacks::class, 'before_standard_footer_html_generation'],
+        'priority' => 0,
+    ],
     [
         'hook' => \core_user\hook\after_login_completed::class,
-        'callback' => \tool_mobile\hook_callbacks::class . '::after_login_completed',
+        'callback' => [\tool_mobile\hook_callbacks::class, 'after_login_completed'],
         'priority' => 500,
     ],
     [
