@@ -38,7 +38,7 @@ class manager {
     /**
      * Default mlbackend
      */
-    const DEFAULT_MLBACKEND = '\mlbackend_php\processor';
+    const DEFAULT_MLBACKEND = '\mlbackend_python\processor';
 
     /**
      * Name of the file where components declare their models.
@@ -48,7 +48,7 @@ class manager {
     /**
      * @var \core_analytics\predictor[]
      */
-    protected static $predictionprocessors = null;
+    protected static $predictionprocessors = [];
 
     /**
      * @var \core_analytics\local\target\base[]
@@ -99,6 +99,22 @@ class manager {
         }
     }
 
+    /**
+     * Is analytics enabled globally?
+     *
+     * return bool
+     */
+    public static function is_analytics_enabled(): bool {
+        global $CFG;
+
+        if (isset($CFG->enableanalytics)) {
+            return $CFG->enableanalytics;
+        }
+
+        // Enabled by default.
+        return true;
+    }
+
     /**
      * Returns all system models that match the provided filters.
      *
@@ -143,6 +159,10 @@ class manager {
                 $models[$modelobj->id] = $model;
             }
         }
+
+        // Sort the models by the model name using the current session language.
+        \core_collator::asort_objects_by_method($models, 'get_name');
+
         return $models;
     }
 
@@ -209,6 +229,14 @@ class manager {
         return $predictionprocessors;
     }
 
+    /**
+     * Resets the cached prediction processors.
+     * @return null
+     */
+    public static function reset_prediction_processors() {
+        self::$predictionprocessors = [];
+    }
+
     /**
      * Returns the name of the provided predictions processor.
      *
@@ -229,11 +257,16 @@ class manager {
     public static function is_mlbackend_used($plugin) {
         $models = self::get_all_models();
         foreach ($models as $model) {
-            $processor = $model->get_predictions_processor();
-            $noprefixnamespace = ltrim(get_class($processor), '\\');
-            $processorplugin = substr($noprefixnamespace, 0, strpos($noprefixnamespace, '\\'));
-            if ($processorplugin == $plugin) {
-                return true;
+            try {
+                $processor = $model->get_predictions_processor();
+                $noprefixnamespace = ltrim(get_class($processor), '\\');
+                $processorplugin = substr($noprefixnamespace, 0, strpos($noprefixnamespace, '\\'));
+                if ($processorplugin == $plugin) {
+                    return true;
+                }
+            } catch (\Exception $e) {
+                // The model does not have a predictions processor.
+                continue;
             }
         }
 
@@ -271,21 +304,6 @@ class manager {
         return self::$alltimesplittings;
     }
 
-    /**
-     * Returns the enabled time splitting methods.
-     *
-     * @deprecated since Moodle 3.7
-     * @todo MDL-65086 This will be deleted in Moodle 4.1
-     * @see \core_analytics\manager::get_time_splitting_methods_for_evaluation
-     * @return \core_analytics\local\time_splitting\base[]
-     */
-    public static function get_enabled_time_splitting_methods() {
-        debugging('This function has been deprecated. You can use self::get_time_splitting_methods_for_evaluation if ' .
-            'you want to get the default time splitting methods for evaluation, or you can use self::get_all_time_splittings if ' .
-            'you want to get all the time splitting methods available on this site.');
-        return self::get_time_splitting_methods_for_evaluation();
-    }
-
     /**
      * Returns the time-splitting methods for model evaluation.
      *
@@ -335,7 +353,7 @@ class manager {
      *
      * @return \core_analytics\local\target\base[]
      */
-    public static function get_all_targets() : array {
+    public static function get_all_targets(): array {
         if (self::$alltargets !== null) {
             return self::$alltargets;
         }
@@ -511,7 +529,7 @@ class manager {
      * @param  int|null $newmodelid A new model to add to the list of models with insights in the provided context.
      * @return int[]
      */
-    public static function cached_models_with_insights(\context $context, int $newmodelid = null) {
+    public static function cached_models_with_insights(\context $context, ?int $newmodelid = null) {
 
         $cache = \cache::make('core', 'contextwithinsights');
         $modelids = $cache->get($context->id);
@@ -576,13 +594,13 @@ class manager {
      * Used to be used to add models included with the Moodle core.
      *
      * @deprecated Deprecated since Moodle 3.7 (MDL-61667) - Use lib/db/analytics.php instead.
-     * @todo Remove this method in Moodle 4.1 (MDL-65186).
+     * @todo Remove this method in Moodle 3.11 (MDL-65186).
      * @return void
      */
     public static function add_builtin_models() {
 
-        debugging('core_analytics\manager::add_builtin_models() has been deprecated. Core models are now automatically '.
-            'updated according to their declaration in the lib/db/analytics.php file.', DEBUG_DEVELOPER);
+        throw new \coding_exception('core_analytics\manager::add_builtin_models() has been removed. Core models ' .
+                        'are now automatically updated according to their declaration in the lib/db/analytics.php file.');
     }
 
     /**
@@ -596,9 +614,25 @@ class manager {
                         LEFT JOIN {context} ctx ON ap.contextid = ctx.id
                             WHERE ctx.id IS NULL)");
 
-        $contextsql = "SELECT id FROM {context} ctx";
-        $DB->delete_records_select('analytics_predictions', "contextid NOT IN ($contextsql)");
-        $DB->delete_records_select('analytics_indicator_calc', "contextid NOT IN ($contextsql)");
+        // Cleanup analaytics predictions/calcs with MySQL friendly sub-select.
+        $DB->execute("DELETE FROM {analytics_predictions} WHERE id IN (
+                        SELECT oldpredictions.id
+                        FROM (
+                            SELECT p.id
+                            FROM {analytics_predictions} p
+                            LEFT JOIN {context} ctx ON p.contextid = ctx.id
+                            WHERE ctx.id IS NULL
+                        ) oldpredictions
+                    )");
+
+        $DB->execute("DELETE FROM {analytics_indicator_calc} WHERE id IN (
+                        SELECT oldcalcs.id FROM (
+                            SELECT c.id
+                            FROM {analytics_indicator_calc} c
+                            LEFT JOIN {context} ctx ON c.contextid = ctx.id
+                            WHERE ctx.id IS NULL
+                        ) oldcalcs
+                    )");
 
         // Clean up stuff that depends on analysable ids that do not exist anymore.
 
@@ -618,6 +652,8 @@ class manager {
             $usedanalysablesanalysableids = array_flip($usedanalysablesanalysableids);
 
             $analyser = $model->get_analyser(array('notimesplitting' => true));
+
+            // We do not honour the list of contexts in this model as it can contain stale records.
             $analysables = $analyser->get_analysables_iterator();
 
             $analysableids = [];
@@ -648,6 +684,13 @@ class manager {
                     $param + $idsparams);
             }
         }
+
+        // Clean up calculations table.
+        $calclifetime = get_config('analytics', 'calclifetime');
+        if (!empty($calclifetime)) {
+            $lifetime = time() - ($calclifetime * DAYSECS); // Value in days.
+            $DB->delete_records_select('analytics_indicator_calc', 'timecreated < ?', [$lifetime]);
+        }
     }
 
     /**
@@ -731,6 +774,27 @@ class manager {
             } else {
                 $model['enabled'] = clean_param($model['enabled'], PARAM_BOOL);
             }
+
+            // For the core models only, automatically remove references to modules that do not
+            // exist. This allows you to install without error if there are missing plugins.
+            if ($componentname === 'moodle') {
+                $updatedindicators = [];
+                $allmodules = [];
+                foreach ($model['indicators'] as $indicator) {
+                    if (preg_match('~^\\\\mod_([^\\\\]+)\\\\~', $indicator, $matches)) {
+                        if (!$allmodules) {
+                            // The first time, get all modules.
+                            $allmodules = \core\plugin_manager::instance()->get_present_plugins('mod');
+                        }
+                        if (!array_key_exists($matches[1], $allmodules)) {
+                            // Module does not exist, so skip indicator.
+                            continue;
+                        }
+                    }
+                    $updatedindicators[] = $indicator;
+                }
+                $model['indicators'] = $updatedindicators;
+            }
         }
 
         static::validate_models_declaration($models);
@@ -862,7 +926,7 @@ class manager {
      * @param array $model Model declaration
      * @return string complying with PARAM_ALPHANUM rules and starting with an 'id' prefix
      */
-    public static function model_declaration_identifier(array $model) : string {
+    public static function model_declaration_identifier(array $model): string {
         return 'id'.sha1(serialize($model));
     }
 
@@ -885,4 +949,79 @@ class manager {
 
         return [$target, $indicators];
     }
+
+    /**
+     * Return the context restrictions that can be applied to the provided context levels.
+     *
+     * @throws \coding_exception
+     * @param  array|null $contextlevels The list of context levels provided by the analyser. Null if all of them.
+     * @param  string|null $query
+     * @return array Associative array with contextid as key and the short version of the context name as value.
+     */
+    public static function get_potential_context_restrictions(?array $contextlevels = null, ?string $query = null) {
+        global $DB;
+
+        if (empty($contextlevels) && !is_null($contextlevels)) {
+            return false;
+        }
+
+        if (!is_null($contextlevels)) {
+            foreach ($contextlevels as $contextlevel) {
+                if ($contextlevel !== CONTEXT_COURSE && $contextlevel !== CONTEXT_COURSECAT) {
+                    throw new \coding_exception('Only CONTEXT_COURSE and CONTEXT_COURSECAT are supported at the moment.');
+                }
+            }
+        }
+
+        $contexts = [];
+
+        // We have a separate process for each context level for performance reasons (to iterate through mdl_context calling
+        // get_context_name() would be too slow).
+        $contextsystem = \context_system::instance();
+        if (is_null($contextlevels) || in_array(CONTEXT_COURSECAT, $contextlevels)) {
+
+            $sql = "SELECT cc.id, cc.name, ctx.id AS contextid
+                      FROM {course_categories} cc
+                      JOIN {context} ctx ON ctx.contextlevel = :ctxlevel AND ctx.instanceid = cc.id";
+            $params = ['ctxlevel' => CONTEXT_COURSECAT];
+
+            if ($query) {
+                $sql .= " WHERE " . $DB->sql_like('cc.name', ':query', false, false);
+                $params['query'] = '%' . $query . '%';
+            }
+
+            $coursecats = $DB->get_recordset_sql($sql, $params);
+            foreach ($coursecats as $record) {
+                $contexts[$record->contextid] = get_string('category') . ': ' .
+                    format_string($record->name, true, array('context' => $contextsystem));
+            }
+            $coursecats->close();
+        }
+
+        if (is_null($contextlevels) || in_array(CONTEXT_COURSE, $contextlevels)) {
+
+            $sql = "SELECT c.id, c.shortname, ctx.id AS contextid
+                      FROM {course} c
+                      JOIN {context} ctx ON ctx.contextlevel = :ctxlevel AND ctx.instanceid = c.id
+                      WHERE c.id != :siteid";
+            $params = ['ctxlevel' => CONTEXT_COURSE, 'siteid' => SITEID];
+
+            if ($query) {
+                $sql .= ' AND (' . $DB->sql_like('c.fullname', ':query1', false, false) . ' OR ' .
+                    $DB->sql_like('c.shortname', ':query2', false, false) . ')';
+                $params['query1'] = '%' . $query . '%';
+                $params['query2'] = '%' . $query . '%';
+            }
+
+            $courses = $DB->get_recordset_sql($sql, $params);
+            foreach ($courses as $record) {
+                $contexts[$record->contextid] = get_string('course') . ': ' .
+                    format_string($record->shortname, true, array('context' => $contextsystem));
+            }
+            $courses->close();
+        }
+
+        return $contexts;
+    }
+
 }
