@@ -58,7 +58,7 @@ require_once($CFG->dirroot . '/backup/util/ui/backup_ui_setting.class.php');
  * @copyright 2010 onwards Eloy Lafuente (stronk7) {@link http://stronk7.com}
  * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
-class settings_test extends \basic_testcase {
+final class settings_test extends \basic_testcase {
 
     /**
      * test base_setting class
