@@ -38,7 +38,7 @@ require_once(__DIR__ . '/../lti_advantage_testcase.php');
  * @license http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  * @coversDefaultClass \enrol_lti\local\ltiadvantage\service\tool_launch_service
  */
-class tool_launch_service_test extends \lti_advantage_testcase {
+final class tool_launch_service_test extends \lti_advantage_testcase {
 
     /**
      * Test the use case "A user launches a tool so they can view an external resource/activity".
@@ -49,7 +49,7 @@ class tool_launch_service_test extends \lti_advantage_testcase {
      * @param array $expected the array detailing expectations.
      * @covers ::user_launches_tool
      */
-    public function test_user_launches_tool(?array $legacydata, ?array $launchdata, array $expected) {
+    public function test_user_launches_tool(?array $legacydata, ?array $launchdata, array $expected): void {
         $this->resetAfterTest();
         // Setup.
         $contextrepo = new context_repository();
@@ -72,7 +72,7 @@ class tool_launch_service_test extends \lti_advantage_testcase {
         }
 
         // Get a mock 1.3 launch, optionally including the lti1p1 migration claim based on a legacy tool secret.
-        $mocklaunch = $this->get_mock_launch($modresource, $launchdata['user'], null, true, true,
+        $mocklaunch = $this->get_mock_launch($modresource, $launchdata['user'], null, [], true,
             $launchdata['launch_migration_claim']);
 
         // Call the service.
@@ -123,12 +123,12 @@ class tool_launch_service_test extends \lti_advantage_testcase {
      *
      * @return array the test case data.
      */
-    public function user_launch_provider(): array {
+    public static function user_launch_provider(): array {
         return [
             'New tool: no legacy data, no migration claim sent' => [
-                'legacy_data' => null,
-                'launch_data' => [
-                    'user' => $this->get_mock_launch_users_with_ids(['1p3_1'])[0],
+                'legacydata' => null,
+                'launchdata' => [
+                    'user' => self::get_mock_launch_users_with_ids(['1p3_1'])[0],
                     'launch_migration_claim' => null,
                 ],
                 'expected' => [
@@ -136,15 +136,15 @@ class tool_launch_service_test extends \lti_advantage_testcase {
                 ]
             ],
             'Migrated tool: Legacy data exists, no change in user_id so omitted from claim' => [
-                'legacy_data' => [
+                'legacydata' => [
                     'consumer_key' => 'CONSUMER_1',
                     'tools' => [
                         ['secret' => 'toolsecret1'],
                         ['secret' => 'toolsecret2'],
                     ]
                 ],
-                'launch_data' => [
-                    'user' => $this->get_mock_launch_users_with_ids(['1'])[0],
+                'launchdata' => [
+                    'user' => self::get_mock_launch_users_with_ids(['1'])[0],
                     'launch_migration_claim' => [
                         'consumer_key' => 'CONSUMER_1',
                         'signing_secret' => 'toolsecret1',
@@ -159,15 +159,15 @@ class tool_launch_service_test extends \lti_advantage_testcase {
             ],
 
             'Migrated tool: Legacy data exists, platform signs with different valid secret' => [
-                'legacy_data' => [
+                'legacydata' => [
                     'consumer_key' => 'CONSUMER_1',
                     'tools' => [
                         ['secret' => 'toolsecret1'],
                         ['secret' => 'toolsecret2'],
                     ]
                 ],
-                'launch_data' => [
-                    'user' => $this->get_mock_launch_users_with_ids(['1p3_1'])[0],
+                'launchdata' => [
+                    'user' => self::get_mock_launch_users_with_ids(['1p3_1'])[0],
                     'launch_migration_claim' => [
                         'consumer_key' => 'CONSUMER_1',
                         'signing_secret' => 'toolsecret2',
@@ -181,15 +181,15 @@ class tool_launch_service_test extends \lti_advantage_testcase {
                 ]
             ],
             'Migrated tool: Legacy data exists, no migration claim sent' => [
-                'legacy_data' => [
+                'legacydata' => [
                     'consumer_key' => 'CONSUMER_1',
                     'tools' => [
                         ['secret' => 'toolsecret1'],
                         ['secret' => 'toolsecret2'],
                     ]
                 ],
-                'launch_data' => [
-                    'user' => $this->get_mock_launch_users_with_ids(['1p3_1'])[0],
+                'launchdata' => [
+                    'user' => self::get_mock_launch_users_with_ids(['1p3_1'])[0],
                     'launch_migration_claim' => null,
                 ],
                 'expected' => [
@@ -197,15 +197,15 @@ class tool_launch_service_test extends \lti_advantage_testcase {
                 ]
             ],
             'Migrated tool: Legacy data exists, migration claim signature generated using invalid secret' => [
-                'legacy_data' => [
+                'legacydata' => [
                     'consumer_key' => 'CONSUMER_1',
                     'tools' => [
                         ['secret' => 'toolsecret1'],
                         ['secret' => 'toolsecret2'],
                     ]
                 ],
-                'launch_data' => [
-                    'user' => $this->get_mock_launch_users_with_ids(['1p3_1'])[0],
+                'launchdata' => [
+                    'user' => self::get_mock_launch_users_with_ids(['1p3_1'])[0],
                     'launch_migration_claim' => [
                         'consumer_key' => 'CONSUMER_1',
                         'signing_secret' => 'secret-not-mapped-to-consumer',
@@ -221,15 +221,15 @@ class tool_launch_service_test extends \lti_advantage_testcase {
                 ]
             ],
             'Migrated tool: Legacy data exists, migration claim signature omitted' => [
-                'legacy_data' => [
+                'legacydata' => [
                     'consumer_key' => 'CONSUMER_1',
                     'tools' => [
                         ['secret' => 'toolsecret1'],
                         ['secret' => 'toolsecret2'],
                     ]
                 ],
-                'launch_data' => [
-                    'user' => $this->get_mock_launch_users_with_ids(['1p3_1'])[0],
+                'launchdata' => [
+                    'user' => self::get_mock_launch_users_with_ids(['1p3_1'])[0],
                     'launch_migration_claim' => [
                         'consumer_key' => 'CONSUMER_1',
                         'user_id' => 'user-id-123',
@@ -244,15 +244,15 @@ class tool_launch_service_test extends \lti_advantage_testcase {
                 ]
             ],
             'Migrated tool: Legacy data exists, migration claim missing oauth_consumer_key' => [
-                'legacy_data' => [
+                'legacydata' => [
                     'consumer_key' => 'CONSUMER_1',
                     'tools' => [
                         ['secret' => 'toolsecret1'],
                         ['secret' => 'toolsecret2'],
                     ]
                 ],
-                'launch_data' => [
-                    'user' => $this->get_mock_launch_users_with_ids(['1p3_1'])[0],
+                'launchdata' => [
+                    'user' => self::get_mock_launch_users_with_ids(['1p3_1'])[0],
                     'launch_migration_claim' => [
                         'user_id' => 'user-id-123',
                         'context_id' => 'd345b',
@@ -272,13 +272,13 @@ class tool_launch_service_test extends \lti_advantage_testcase {
      *
      * @covers ::user_launches_tool
      */
-    public function test_user_launches_tool_missing_custom_id() {
+    public function test_user_launches_tool_missing_custom_id(): void {
         $this->resetAfterTest();
         [$course, $modresource] = $this->create_test_environment();
         $instructoruser = $this->getDataGenerator()->create_user();
         $launchservice = $this->get_tool_launch_service();
-        $mockuser = $this->get_mock_launch_users_with_ids(['1p3_1'])[0];
-        $mocklaunch = $this->get_mock_launch($modresource, $mockuser, null, false, false, null, []);
+        $mockuser = self::get_mock_launch_users_with_ids(['1p3_1'])[0];
+        $mocklaunch = $this->get_mock_launch($modresource, $mockuser, null, null, false, null, []);
 
         $this->expectException(\moodle_exception::class);
         $this->expectExceptionMessage(get_string('ltiadvlauncherror:missingid', 'enrol_lti'));
@@ -290,13 +290,13 @@ class tool_launch_service_test extends \lti_advantage_testcase {
      *
      * @covers ::user_launches_tool
      */
-    public function test_user_launches_tool_invalid_custom_id() {
+    public function test_user_launches_tool_invalid_custom_id(): void {
         $this->resetAfterTest();
         [$course, $modresource] = $this->create_test_environment();
         $instructoruser = $this->getDataGenerator()->create_user();
         $launchservice = $this->get_tool_launch_service();
-        $mockuser = $this->get_mock_launch_users_with_ids(['1p3_1'])[0];
-        $mocklaunch = $this->get_mock_launch($modresource, $mockuser, null, false, false, null, ['id' => 999999]);
+        $mockuser = self::get_mock_launch_users_with_ids(['1p3_1'])[0];
+        $mocklaunch = $this->get_mock_launch($modresource, $mockuser, null, null, false, null, ['id' => 999999]);
 
         $this->expectException(\moodle_exception::class);
         $this->expectExceptionMessage(get_string('ltiadvlauncherror:invalidid', 'enrol_lti', 999999));
@@ -308,7 +308,7 @@ class tool_launch_service_test extends \lti_advantage_testcase {
      *
      * @covers ::user_launches_tool
      */
-    public function test_user_launches_tool_missing_registration() {
+    public function test_user_launches_tool_missing_registration(): void {
         $this->resetAfterTest();
         // Setup.
         [
@@ -327,7 +327,7 @@ class tool_launch_service_test extends \lti_advantage_testcase {
 
         // Call the service.
         $launchservice = $this->get_tool_launch_service();
-        $mockuser = $this->get_mock_launch_users_with_ids(['1p3_1'])[0];
+        $mockuser = self::get_mock_launch_users_with_ids(['1p3_1'])[0];
         $mocklaunch = $this->get_mock_launch($modresource, $mockuser);
 
         $this->expectException(\moodle_exception::class);
@@ -341,7 +341,7 @@ class tool_launch_service_test extends \lti_advantage_testcase {
      *
      * @covers ::user_launches_tool
      */
-    public function test_user_launches_tool_missing_deployment() {
+    public function test_user_launches_tool_missing_deployment(): void {
         $this->resetAfterTest();
         // Setup.
         [
@@ -360,7 +360,7 @@ class tool_launch_service_test extends \lti_advantage_testcase {
 
         // Call the service.
         $launchservice = $this->get_tool_launch_service();
-        $mockuser = $this->get_mock_launch_users_with_ids(['1p3_1'])[0];
+        $mockuser = self::get_mock_launch_users_with_ids(['1p3_1'])[0];
         $mocklaunch = $this->get_mock_launch($modresource, $mockuser);
 
         $this->expectException(\moodle_exception::class);
@@ -374,7 +374,7 @@ class tool_launch_service_test extends \lti_advantage_testcase {
      *
      * @covers ::user_launches_tool
      */
-    public function test_user_launches_tool_role_mapping() {
+    public function test_user_launches_tool_role_mapping(): void {
         $this->resetAfterTest();
         // Create mock launches for 3 different user types: instructor, admin, learner.
         [$course, $modresource] = $this->create_test_environment();
@@ -382,18 +382,18 @@ class tool_launch_service_test extends \lti_advantage_testcase {
         $instructor2user = $this->getDataGenerator()->create_user();
         $adminuser = $this->getDataGenerator()->create_user();
         $learneruser = $this->getDataGenerator()->create_user();
-        $mockinstructoruser = $this->get_mock_launch_users_with_ids(['1'])[0];
-        $mockadminuser = $this->get_mock_launch_users_with_ids(
+        $mockinstructoruser = self::get_mock_launch_users_with_ids(['1'])[0];
+        $mockadminuser = self::get_mock_launch_users_with_ids(
             ['2'],
             false,
             'http://purl.imsglobal.org/vocab/lis/v2/system/person#Administrator'
         )[0];
-        $mocklearneruser = $this->get_mock_launch_users_with_ids(
+        $mocklearneruser = self::get_mock_launch_users_with_ids(
             ['3'],
             false,
             'http://purl.imsglobal.org/vocab/lis/v2/membership#Learner'
         )[0];
-        $mockinstructor2user = $this->get_mock_launch_users_with_ids(
+        $mockinstructor2user = self::get_mock_launch_users_with_ids(
             ['3'],
             false,
             'Instructor' // Using the legacy (deprecated in 1.3) simple name.
@@ -429,11 +429,11 @@ class tool_launch_service_test extends \lti_advantage_testcase {
      *
      * @covers ::user_launches_tool
      */
-    public function test_user_launches_tool_user_fields_updated() {
+    public function test_user_launches_tool_user_fields_updated(): void {
         $this->resetAfterTest();
         [$course, $modresource] = $this->create_test_environment();
         $user = $this->getDataGenerator()->create_user();
-        $mockinstructoruser = $this->get_mock_launch_users_with_ids(['1'])[0];
+        $mockinstructoruser = self::get_mock_launch_users_with_ids(['1'])[0];
         $launchservice = $this->get_tool_launch_service();
         $userrepo = new user_repository();
 
@@ -476,12 +476,12 @@ class tool_launch_service_test extends \lti_advantage_testcase {
      *
      * @covers ::user_launches_tool
      */
-    public function test_user_launches_tool_max_enrolment_start_restriction() {
+    public function test_user_launches_tool_max_enrolment_start_restriction(): void {
         $this->resetAfterTest();
         [$course, $modresource] = $this->create_test_environment(true, true, false,
             \enrol_lti\helper::MEMBER_SYNC_ENROL_NEW, false, false, time() + DAYSECS);
         $instructoruser = $this->getDataGenerator()->create_user();
-        $mockinstructoruser = $this->get_mock_launch_users_with_ids(['1'])[0];
+        $mockinstructoruser = self::get_mock_launch_users_with_ids(['1'])[0];
         $mockinstructorlaunch = $this->get_mock_launch($modresource, $mockinstructoruser);
         $launchservice = $this->get_tool_launch_service();
 
@@ -494,18 +494,18 @@ class tool_launch_service_test extends \lti_advantage_testcase {
      *
      * @covers ::user_launches_tool
      */
-    public function test_user_launches_tool_force_embedding_custom_param() {
+    public function test_user_launches_tool_force_embedding_custom_param(): void {
         $this->resetAfterTest();
         [$course, $modresource] = $this->create_test_environment();
         $instructoruser = $this->getDataGenerator()->create_user();
         $learneruser = $this->getDataGenerator()->create_user();
-        $mockinstructoruser = $this->get_mock_launch_users_with_ids(['1'])[0];
-        $mocklearneruser = $this->get_mock_launch_users_with_ids(['1'], false, '')[0];
-        $mockinstructorlaunch = $this->get_mock_launch($modresource, $mockinstructoruser, null, false, false, null, [
+        $mockinstructoruser = self::get_mock_launch_users_with_ids(['1'])[0];
+        $mocklearneruser = self::get_mock_launch_users_with_ids(['1'], false, '')[0];
+        $mockinstructorlaunch = $this->get_mock_launch($modresource, $mockinstructoruser, null, null, false, null, [
             'id' => $modresource->uuid,
             'forcedembed' => true
         ]);
-        $mocklearnerlaunch = $this->get_mock_launch($modresource, $mocklearneruser, null, false, false, null, [
+        $mocklearnerlaunch = $this->get_mock_launch($modresource, $mocklearneruser, null, null, false, null, [
             'id' => $modresource->uuid,
             'forcedembed' => true
         ]);
@@ -514,10 +514,315 @@ class tool_launch_service_test extends \lti_advantage_testcase {
 
         // Instructors aren't subject to forceembed.
         $launchservice->user_launches_tool($instructoruser, $mockinstructorlaunch);
-        $this->assertObjectNotHasAttribute('forcepagelayout', $SESSION);
+        $this->assertObjectNotHasProperty('forcepagelayout', $SESSION);
 
         // Learners are.
         $launchservice->user_launches_tool($learneruser, $mocklearnerlaunch);
         $this->assertEquals('embedded', $SESSION->forcepagelayout);
     }
+
+    /**
+     * Test launching the tool with different 'aud' values, confirming the service handles all variations appropriately.
+     *
+     * @param mixed $aud the aud value to test
+     * @param array $expected the array of expectations to check
+     * @dataProvider aud_data_provider
+     * @covers ::user_launches_tool
+     */
+    public function test_user_launches_tool_aud_variations($aud, array $expected): void {
+        $this->resetAfterTest();
+        [$course, $modresource] = $this->create_test_environment();
+        $instructoruser = $this->getDataGenerator()->create_user();
+        $mockinstructoruser = self::get_mock_launch_users_with_ids(['1'])[0];
+        $mockinstructorlaunch = $this->get_mock_launch($modresource, $mockinstructoruser, null, null, false, null, [
+            'id' => $modresource->uuid,
+        ], $aud);
+
+        $launchservice = $this->get_tool_launch_service();
+        if (isset($expected['exception'])) {
+            $this->expectException($expected['exception']);
+            $this->expectExceptionMessage($expected['exceptionmessage']);
+        }
+        [$userid, $resource] = $launchservice->user_launches_tool($instructoruser, $mockinstructorlaunch);
+
+        $this->assertNotEmpty($userid);
+        $this->assertNotEmpty($resource);
+    }
+
+    /**
+     * Data provider for testing variations of the 'aud' JWT property.
+     *
+     * @return array the test case data
+     */
+    public static function aud_data_provider(): array {
+        return [
+            'valid, array having multiple entries with the first one being clientid' => [
+                'aud' => ['123', 'something else', 'blah'],
+                'expected' => [
+                    'valid' => true
+                ]
+            ],
+            'valid, array having a single entry being clientid' => [
+                'aud' => ['123'],
+                'expected' => [
+                    'valid' => true
+                ]
+            ],
+            'valid, string containing the single clientid' => [
+                'aud' => '123',
+                'expected' => [
+                    'valid' => true
+                ]
+            ],
+            'invalid, array having multiple values where the first item is not clientid' => [
+                'aud' => ['cat', 'dog', '123'],
+                'expected' => [
+                    'valid' => false,
+                    'exception' => \moodle_exception::class,
+                    'exceptionmessage' => get_string('ltiadvlauncherror:invalidregistration', 'enrol_lti')
+                ]
+            ],
+            'invalid, array containing a single item which is not clientid' => [
+                'aud' => ['cat'],
+                'expected' => [
+                    'valid' => false,
+                    'exception' => \moodle_exception::class,
+                    'exceptionmessage' => get_string('ltiadvlauncherror:invalidregistration', 'enrol_lti')
+                ]
+            ],
+            'invalid, string contains the item and it is not the clientid' => [
+                'aud' => 'cat',
+                'expected' => [
+                    'valid' => false,
+                    'exception' => \moodle_exception::class,
+                    'exceptionmessage' => get_string('ltiadvlauncherror:invalidregistration', 'enrol_lti')
+                ]
+            ],
+            'invalid, empty string' => [
+                'aud' => '',
+                'expected' => [
+                    'valid' => false,
+                    'exception' => \moodle_exception::class,
+                    'exceptionmessage' => get_string('ltiadvlauncherror:invalidregistration', 'enrol_lti')
+                ]
+            ],
+        ];
+    }
+
+    /**
+     * Test verifying how changes to lti-ags claim information is handled across different launches.
+     *
+     * @param array $agsclaim1 the lti-ags claim data to use in the first launch.
+     * @param array $agsclaim2 the lti-ags claim data to use in the second launch.
+     * @param array $expected the array of test case expectations.
+     * @dataProvider ags_claim_provider
+     * @covers ::user_launches_tool
+     */
+    public function test_user_launches_tool_ags_claim_handling(array $agsclaim1, array $agsclaim2, array $expected): void {
+        $this->resetAfterTest();
+        [$course, $modresource] = $this->create_test_environment();
+        $instructoruser = $this->getDataGenerator()->create_user();
+        $mockinstructoruser = self::get_mock_launch_users_with_ids(['1'])[0];
+        $userrepo = new user_repository();
+        $resourcelinkrepo = new resource_link_repository();
+        $launchservice = $this->get_tool_launch_service();
+
+        // Launch the first time.
+        $mockinstructorlaunch = $this->get_mock_launch($modresource, $mockinstructoruser, null, $agsclaim1, false, null, [
+            'id' => $modresource->uuid,
+        ]);
+        [$userid, $resource] = $launchservice->user_launches_tool($instructoruser, $mockinstructorlaunch);
+
+        $ltiuser = $userrepo->find_single_user_by_resource($userid, $resource->id);
+        $resourcelink = $resourcelinkrepo->find_by_resource_and_user($resource->id, $ltiuser->get_id())[0];
+        $gradeservice = $resourcelink->get_grade_service();
+        $lineitemurl = $agsclaim1['lineitem'] ?? null;
+        $lineitemsurl = $agsclaim1['lineitems'] ?? null;
+        $this->assertEquals($agsclaim1['scope'], $gradeservice->get_scopes());
+        $this->assertEquals($lineitemurl, $gradeservice->get_lineitemurl());
+        $this->assertEquals($lineitemsurl, $gradeservice->get_lineitemsurl());
+
+        // Launch again, with a new lti-ags claim.
+        $mockinstructorlaunch = $this->get_mock_launch($modresource, $mockinstructoruser, null, $agsclaim2, false, null, [
+            'id' => $modresource->uuid,
+        ]);
+        [$userid, $resource] = $launchservice->user_launches_tool($instructoruser, $mockinstructorlaunch);
+        $ltiuser = $userrepo->find_single_user_by_resource($userid, $resource->id);
+        $resourcelink = $resourcelinkrepo->find_by_resource_and_user($resource->id, $ltiuser->get_id())[0];
+        $gradeservice = $resourcelink->get_grade_service();
+        $lineitemurl = $expected['lineitem'] ?? null;
+        $lineitemsurl = $expected['lineitems'] ?? null;
+        $this->assertEquals($expected['scope'], $gradeservice->get_scopes());
+        $this->assertEquals($lineitemurl, $gradeservice->get_lineitemurl());
+        $this->assertEquals($lineitemsurl, $gradeservice->get_lineitemsurl());
+    }
+
+    /**
+     * Data provider for testing user_launches tool with varying mocked lti-ags claim data over several launches.
+     *
+     * @return array the array of test case data.
+     */
+    public static function ags_claim_provider(): array {
+        return [
+            'Coupled line item with score post only, no change to scopes on subsequent launch' => [
+                'agsclaim1' => [
+                    "scope" => [
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/score"
+                    ],
+                    "lineitem" => "https://platform.example.com/10/lineitems/45/lineitem"
+                ],
+                'agsclaim2' => [
+                    "scope" => [
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/score"
+                    ],
+                    "lineitem" => "https://platform.example.com/10/lineitems/45/lineitem"
+                ],
+                'expected' => [
+                    "scope" => [
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/score"
+                    ],
+                    "lineitem" => "https://platform.example.com/10/lineitems/45/lineitem"
+                ]
+            ],
+            'Coupled line item with score post only, addition to scopes on subsequent launch' => [
+                'agsclaim1' => [
+                    "scope" => [
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/score"
+                    ],
+                    "lineitem" => "https://platform.example.com/10/lineitems/45/lineitem"
+                ],
+                'agsclaim2' => [
+                    "scope" => [
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/result.readonly",
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/score",
+                    ],
+                    "lineitem" => "https://platform.example.com/10/lineitems/45/lineitem"
+                ],
+                'expected' => [
+                    "scope" => [
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/result.readonly",
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/score",
+                    ],
+                    "lineitem" => "https://platform.example.com/10/lineitems/45/lineitem"
+                ]
+            ],
+            'Coupled line item with score post + result read, removal of scopes on subsequent launch' => [
+                'agsclaim1' => [
+                    "scope" => [
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/result.readonly",
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/score"
+                    ],
+                    "lineitem" => "https://platform.example.com/10/lineitems/45/lineitem"
+                ],
+                'agsclaim2' => [
+                    "scope" => [
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/score",
+                    ],
+                    "lineitem" => "https://platform.example.com/10/lineitems/45/lineitem"
+                ],
+                'expected' => [
+                    "scope" => [
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/score",
+                    ],
+                    "lineitem" => "https://platform.example.com/10/lineitems/45/lineitem"
+                ]
+            ],
+            'Decoupled line items with all capabilities, change and removal of scopes on subsequent launch' => [
+                'agsclaim1' => [
+                    "scope" => [
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/lineitem",
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/result.readonly",
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/score"
+                    ],
+                    "lineitems" => "https://platform.example.com/10/lineitems/",
+                ],
+                'agsclaim2' => [
+                    "scope" => [
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/lineitem.readonly",
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/score"
+                    ],
+                    "lineitems" => "https://platform.example.com/10/lineitems/",
+                ],
+                'expected' => [
+                    "scope" => [
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/lineitem.readonly",
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/score"
+                    ],
+                    "lineitems" => "https://platform.example.com/10/lineitems/",
+                ]
+            ],
+            'Decoupled line items with all capabilities, removal of scopes on subsequent launch' => [
+                'agsclaim1' => [
+                    "scope" => [
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/lineitem",
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/result.readonly",
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/score"
+                    ],
+                    "lineitems" => "https://platform.example.com/10/lineitems/",
+                ],
+                'agsclaim2' => [
+                    "scope" => [
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/lineitem",
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/score"
+                    ],
+                    "lineitems" => "https://platform.example.com/10/lineitems/",
+                ],
+                'expected' => [
+                    "scope" => [
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/lineitem",
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/score"
+                    ],
+                    "lineitems" => "https://platform.example.com/10/lineitems/",
+                ]
+            ],
+            'Coupled line items with score post only, addition of lineitemsurl and all capabilities on subsequent launch' => [
+                'agsclaim1' => [
+                    "scope" => [
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/score"
+                    ],
+                    "lineitem" => "https://platform.example.com/10/lineitems/45/lineitem"
+                ],
+                'agsclaim2' => [
+                    "scope" => [
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/lineitem",
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/result.readonly",
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/score"
+                    ],
+                    "lineitems" => "https://platform.example.com/10/lineitems/",
+                    "lineitem" => "https://platform.example.com/10/lineitems/45/lineitem"
+                ],
+                'expected' => [
+                    "scope" => [
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/lineitem",
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/result.readonly",
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/score"
+                    ],
+                    "lineitems" => "https://platform.example.com/10/lineitems/",
+                    "lineitem" => "https://platform.example.com/10/lineitems/45/lineitem"
+                ]
+            ],
+            'Decoupled line items with all capabilities, change to coupled line item with score post only on subsequent launch' => [
+                'agsclaim1' => [
+                    "scope" => [
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/lineitem",
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/result.readonly",
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/score"
+                    ],
+                    "lineitems" => "https://platform.example.com/10/lineitems/",
+                ],
+                'agsclaim2' => [
+                    "scope" => [
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/score"
+                    ],
+                    "lineitem" => "https://platform.example.com/10/lineitems/45/lineitem"
+                ],
+                'expected' => [
+                    "scope" => [
+                        "https://purl.imsglobal.org/spec/lti-ags/scope/score"
+                    ],
+                    "lineitem" => "https://platform.example.com/10/lineitems/45/lineitem"
+                ]
+            ],
+        ];
+    }
 }
