@@ -1,5 +1,4 @@
 <?php
-
 // This file is part of Moodle - http://moodle.org/
 //
 // Moodle is free software: you can redistribute it and/or modify
@@ -15,6 +14,14 @@
 // You should have received a copy of the GNU General Public License
 // along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
 
+/**
+ * A page for managing custom and standard scales
+ *
+ * @package   core_grades
+ * @copyright 2007 Petr Skoda
+ * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
+ */
+
 require_once '../../../config.php';
 require_once $CFG->dirroot.'/grade/lib.php';
 require_once $CFG->libdir.'/gradelib.php';
@@ -27,15 +34,17 @@ $PAGE->set_url('/grade/edit/scale/index.php', array('id' => $courseid));
 /// Make sure they can even access this course
 if ($courseid) {
     if (!$course = $DB->get_record('course', array('id' => $courseid))) {
-        print_error('nocourseid');
+        throw new \moodle_exception('invalidcourseid');
     }
     require_login($course);
-    $context = get_context_instance(CONTEXT_COURSE, $course->id);
+    $context = context_course::instance($course->id);
     require_capability('moodle/course:managescales', $context);
     $PAGE->set_pagelayout('admin');
 } else {
     require_once $CFG->libdir.'/adminlib.php';
     admin_externalpage_setup('scales');
+    $context = context_system::instance();
+    $PAGE->set_primary_active_tab('siteadminnode');
 }
 
 /// return tracking object
@@ -47,7 +56,6 @@ $strcustomscales   = get_string('scalescustom');
 $strname           = get_string('name');
 $strdelete         = get_string('delete');
 $stredit           = get_string('edit');
-$srtcreatenewscale = get_string('scalescustomcreate');
 $strused           = get_string('used');
 $stredit           = get_string('edit');
 
@@ -62,9 +70,9 @@ switch ($action) {
         }
 
         if (empty($scale->courseid)) {
-            require_capability('moodle/course:managescales', get_context_instance(CONTEXT_SYSTEM));
+            require_capability('moodle/course:managescales', context_system::instance());
         } else if ($scale->courseid != $courseid) {
-            print_error('invalidcourseid');
+            throw new \moodle_exception('invalidcourseid');
         }
 
         if (!$scale->can_delete()) {
@@ -74,7 +82,11 @@ switch ($action) {
         $deleteconfirmed = optional_param('deleteconfirmed', 0, PARAM_BOOL);
 
         if (!$deleteconfirmed) {
-            $strdeletescale = get_string('delete'). ' '. get_string('scale');
+            if ($courseid) {
+                $PAGE->navbar->add(get_string('scales'), new moodle_url('/grade/edit/scale/index.php',
+                    ['id' => $courseid]));
+            }
+            $strdeletescale = get_string('deletescale', 'grades');
             $PAGE->navbar->add($strdeletescale);
             $PAGE->set_title($strdeletescale);
             $PAGE->set_heading($COURSE->fullname);
@@ -85,7 +97,8 @@ switch ($action) {
                     'sesskey' =>  sesskey(),
                     'deleteconfirmed'=> 1));
 
-            echo $OUTPUT->confirm(get_string('scaleconfirmdelete', 'grades', $scale->name), $confirmurl, "index.php?id={$courseid}");
+            echo $OUTPUT->confirm(get_string('scaleconfirmdelete', 'grades', $scale->get_name()), $confirmurl,
+                "index.php?id={$courseid}");
             echo $OUTPUT->footer();
             die;
         } else {
@@ -108,7 +121,7 @@ if ($courseid and $scales = grade_scale::fetch_all_local($courseid)) {
     $data = array();
     foreach($scales as $scale) {
         $line = array();
-        $line[] = format_string($scale->name).'<div class="scale_options">'.str_replace(",",", ",$scale->scale).'</div>';
+        $line[] = $scale->get_name() .'<div class="scale_options">'.str_replace(",", ", ", $scale->scale).'</div>';
 
         $used = $scale->is_used();
         $line[] = $used ? get_string('yes') : get_string('no');
@@ -134,16 +147,16 @@ if ($scales = grade_scale::fetch_all_global()) {
     $data = array();
     foreach($scales as $scale) {
         $line = array();
-        $line[] = format_string($scale->name).'<div class="scale_options">'.str_replace(",",", ",$scale->scale).'</div>';
+        $line[] = $scale->get_name().'<div class="scale_options">'.str_replace(",", ", ", $scale->scale).'</div>';
 
         $used = $scale->is_used();
         $line[] = $used ? get_string('yes') : get_string('no');
 
         $buttons = "";
-        if (has_capability('moodle/course:managescales', get_context_instance(CONTEXT_SYSTEM))) {
+        if (has_capability('moodle/course:managescales', context_system::instance())) {
             $buttons .= grade_button('edit', $courseid, $scale);
         }
-        if (!$used and has_capability('moodle/course:managescales', get_context_instance(CONTEXT_SYSTEM))) {
+        if (!$used and has_capability('moodle/course:managescales', context_system::instance())) {
             $buttons .= grade_button('delete', $courseid, $scale);
         }
         $line[] = $buttons;
@@ -156,16 +169,32 @@ if ($scales = grade_scale::fetch_all_global()) {
     $table2->data  = $data;
 }
 
+$actionbar = new \core_grades\output\scales_action_bar($context);
 
 if ($courseid) {
-    print_grade_page_head($courseid, 'scale', 'scale', get_string('coursescales', 'grades'));
+    print_grade_page_head($courseid, 'scale', 'scale', false,
+        false, false, true, null, null, null, $actionbar);
+} else {
+    $renderer = $PAGE->get_renderer('core_grades');
+    echo $renderer->render_action_bar($actionbar);
+    echo $OUTPUT->heading(get_string('scales', 'core'));
 }
 
-echo $OUTPUT->heading($strcustomscales, 3, 'main');
-echo html_writer::table($table);
-echo $OUTPUT->heading($strstandardscale, 3, 'main');
-echo html_writer::table($table2);
-echo $OUTPUT->container_start('buttons');
-echo $OUTPUT->single_button(new moodle_url('edit.php', array('courseid'=>$courseid)), $srtcreatenewscale);
-echo $OUTPUT->container_end();
+$hascustomscales = !empty($table->data);
+$hasstandardscales = !empty($table2->data);
+
+// If there are custom scales available in this context, output the custom scales table and a heading.
+if ($hascustomscales) {
+    echo $OUTPUT->heading($strcustomscales, 3, 'main mt-3');
+    echo html_writer::table($table);
+}
+// If there are standard scales available in this context, output the standard scales table and a heading.
+if ($hasstandardscales) {
+    echo $OUTPUT->heading($strstandardscale, 3, 'main  mt-3');
+    echo html_writer::table($table2);
+}
+// If the are no available scales, display a notification.
+if (!$hascustomscales && !$hasstandardscales) {
+    echo $OUTPUT->notification(get_string('noexistingscales', 'grades'), 'info', false);
+}
 echo $OUTPUT->footer();
