@@ -34,7 +34,7 @@ require_once($CFG->dirroot . '/webservice/tests/helpers.php');
  * @copyright  2015 Juan Leyva <juan@moodle.com>
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
-class externallib_test extends externallib_advanced_testcase {
+final class externallib_test extends externallib_advanced_testcase {
 
     /**
      * Loads some data to be used by the different tests
@@ -95,7 +95,7 @@ class externallib_test extends externallib_advanced_testcase {
     /**
      * Test get_grades_table function case teacher
      */
-    public function test_get_grades_table_teacher() {
+    public function test_get_grades_table_teacher(): void {
 
         $this->resetAfterTest(true);
 
@@ -129,7 +129,7 @@ class externallib_test extends externallib_advanced_testcase {
     /**
      * Test get_grades_table function case student
      */
-    public function test_get_grades_table_student() {
+    public function test_get_grades_table_student(): void {
         global $CFG, $DB;
 
         $this->resetAfterTest(true);
@@ -173,7 +173,7 @@ class externallib_test extends externallib_advanced_testcase {
     /**
      * Test get_grades_table function case incorrect permissions
      */
-    public function test_get_grades_table_permissions() {
+    public function test_get_grades_table_permissions(): void {
         global $CFG, $DB;
 
         $this->resetAfterTest(true);
@@ -197,7 +197,7 @@ class externallib_test extends externallib_advanced_testcase {
     /**
      * Test view_grade_report function
      */
-    public function test_view_grade_report() {
+    public function test_view_grade_report(): void {
         global $USER;
 
         $this->resetAfterTest(true);
@@ -246,7 +246,7 @@ class externallib_test extends externallib_advanced_testcase {
     /**
      * Test get_grades_items function case teacher
      */
-    public function test_get_grade_items_teacher() {
+    public function test_get_grade_items_teacher(): void {
 
         $this->resetAfterTest(true);
 
@@ -350,7 +350,7 @@ class externallib_test extends externallib_advanced_testcase {
     /**
      * Test get_grades_items function case student
      */
-    public function test_get_grade_items_student() {
+    public function test_get_grade_items_student(): void {
 
         $this->resetAfterTest(true);
 
