@@ -15,7 +15,7 @@
 // along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
 
 /**
- * Defines names of plugin types and some strings used at the plugin managment
+ * Strings for component 'core_plugin', language 'en'.
  *
  * @package    core
  * @subpackage plugin
@@ -47,13 +47,17 @@ $string['dependencymissing'] = 'Missing';
 $string['dependencyunavailable'] = 'Unavailable';
 $string['dependencyupload'] = 'Upload';
 $string['dependencyuploadmissing'] = 'Upload ZIP files';
+$string['deprecated_type'] = "Deprecated plugin type";
 $string['displayname'] = 'Plugin name';
 $string['err_response_curl'] = 'Unable to fetch available updates data - unexpected cURL error.';
 $string['err_response_format_version'] = 'Unexpected version of the response format. Please try to re-check for available updates.';
 $string['err_response_http_code'] = 'Unable to fetch available updates data - unexpected HTTP response code.';
 $string['filterall'] = 'Show all';
 $string['filtercontribonly'] = 'Show additional plugins only';
+$string['filtermissingonly'] = 'Show missing from disk only';
+$string['filternewpluginsonly'] = 'Show only plugins to be installed';
 $string['filterupdatesonly'] = 'Show updateable only';
+$string['incompatibleversion'] = 'Incompatible Moodle version: {$a}';
 $string['isenabled'] = 'Enabled?';
 $string['misdepinfoplugin'] = 'Plugin info';
 $string['misdepinfoversion'] = 'Version info';
@@ -61,12 +65,14 @@ $string['misdepsavail'] = 'Available missing dependencies';
 $string['misdepsunavail'] = 'Unavailable missing dependencies';
 $string['misdepsunavaillist'] = 'No version found to fulfill the dependency requirements: {$a}.';
 $string['misdepsunknownlist'] = 'Not in the Plugins directory: <strong>{$a}</strong>.';
+$string['moodlebranch'] = 'Moodle {$a->min} - {$a->max}';
 $string['moodleversion'] = 'Moodle {$a}';
 $string['noneinstalled'] = 'No plugins of this type are installed';
 $string['notes'] = 'Notes';
 $string['notdownloadable'] = 'Can not download the package';
-$string['notdownloadable_help'] = 'ZIP package with the update can not be downloaded automatically. Please refer to the documentation page for more help.';
+$string['notdownloadable_help'] = 'The ZIP package with the update cannot be downloaded automatically.';
 $string['notdownloadable_link'] = 'admin/mdeploy/notdownloadable';
+$string['notsupported'] = 'Plugin may not be compatible with Moodle version {$a}';
 $string['notwritable'] = 'Plugin files not writable';
 $string['notwritable_help'] = 'Plugin files are not writable by the web server. The web server process must have write access to the plugin folder and all its contents. Write access to the root folder of the given plugin type may also be required.';
 $string['otherplugin'] = '{$a->component}';
@@ -107,6 +113,10 @@ $string['status_uptodate'] = 'Installed';
 $string['supportedmoodleversions'] = 'Supported Moodle versions';
 $string['supportedconversions'] = 'Supported document conversions';
 $string['systemname'] = 'Identifier';
+$string['type_aiplacement'] = 'AI placement plugin';
+$string['type_aiplacement_plural'] = 'AI placement plugins';
+$string['type_aiprovider'] = 'AI provider plugin';
+$string['type_aiprovider_plural'] = 'AI provider plugins';
 $string['type_antivirus'] = 'Antivirus plugin';
 $string['type_antivirus_plural'] = 'Antivirus plugins';
 $string['type_auth'] = 'Authentication method';
@@ -121,6 +131,10 @@ $string['type_cachestore'] = 'Cache store';
 $string['type_cachestore_plural'] = 'Cache stores';
 $string['type_calendartype'] = 'Calendar type';
 $string['type_calendartype_plural'] = 'Calendar types';
+$string['type_communication'] = 'Communication provider';
+$string['type_communication_plural'] = 'Communication providers';
+$string['type_contenttype'] = 'Content bank';
+$string['type_contenttype_plural'] = 'Content bank plugins';
 $string['type_customfield'] = 'Custom field';
 $string['type_customfield_plural'] = 'Custom fields';
 $string['type_coursereport'] = 'Course report';
@@ -142,10 +156,14 @@ $string['type_gradeexport'] = 'Grade export method';
 $string['type_gradeexport_plural'] = 'Grade export methods';
 $string['type_gradeimport'] = 'Grade import method';
 $string['type_gradeimport_plural'] = 'Grade import methods';
+$string['type_gradepenalty'] = 'Grade penalty';
+$string['type_gradepenalty_plural'] = 'Grade penalties';
 $string['type_gradereport'] = 'Gradebook report';
 $string['type_gradereport_plural'] = 'Gradebook reports';
 $string['type_gradingform'] = 'Advanced grading method';
 $string['type_gradingform_plural'] = 'Advanced grading methods';
+$string['type_h5plib'] = 'H5P framework';
+$string['type_h5plib_plural'] = 'H5P frameworks';
 $string['type_mlbackend'] = 'Machine learning backend';
 $string['type_mlbackend_plural'] = 'Machine learning backends';
 $string['type_local'] = 'Local plugin';
@@ -158,6 +176,9 @@ $string['type_mnetservice'] = 'MNet service';
 $string['type_mnetservice_plural'] = 'MNet services';
 $string['type_mod'] = 'Activity module';
 $string['type_mod_plural'] = 'Activity modules';
+$string['type_paygwmanage'] = 'Manage payment gateways';
+$string['type_paygw'] = 'Payment gateway';
+$string['type_paygw_plural'] = 'Payment gateways';
 $string['type_plagiarism'] = 'Plagiarism plugin';
 $string['type_plagiarism_plural'] = 'Plagiarism plugins';
 $string['type_portfolio'] = 'Portfolio';
@@ -176,12 +197,16 @@ $string['type_repository'] = 'Repository';
 $string['type_repository_plural'] = 'Repositories';
 $string['type_search'] = 'Search engine';
 $string['type_search_plural'] = 'Search engines';
+$string['type_smsgateway'] = 'SMS gateway';
+$string['type_smsgateway_plural'] = 'SMS gateways';
 $string['type_theme'] = 'Theme';
 $string['type_theme_plural'] = 'Themes';
 $string['type_tool'] = 'Admin tool';
 $string['type_tool_plural'] = 'Admin tools';
 $string['type_webservice'] = 'Webservice protocol';
 $string['type_webservice_plural'] = 'Webservice protocols';
+$string['type_qbank'] = 'Question bank plugin';
+$string['type_qbank_plural'] = 'Question bank plugins';
 $string['updateavailable'] = 'There is a new version {$a} available!';
 $string['updateavailable_moreinfo'] = 'More info...';
 $string['updateavailable_release'] = 'Release {$a}';
@@ -225,6 +250,7 @@ $string['validationmsg_onedir'] = 'Invalid structure of the ZIP package.';
 $string['validationmsg_onedir_help'] = 'The ZIP package must contain just one root directory that holds the plugin code. The name of that root directory must match the name of the plugin.';
 $string['validationmsg_pathwritable'] = 'Write access check';
 $string['validationmsg_pluginversion'] = 'Plugin version';
+$string['validationmsg_pluginversiontoolow'] = 'A higher version of this plugin is already installed';
 $string['validationmsg_release'] = 'Plugin release';
 $string['validationmsg_requiresmoodle'] = 'Required Moodle version';
 $string['validationmsg_rootdir'] = 'Name of the plugin to be installed';
