@@ -23,6 +23,9 @@
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
 
+use core\di;
+use core\hook;
+
 require_once(__DIR__.'/../../testing/classes/util.php');
 require_once(__DIR__ . "/coverage_info.php");
 
@@ -105,7 +108,7 @@ class phpunit_util extends testing_util {
         global $DB, $CFG, $USER, $SITE, $COURSE, $PAGE, $OUTPUT, $SESSION, $FULLME, $FILTERLIB_PRIVATE;
 
         // Stop all hook redirections.
-        \core\hook\manager::get_instance()->phpunit_stop_redirections();
+        di::get(hook\manager::class)->phpunit_stop_redirections();
 
         // Stop any message redirection.
         self::stop_message_redirection();
@@ -270,6 +273,9 @@ class phpunit_util extends testing_util {
         if (class_exists('\core_group\customfield\grouping_handler')) {
             \core_group\customfield\grouping_handler::reset_caches();
         }
+        if (class_exists('\core_reportbuilder\customfield\report_handler')) {
+            \core_reportbuilder\customfield\report_handler::reset_caches();
+        }
 
         // Clear static cache within restore.
         if (class_exists('restore_section_structure_step')) {
@@ -300,6 +306,9 @@ class phpunit_util extends testing_util {
         // Reset user agent.
         core_useragent::instance(true, null);
 
+        // Reset the DI container.
+        \core\di::reset_container();
+
         // verify db writes just in case something goes wrong in reset
         if (self::$lastdbwrites != $DB->perf_get_writes()) {
             error_log('Unexpected DB writes in phpunit_util::reset_all_data()');
@@ -308,7 +317,7 @@ class phpunit_util extends testing_util {
 
         if ($warnings) {
             $warnings = implode("\n", $warnings);
-            trigger_error($warnings, E_USER_WARNING);
+            throw new \core_phpunit\exception\test_exception($warnings);
         }
     }
 
@@ -974,7 +983,6 @@ class phpunit_util extends testing_util {
     public static function call_internal_method($object, $methodname, array $params, $classname) {
         $reflection = new \ReflectionClass($classname);
         $method = $reflection->getMethod($methodname);
-        $method->setAccessible(true);
         return $method->invokeArgs($object, $params);
     }
 
@@ -985,10 +993,20 @@ class phpunit_util extends testing_util {
      * @param   int     $level The number of levels of indentation to pad
      * @return  string
      */
-    protected static function pad(string $string, int $level) : string {
+    protected static function pad(string $string, int $level): string {
         return str_repeat(" ", $level * 2) . "{$string}\n";
     }
 
+    /**
+     * Normalise any text to always use unix line endings (line-feeds).
+     *
+     * @param   string  $text The text to normalize
+     * @return  string
+     */
+    public static function normalise_line_endings(string $text): string {
+        return str_replace(["\r\n", "\r"], "\n", $text);
+    }
+
     /**
      * Get the coverage config for the supplied includelist and excludelist configuration.
      *
@@ -996,7 +1014,7 @@ class phpunit_util extends testing_util {
      * @param   string[] $excludelists The list of files/folders in the excludelist.
      * @return  string
      */
-    protected static function get_coverage_config(array $includelists, array $excludelists) : string {
+    protected static function get_coverage_config(array $includelists, array $excludelists): string {
         $coverages = '';
         if (!empty($includelists)) {
             $coverages .= self::pad("<include>", 2);
