@@ -1,10 +1,3 @@
-/*
-YUI 3.17.2 (build 9c3c78e)
-Copyright 2014 Yahoo! Inc. All rights reserved.
-Licensed under the BSD License.
-http://yuilibrary.com/license/
-*/
-
 YUI.add('scrollview-base-ie', function (Y, NAME) {
 
 /**
@@ -59,4 +52,4 @@ Y.mix(Y.ScrollView.prototype, {
 }, true);
 
 
-}, '3.17.2', {"requires": ["scrollview-base"]});
+}, '3.18.1', {"requires": ["scrollview-base"]});
