@@ -27,6 +27,7 @@ define(
     'core/notification',
     'core/pubsub',
     'core/str',
+    'core/pending',
     'core/templates',
     'core/user_date',
     'core_message/message_repository',
@@ -42,6 +43,7 @@ function(
     Notification,
     PubSub,
     Str,
+    Pending,
     Templates,
     UserDate,
     MessageRepository,
@@ -63,6 +65,7 @@ function(
         SECTION_TOTAL_COUNT: '[data-region="section-total-count"]',
         SECTION_TOTAL_COUNT_CONTAINER: '[data-region="section-total-count-container"]',
         SECTION_UNREAD_COUNT: '[data-region="section-unread-count"]',
+        SECTION_UNREAD_COUNT_CONTAINER: '[data-region="section-unread-count-container"]',
         PLACEHOLDER_CONTAINER: '[data-region="placeholder-container"]'
     };
 
@@ -73,6 +76,7 @@ function(
 
     var LOAD_LIMIT = 50;
     var loadedConversationsById = {};
+    var deletedConversationsById = {};
     var loadedTotalCounts = false;
     var loadedUnreadCounts = false;
 
@@ -117,7 +121,7 @@ function(
         countElement.text(count);
         container.removeClass('hidden');
         Str.get_string('totalconversations', 'core_message', count).done(function(string) {
-            container.attr('aria-label', string);
+            $('#' + container.attr('aria-labelledby')).text(string);
         });
 
         var numPlaceholders = count > 20 ? 20 : count;
@@ -146,15 +150,18 @@ function(
      * @param {Number} count The unread count
      */
     var renderUnreadCount = function(root, count) {
-        var countElement = root.find(SELECTORS.SECTION_UNREAD_COUNT);
+        var container = root.find(SELECTORS.SECTION_UNREAD_COUNT_CONTAINER);
+        var countElement = container.find(SELECTORS.SECTION_UNREAD_COUNT);
         countElement.text(count);
 
         Str.get_string('unreadconversations', 'core_message', count).done(function(string) {
-            countElement.attr('aria-label', string);
+            $('#' + container.attr('aria-labelledby')).text(string);
         });
 
         if (count > 0) {
-            countElement.removeClass('hidden');
+            container.removeClass('hidden');
+        } else {
+            container.addClass('hidden');
         }
     };
 
@@ -194,68 +201,132 @@ function(
     /**
      * Render the messages in the overview page.
      *
-     * @param {Object} contentContainer Conversations content container.
      * @param {Array} conversations List of conversations to render.
      * @param {Number} userId Logged in user id.
      * @return {Object} jQuery promise.
      */
     var render = function(conversations, userId) {
-        var formattedConversations = conversations.map(function(conversation) {
+
+        // Helper to format the last message for rendering.
+        // Returns a promise which resolves to either a string, or null
+        // (such as in the event of an empty personal space).
+        var pending = new Pending();
+
+        var formatMessagePreview = async function(lastMessage) {
+            if (!lastMessage) {
+                return null;
+            }
+            // Check the message html for a src attribute, indicative of media.
+            // Replace <img with <noimg to stop browsers pre-fetching the image as part of tmp element creation.
+            var tmpElement = document.createElement("element");
+            tmpElement.innerHTML = lastMessage.text.replace(/<img /g, '<noimg ');
+            var isMedia = tmpElement.querySelector("[src]") || false;
+
+            if (!isMedia) {
+                // Try to get the text value of the content.
+                // If that's not possible, we'll report it under the catch-all 'other media'.
+                var messagePreview = $(lastMessage.text).text();
+                if (messagePreview) {
+                    // The text value of the message must have no html/script tags.
+                    if (messagePreview.indexOf('<') == -1) {
+                        return messagePreview;
+                    }
+                }
+            }
+
+            // As a fallback, report unknowns as 'other media' type content.
+            var pix = 'i/messagecontentmultimediageneral';
+            var label = 'messagecontentmultimediageneral';
+
+            if (lastMessage.text.includes('<img')) {
+                pix = 'i/messagecontentimage';
+                label = 'messagecontentimage';
+            } else if (lastMessage.text.includes('<video')) {
+                pix = 'i/messagecontentvideo';
+                label = 'messagecontentvideo';
+            } else if (lastMessage.text.includes('<audio')) {
+                pix = 'i/messagecontentaudio';
+                label = 'messagecontentaudio';
+            }
+
+            try {
+                var labelString = await Str.get_string(label, 'core_message');
+                var icon = await Templates.renderPix(pix, 'core', labelString);
+                return icon + ' ' + labelString;
+            } catch (error) {
+                Notification.exception(error);
+                return null;
+            }
+        };
+
+        var mapPromises = conversations.map(function(conversation) {
 
             var lastMessage = conversation.messages.length ? conversation.messages[conversation.messages.length - 1] : null;
 
-            var formattedConversation = {
-                id: conversation.id,
-                imageurl: conversation.imageurl,
-                name: conversation.name,
-                subname: conversation.subname,
-                unreadcount: conversation.unreadcount,
-                ismuted: conversation.ismuted,
-                lastmessagedate: lastMessage ? lastMessage.timecreated : null,
-                sentfromcurrentuser: lastMessage ? lastMessage.useridfrom == userId : null,
-                lastmessage: lastMessage ? $(lastMessage.text).text() || lastMessage.text : null
-            };
+            return formatMessagePreview(lastMessage)
+                .then(function(messagePreview) {
+                    var formattedConversation = {
+                        id: conversation.id,
+                        imageurl: conversation.imageurl,
+                        name: conversation.name,
+                        subname: conversation.subname,
+                        unreadcount: conversation.unreadcount,
+                        ismuted: conversation.ismuted,
+                        lastmessagedate: lastMessage ? lastMessage.timecreated : null,
+                        sentfromcurrentuser: lastMessage ? lastMessage.useridfrom == userId : null,
+                        lastmessage: messagePreview
+                    };
 
-            var otherUser = null;
-            if (conversation.type == MessageDrawerViewConversationContants.CONVERSATION_TYPES.SELF) {
-                // Self-conversations have only one member.
-                otherUser = conversation.members[0];
-            } else if (conversation.type == MessageDrawerViewConversationContants.CONVERSATION_TYPES.PRIVATE) {
-                // For private conversations, remove the current userId from the members to get the other user.
-                otherUser = conversation.members.reduce(function(carry, member) {
-                    if (!carry && member.id != userId) {
-                        carry = member;
+                    var otherUser = null;
+                    if (conversation.type == MessageDrawerViewConversationContants.CONVERSATION_TYPES.SELF) {
+                        // Self-conversations have only one member.
+                        otherUser = conversation.members[0];
+                    } else if (conversation.type == MessageDrawerViewConversationContants.CONVERSATION_TYPES.PRIVATE) {
+                        // For private conversations, remove the current userId from the members to get the other user.
+                        otherUser = conversation.members.reduce(function(carry, member) {
+                            if (!carry && member.id != userId) {
+                                carry = member;
+                            }
+                            return carry;
+                        }, null);
                     }
-                    return carry;
-                }, null);
-            }
 
-            if (otherUser !== null) {
-                formattedConversation.userid = otherUser.id;
-                formattedConversation.showonlinestatus = otherUser.showonlinestatus;
-                formattedConversation.isonline = otherUser.isonline;
-                formattedConversation.isblocked = otherUser.isblocked;
-            }
-
-            if (conversation.type == MessageDrawerViewConversationContants.CONVERSATION_TYPES.PUBLIC) {
-                formattedConversation.lastsendername = conversation.members.reduce(function(carry, member) {
-                    if (!carry && lastMessage && member.id == lastMessage.useridfrom) {
-                        carry = member.fullname;
+                    if (otherUser !== null) {
+                        formattedConversation.userid = otherUser.id;
+                        formattedConversation.showonlinestatus = otherUser.showonlinestatus;
+                        formattedConversation.isonline = otherUser.isonline;
+                        formattedConversation.isblocked = otherUser.isblocked;
                     }
-                    return carry;
-                }, null);
-            }
 
-            return formattedConversation;
+                    if (conversation.type == MessageDrawerViewConversationContants.CONVERSATION_TYPES.PUBLIC) {
+                        formattedConversation.lastsendername = conversation.members.reduce(function(carry, member) {
+                            if (!carry && lastMessage && member.id == lastMessage.useridfrom) {
+                                carry = member.fullname;
+                            }
+                            return carry;
+                        }, null);
+                    }
+
+                    return formattedConversation;
+                }).catch(Notification.exception);
         });
 
-        formattedConversations.forEach(function(conversation) {
-            if (new Date().toDateString() == new Date(conversation.lastmessagedate * 1000).toDateString()) {
-                conversation.istoday = true;
-            }
-        });
+        return Promise.all(mapPromises)
+            .then(function(formattedConversations) {
+                formattedConversations.forEach(function(conversation) {
+                    if (new Date().toDateString() == new Date(conversation.lastmessagedate * 1000).toDateString()) {
+                        conversation.istoday = true;
+                    }
+                });
 
-        return Templates.render(TEMPLATES.CONVERSATIONS_LIST, {conversations: formattedConversations});
+                return Templates.render(TEMPLATES.CONVERSATIONS_LIST, {conversations: formattedConversations});
+            }).then(function(html, js) {
+                pending.resolve();
+                return $.Deferred().resolve(html, js);
+            }).catch(function(error) {
+                pending.resolve();
+                Notification.exception(error);
+            });
     };
 
     /**
@@ -377,11 +448,8 @@ function(
             var element = getTotalUnreadConversationCountElement(root);
             var count = parseInt(element.text());
             count = count - 1;
-            element.text(count);
-
-            if (count < 1) {
-                element.addClass('hidden');
-            }
+            // Re-render the unread count.
+            renderUnreadCount(root, count);
         }
     };
 
@@ -538,12 +606,8 @@ function(
             return true;
         };
 
-        // Set the minimum height of the section to the height of the toggle. This
-        // smooths out the collapse animation.
-        var toggle = root.find(SELECTORS.TOGGLE);
-        root.css('min-height', toggle.outerHeight());
 
-        root.on('show.bs.collapse', function() {
+        root[0].addEventListener('show.bs.collapse', function() {
             setExpanded(root);
             LazyLoadList.show(listRoot, loadCallback, function(contentContainer, conversations, userId) {
                 return render(conversations, userId)
@@ -555,7 +619,7 @@ function(
             });
         });
 
-        root.on('hidden.bs.collapse', function() {
+        root[0].addEventListener('hidden.bs.collapse', function() {
             setCollapsed(root);
         });
 
@@ -595,6 +659,7 @@ function(
                 return;
             }
 
+            var pendingPromise = new Pending('core_message/message_drawer_view_overview_section:new');
             var loggedInUserId = conversation.loggedInUserId;
             var conversationId = conversation.id;
             var element = getConversationElement(root, conversationId);
@@ -603,19 +668,34 @@ function(
                 var contentContainer = LazyLoadList.getContentContainer(root);
                 render([conversation], loggedInUserId)
                     .then(function(html) {
-                            contentContainer.prepend(html);
-                            element.remove();
-                            return html;
-                        })
+                        if (deletedConversationsById[conversationId]) {
+                            // This conversation was deleted at some point since the messaging drawer was created.
+                            if (conversation.messages[0].timeadded < deletedConversationsById[conversationId]) {
+                                // The 'new' message was added before the conversation was deleted.
+                                // This is probably stale data.
+                                return;
+                            }
+                        }
+                        contentContainer.prepend(html);
+                        element.remove();
+
+                        return;
+                    })
+                    .then(pendingPromise.resolve)
                     .catch(Notification.exception);
+            } else if (conversation.messages.length) {
+                createNewConversationFromEvent(root, conversation, loggedInUserId)
+                .then(pendingPromise.resolve)
+                .catch();
             } else {
-                createNewConversationFromEvent(root, conversation, loggedInUserId);
+                pendingPromise.resolve();
             }
         });
 
         PubSub.subscribe(MessageDrawerEvents.CONVERSATION_DELETED, function(conversationId) {
             var conversationElement = getConversationElement(root, conversationId);
             delete loadedConversationsById[conversationId];
+            deletedConversationsById[conversationId] = new Date();
             if (conversationElement.length) {
                 deleteConversation(root, conversationElement);
             }
