@@ -106,8 +106,9 @@ class assign_override_form extends moodleform {
             // Group override.
             if ($this->groupid) {
                 // There is already a groupid, so freeze the selector.
-                $groupchoices = array();
-                $groupchoices[$this->groupid] = groups_get_group_name($this->groupid);
+                $groupchoices = [
+                    $this->groupid => format_string(groups_get_group_name($this->groupid), true, ['context' => $this->context]),
+                ];
                 $mform->addElement('select', 'groupid',
                         get_string('overridegroup', 'assign'), $groupchoices);
                 $mform->freeze('groupid');
@@ -122,12 +123,14 @@ class assign_override_form extends moodleform {
                 if (empty($groups)) {
                     // Generate an error.
                     $link = new moodle_url('/mod/assign/overrides.php', array('cmid' => $cm->id));
-                    print_error('groupsnone', 'assign', $link);
+                    throw new \moodle_exception('groupsnone', 'assign', $link);
                 }
 
                 $groupchoices = array();
                 foreach ($groups as $group) {
-                    $groupchoices[$group->id] = $group->name;
+                    if ($group->visibility != GROUPS_VISIBILITY_NONE) {
+                        $groupchoices[$group->id] = format_string($group->name, true, ['context' => $this->context]);
+                    }
                 }
                 unset($groups);
 
@@ -157,23 +160,14 @@ class assign_override_form extends moodleform {
 
                 // Get the list of appropriate users, depending on whether and how groups are used.
                 $userfieldsapi = \core_user\fields::for_name();
-                if ($accessallgroups) {
-                    $users = get_enrolled_users($this->context, '', 0,
-                            'u.id, u.email, ' . $userfieldsapi->get_sql('u', false, '', '', false)->selects, $sort);
-                } else if ($groups = groups_get_activity_allowed_groups($cm)) {
-                    $enrolledjoin = get_enrolled_join($this->context, 'u.id');
-                    $userfields = 'u.id, u.email, ' . $userfieldsapi->get_sql('u', false, '', '', false)->selects;
-                    list($ingroupsql, $ingroupparams) = $DB->get_in_or_equal(array_keys($groups), SQL_PARAMS_NAMED);
-                    $params = $enrolledjoin->params + $ingroupparams;
-                    $sql = "SELECT $userfields
-                              FROM {user} u
-                              JOIN {groups_members} gm ON gm.userid = u.id
-                                   {$enrolledjoin->joins}
-                             WHERE gm.groupid $ingroupsql
-                                   AND {$enrolledjoin->wheres}
-                          ORDER BY $sort";
-                    $users = $DB->get_records_sql($sql, $params);
+                $userfields = 'u.id, u.email, ' . $userfieldsapi->get_sql('u', false, '', '', false)->selects;
+                $groupids = 0;
+                if (!$accessallgroups) {
+                    $groups = groups_get_activity_allowed_groups($cm);
+                    $groupids = array_keys($groups);
                 }
+                $users = get_enrolled_users($this->context, '',
+                        $groupids, $userfields, $sort);
 
                 // Filter users based on any fixed restrictions (groups, profile).
                 $info = new \core_availability\info_module($cm);
@@ -182,7 +176,7 @@ class assign_override_form extends moodleform {
                 if (empty($users)) {
                     // Generate an error.
                     $link = new moodle_url('/mod/assign/overrides.php', array('cmid' => $cm->id));
-                    print_error('usersnone', 'assign', $link);
+                    throw new \moodle_exception('usersnone', 'assign', $link);
                 }
 
                 $userchoices = array();
@@ -257,8 +251,22 @@ class assign_override_form extends moodleform {
             get_string('allowsubmissionsfromdate', 'assign'), array('optional' => true));
         $mform->setDefault('allowsubmissionsfromdate', $assigninstance->allowsubmissionsfromdate);
 
+        // Add the option to recalculate the penalty if there is existing grade.
+        if (\mod_assign\penalty\helper::is_penalty_enabled($assigninstance->id) && $this->assign->count_grades() > 0) {
+            // Create notification.
+            $notice = $OUTPUT->notification(get_string('penaltyduedatechangemessage', 'assign'), 'warning', false);
+            $mform->addElement('html', $notice);
+            $mform->addElement('select', 'recalculatepenalty', get_string('modgraderecalculatepenalty', 'grades'), [
+                '' => get_string('choose'),
+                'no' => get_string('no'),
+                'yes' => get_string('yes'),
+            ]);
+            $mform->addHelpButton('recalculatepenalty', 'modgraderecalculatepenalty', 'grades');
+        }
+
         $mform->addElement('date_time_selector', 'duedate', get_string('duedate', 'assign'), array('optional' => true));
         $mform->setDefault('duedate', $assigninstance->duedate);
+        $mform->disabledIf('duedate', 'recalculatepenalty', 'eq', '');
 
         $mform->addElement('date_time_selector', 'cutoffdate', get_string('cutoffdate', 'assign'), array('optional' => true));
         $mform->setDefault('cutoffdate', $assigninstance->cutoffdate);
@@ -269,9 +277,12 @@ class assign_override_form extends moodleform {
         }
 
         // Time limit.
-        $mform->addElement('duration', 'timelimit',
-            get_string('timelimit', 'assign'), array('optional' => true));
-        $mform->setDefault('timelimit', $assigninstance->timelimit);
+        $timelimitenabled = get_config('assign', 'enabletimelimit');
+        if ($timelimitenabled) {
+            $mform->addElement('duration', 'timelimit',
+                get_string('timelimit', 'assign'), array('optional' => true));
+            $mform->setDefault('timelimit', $assigninstance->timelimit);
+        }
 
         // Submit buttons.
         $mform->addElement('submit', 'resetbutton',
@@ -289,6 +300,24 @@ class assign_override_form extends moodleform {
 
     }
 
+    /**
+     * Override definition after data has been set.
+     *
+     * The value of date time selector will be lost in a POST request, if the selector is disabled.
+     * So, we need to set the value again.
+     *
+     * return void
+     */
+    public function definition_after_data() {
+        $mform = $this->_form;
+
+        // The value of date time selector will be lost in a POST request.
+        $recalculatepenalty = optional_param('recalculatepenalty', null, PARAM_TEXT);
+        if ($recalculatepenalty === '') {
+            $mform->setConstant('duedate', $mform->_defaultValues['duedate']);
+        }
+    }
+
     /**
      * Validate the submitted form data.
      *
@@ -323,8 +352,8 @@ class assign_override_form extends moodleform {
         }
 
         if (!empty($data['allowsubmissionsfromdate']) && !empty($data['duedate'])) {
-            if ($data['duedate'] < $data['allowsubmissionsfromdate']) {
-                $errors['duedate'] = get_string('duedatevalidation', 'assign');
+            if ($data['duedate'] <= $data['allowsubmissionsfromdate']) {
+                $errors['duedate'] = get_string('duedateaftersubmissionvalidation', 'assign');
             }
         }
 
@@ -350,7 +379,7 @@ class assign_override_form extends moodleform {
         $changed = false;
         $keys = array('duedate', 'cutoffdate', 'allowsubmissionsfromdate', 'timelimit');
         foreach ($keys as $key) {
-            if ($data[$key] != $assigninstance->{$key}) {
+            if (isset($data[$key]) && $data[$key] != $assigninstance->{$key}) {
                 $changed = true;
                 break;
             }
