@@ -24,6 +24,6 @@
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
 
-$string['pluginname'] = 'Latlong';
-$string['fieldtypelabel'] = 'Latitude/longitude field';
-$string['privacy:metadata'] = 'The Latitude/longitude field component does not store any personal data, it uses tables defined in mod_data.';
+$string['pluginname'] = 'Coordinates';
+$string['fieldtypelabel'] = 'Coordinates';
+$string['privacy:metadata'] = 'The Coordinates field component doesn\'t store any personal data; it uses tables defined in mod_data.';
