@@ -24,6 +24,29 @@
 
 class data_field_url extends data_field_base {
     var $type = 'url';
+    /**
+     * priority for globalsearch indexing
+     *
+     * @var int
+     */
+    protected static $priority = self::MIN_PRIORITY;
+
+    public function supports_preview(): bool {
+        return true;
+    }
+
+    public function get_data_content_preview(int $recordid): stdClass {
+        return (object)[
+            'id' => 0,
+            'fieldid' => $this->field->id,
+            'recordid' => $recordid,
+            'content' => 'https://example.com',
+            'content1' => null,
+            'content2' => null,
+            'content3' => null,
+            'content4' => null,
+        ];
+    }
 
     function display_add_field($recordid = 0, $formdata = null) {
         global $CFG, $DB, $OUTPUT, $PAGE;
@@ -56,29 +79,46 @@ class data_field_url extends data_field_base {
                 $text = $content->content1;
             }
         }
-        $str = '<div title="' . s($this->field->description) . '">';
+
+        $autolinkable = !empty($this->field->param1) && empty($this->field->param2);
+
+        $str = '<div title="' . s($this->field->description) . '" class="d-flex flex-wrap align-items-center">';
 
         $label = '<label for="' . $fieldid . '"><span class="accesshide">' . $this->field->name . '</span>';
         if ($this->field->required) {
-            $label .= html_writer::img($OUTPUT->pix_url('req'), get_string('requiredelement', 'form'),
-                                      array('class' => 'req', 'title' => get_string('requiredelement', 'form')));
+            $image = $OUTPUT->pix_icon('req', get_string('requiredelement', 'form'));
+            if ($autolinkable) {
+                $label .= html_writer::div(get_string('requiredelement', 'form'), 'accesshide');
+            } else {
+                $label .= html_writer::div($image, 'inline-req');
+            }
         }
         $label .= '</label>';
 
-        if (!empty($this->field->param1) and empty($this->field->param2)) {
+        if ($autolinkable) {
             $str .= '<table><tr><td align="right">';
-            $str .= get_string('url','data').':</td><td>';
+            $str .= '<span class="mod-data-input">' . get_string('url', 'data') . ':</span>';
+            if (!empty($image)) {
+                $str .= $image;
+            }
+            $str .= '</td><td>';
             $str .= $label;
-            $str .= '<input type="text" name="field_'.$this->field->id.'_0" id="'.$fieldid.'" value="'.$url.'" size="60" />';
-            $str .= '<button id="filepicker-button-'.$options->client_id.'" style="display:none">'.$straddlink.'</button></td></tr>';
-            $str .= '<tr><td align="right">'.get_string('text','data').':</td><td><input type="text" name="field_'.$this->field->id.'_1" id="field_'.$this->field->id.'_1" value="'.s($text).'" size="60" /></td></tr>';
+            $str .= '<input type="text" name="field_' . $this->field->id . '_0" id="' . $fieldid . '" value="' . s($url) . '" ' .
+                    'size="40" class="form-control d-inline"/>';
+            $str .= '<button class="btn btn-secondary ms-1" id="filepicker-button-' . $options->client_id . '" ' .
+                    'style="display:none">' . $straddlink . '</button></td></tr>';
+            $str .= '<tr><td align="right"><span class="mod-data-input">' . get_string('text', 'data') . ':</span></td><td>';
+            $str .= '<input type="text" name="field_' . $this->field->id . '_1" id="field_' . $this->field->id . '_1" ' .
+                    'value="' . s($text) . '" size="40" class="form-control d-inline"/></td></tr>';
             $str .= '</table>';
         } else {
             // Just the URL field
             $str .= $label;
-            $str .= '<input type="text" name="field_'.$this->field->id.'_0" id="'.$fieldid.'" value="'.s($url).'" size="60" />';
+            $str .= '<input type="text" name="field_'.$this->field->id.'_0" id="'.$fieldid.'" value="'.s($url).'"';
+            $str .= ' size="40" class="mod-data-input form-control d-inline" />';
             if (count($options->repositories) > 0) {
-                $str .= '<button id="filepicker-button-'.$options->client_id.'" class="visibleifjs">'.$straddlink.'</button>';
+                $str .= '<button id="filepicker-button-' . $options->client_id . '" class="visibleifjs btn btn-secondary ms-1">' .
+                        $straddlink . '</button>';
             }
         }
 
@@ -92,12 +132,17 @@ class data_field_url extends data_field_base {
     }
 
     function display_search_field($value = '') {
-        return '<label class="accesshide" for="f_'.$this->field->id.'">' . get_string('fieldname', 'data') . '</label>' .
-               '<input type="text" size="16" id="f_'.$this->field->id.'" name="f_'.$this->field->id.'" value="'.$value.'" />';
+        return '<label class="accesshide" for="f_' . $this->field->id . '">' . get_string('fieldname', 'data') . '</label>' .
+               '<input type="text" size="16" id="f_' . $this->field->id . '" '.
+               ' name="f_' . $this->field->id . '" value="' . s($value) . '" class="form-control d-inline"/>';
     }
 
-    function parse_search_field() {
-        return optional_param('f_'.$this->field->id, '', PARAM_NOTAGS);
+    public function parse_search_field($defaults = null) {
+        $param = 'f_'.$this->field->id;
+        if (empty($defaults[$param])) {
+            $defaults = array($param => '');
+        }
+        return optional_param($param, $defaults[$param], PARAM_NOTAGS);
     }
 
     function generate_sql($tablealias, $value) {
@@ -110,37 +155,47 @@ class data_field_url extends data_field_base {
     }
 
     function display_browse_field($recordid, $template) {
-        global $DB;
 
-        if ($content = $DB->get_record('data_content', array('fieldid'=>$this->field->id, 'recordid'=>$recordid))) {
-            $url = empty($content->content)? '':$content->content;
-            $text = empty($content->content1)? '':$content->content1;
-            if (empty($url) or ($url == 'http://')) {
-                return '';
+        $content = $this->get_data_content($recordid);
+        if (!$content) {
+            return '';
+        }
+
+        $url = empty($content->content) ? '' : $content->content;
+        $text = empty($content->content1) ? '' : $content->content1;
+        if (empty($url) || ($url == 'http://')) {
+            return '';
+        }
+        if (!empty($this->field->param2)) {
+            // Param2 forces the text to something.
+            $text = $this->field->param2;
+        }
+        if ($this->field->param1) {
+            // Param1 defines whether we want to autolink the url.
+            $attributes = ['class' => 'data-field-link'];
+            if ($this->field->param3) {
+                // Param3 defines whether this URL should open in a new window.
+                $attributes['target'] = '_blank';
+                $attributes['rel'] = 'noreferrer';
             }
-            if (!empty($this->field->param2)) {
-                // param2 forces the text to something
-                $text = $this->field->param2;
+
+            if (empty($text)) {
+                $text = $url;
             }
-            if ($this->field->param1) {
-                // param1 defines whether we want to autolink the url.
-                $attributes = array();
-                if ($this->field->param3) {
-                    // param3 defines whether this URL should open in a new window.
-                    $attributes['target'] = '_blank';
-                }
 
-                if (empty($text)) {
-                    $text = $url;
-                }
+            $str = html_writer::link($url, $text, $attributes);
+        } else {
+            $str = $url;
+        }
+        return $str;
+    }
 
-                $str = html_writer::link($url, $text, $attributes);
-            } else {
-                $str = $url;
-            }
-            return $str;
+    function update_content_import($recordid, $value, $name='') {
+        $values = explode(" ", $value, 2);
+
+        foreach ($values as $index => $value) {
+            $this->update_content($recordid, $value, $name . '_' . $index);
         }
-        return false;
     }
 
     function update_content($recordid, $value, $name='') {
@@ -191,4 +246,18 @@ class data_field_url extends data_field_base {
         return $record->content . " " . $record->content1;
     }
 
+    /**
+     * Return the plugin configs for external functions.
+     *
+     * @return array the list of config parameters
+     * @since Moodle 3.3
+     */
+    public function get_config_for_external() {
+        // Return all the config parameters.
+        $configs = [];
+        for ($i = 1; $i <= 10; $i++) {
+            $configs["param$i"] = $this->field->{"param$i"};
+        }
+        return $configs;
+    }
 }
