@@ -36,6 +36,7 @@ use mod_forum\subscriptions;
 use context;
 use context_system;
 use stdClass;
+use moodle_exception;
 
 require_once($CFG->dirroot . '/mod/forum/lib.php');
 
@@ -60,6 +61,8 @@ class capability {
     private $forumrecord;
     /** @var context $context Module context for the forum */
     private $context;
+    /** @var array $canviewpostcache Cache of discussion posts that can be viewed by a user. */
+    protected $canviewpostcache = [];
 
     /**
      * Constructor.
@@ -89,7 +92,7 @@ class capability {
      * @param stdClass $user The user to check
      * @return bool
      */
-    public function can_subscribe_to_forum(stdClass $user) : bool {
+    public function can_subscribe_to_forum(stdClass $user): bool {
         if ($this->forum->get_type() == 'single') {
             return false;
         }
@@ -105,7 +108,7 @@ class capability {
      * @param int|null $groupid The current activity group id
      * @return bool
      */
-    public function can_create_discussions(stdClass $user, int $groupid = null) : bool {
+    public function can_create_discussions(stdClass $user, ?int $groupid = null): bool {
         if (isguestuser($user) or !isloggedin()) {
             return false;
         }
@@ -116,6 +119,12 @@ class capability {
             }
         }
 
+        // If the user reaches the number of posts equal to warning/blocking setting then return the value of canpost in $warningobj.
+        $cmrecord = $this->forum->get_course_module_record();
+        if ($warningobj = forum_check_throttling($this->forumrecord, $cmrecord)) {
+            return $warningobj->canpost;
+        }
+
         switch ($this->forum->get_type()) {
             case 'news':
                 $capability = 'mod/forum:addnews';
@@ -150,7 +159,7 @@ class capability {
      * @param stdClass $user The user to check
      * @return bool
      */
-    public function can_access_all_groups(stdClass $user) : bool {
+    public function can_access_all_groups(stdClass $user): bool {
         return has_capability('moodle/site:accessallgroups', $this->get_context(), $user);
     }
 
@@ -161,7 +170,7 @@ class capability {
      * @param int $groupid The id of the group that the forum is set to
      * @return bool
      */
-    public function can_access_group(stdClass $user, int $groupid) : bool {
+    public function can_access_group(stdClass $user, int $groupid): bool {
         if ($this->can_access_all_groups($user)) {
             // This user has access to all groups.
             return true;
@@ -179,7 +188,7 @@ class capability {
      * @param stdClass $user The user to check
      * @return bool
      */
-    public function can_post_to_my_groups(stdClass $user) : bool {
+    public function can_post_to_my_groups(stdClass $user): bool {
         return has_capability('mod/forum:canposttomygroups', $this->get_context(), $user);
     }
 
@@ -189,7 +198,7 @@ class capability {
      * @param stdClass $user The user to check
      * @return bool
      */
-    public function can_view_discussions(stdClass $user) : bool {
+    public function can_view_discussions(stdClass $user): bool {
         return has_capability('mod/forum:viewdiscussion', $this->get_context(), $user);
     }
 
@@ -199,7 +208,7 @@ class capability {
      * @param stdClass $user The user to check
      * @return bool
      */
-    public function can_move_discussions(stdClass $user) : bool {
+    public function can_move_discussions(stdClass $user): bool {
         $forum = $this->get_forum();
         return $forum->get_type() !== 'single' &&
                 has_capability('mod/forum:movediscussions', $this->get_context(), $user);
@@ -211,7 +220,7 @@ class capability {
      * @param stdClass $user The user to check
      * @return bool
      */
-    public function can_pin_discussions(stdClass $user) : bool {
+    public function can_pin_discussions(stdClass $user): bool {
         return $this->forum->get_type() !== 'single' &&
                 has_capability('mod/forum:pindiscussions', $this->get_context(), $user);
     }
@@ -222,7 +231,7 @@ class capability {
      * @param stdClass $user The user to check
      * @return bool
      */
-    public function can_split_discussions(stdClass $user) : bool {
+    public function can_split_discussions(stdClass $user): bool {
         $forum = $this->get_forum();
         return $forum->get_type() !== 'single' && has_capability('mod/forum:splitdiscussions', $this->get_context(), $user);
     }
@@ -233,7 +242,7 @@ class capability {
      * @param stdClass $user The user to check
      * @return bool
      */
-    public function can_export_discussions(stdClass $user) : bool {
+    public function can_export_discussions(stdClass $user): bool {
         global $CFG;
         return $CFG->enableportfolios && has_capability('mod/forum:exportdiscussion', $this->get_context(), $user);
     }
@@ -244,7 +253,7 @@ class capability {
      * @param stdClass $user The user to check
      * @return bool
      */
-    public function can_manually_control_post_read_status(stdClass $user) : bool {
+    public function can_manually_control_post_read_status(stdClass $user): bool {
         global $CFG;
         return $CFG->forum_usermarksread && isloggedin() && forum_tp_is_tracked($this->get_forum_record(), $user);
     }
@@ -256,7 +265,7 @@ class capability {
      * @param discussion_entity $discussion The discussion to check
      * @return bool
      */
-    public function must_post_before_viewing_discussion(stdClass $user, discussion_entity $discussion) : bool {
+    public function must_post_before_viewing_discussion(stdClass $user, discussion_entity $discussion): bool {
         $forum = $this->get_forum();
 
         if ($forum->get_type() === 'qanda') {
@@ -277,7 +286,7 @@ class capability {
      * @param discussion_entity $discussion The discussion to check
      * @return bool
      */
-    public function can_subscribe_to_discussion(stdClass $user, discussion_entity $discussion) : bool {
+    public function can_subscribe_to_discussion(stdClass $user, discussion_entity $discussion): bool {
         return $this->can_subscribe_to_forum($user);
     }
 
@@ -288,7 +297,7 @@ class capability {
      * @param discussion_entity $discussion The discussion to check
      * @return bool
      */
-    public function can_move_discussion(stdClass $user, discussion_entity $discussion) : bool {
+    public function can_move_discussion(stdClass $user, discussion_entity $discussion): bool {
         return $this->can_move_discussions($user);
     }
 
@@ -299,7 +308,7 @@ class capability {
      * @param discussion_entity $discussion The discussion to check
      * @return bool
      */
-    public function can_pin_discussion(stdClass $user, discussion_entity $discussion) : bool {
+    public function can_pin_discussion(stdClass $user, discussion_entity $discussion): bool {
         return $this->can_pin_discussions($user);
     }
 
@@ -310,15 +319,22 @@ class capability {
      * @param discussion_entity $discussion The discussion to check
      * @return bool
      */
-    public function can_post_in_discussion(stdClass $user, discussion_entity $discussion) : bool {
+    public function can_post_in_discussion(stdClass $user, discussion_entity $discussion): bool {
         $forum = $this->get_forum();
         $forumrecord = $this->get_forum_record();
+
         $discussionrecord = $this->get_discussion_record($discussion);
         $context = $this->get_context();
         $coursemodule = $forum->get_course_module_record();
         $course = $forum->get_course_record();
 
-        return forum_user_can_post($forumrecord, $discussionrecord, $user, $coursemodule, $course, $context);
+        $status = forum_user_can_post($forumrecord, $discussionrecord, $user, $coursemodule, $course, $context);
+
+        // If the user reaches the number of posts equal to warning/blocking setting then logically and canpost value with $status.
+        if ($warningobj = forum_check_throttling($forumrecord, $coursemodule)) {
+            return $status && $warningobj->canpost;
+        }
+        return $status;
     }
 
     /**
@@ -327,7 +343,7 @@ class capability {
      * @param stdClass $user The user to check
      * @return bool
      */
-    public function can_favourite_discussion(stdClass $user) : bool {
+    public function can_favourite_discussion(stdClass $user): bool {
         $context = $this->get_context();
         return has_capability('mod/forum:cantogglefavourite', $context, $user);
     }
@@ -339,7 +355,7 @@ class capability {
      * @param discussion_entity $discussion The discussion to check
      * @return bool
      */
-    public function can_view_discussion(stdClass $user, discussion_entity $discussion) : bool {
+    public function can_view_discussion(stdClass $user, discussion_entity $discussion): bool {
         $forumrecord = $this->get_forum_record();
         $discussionrecord = $this->get_discussion_record($discussion);
         $context = $this->get_context();
@@ -355,17 +371,28 @@ class capability {
      * @param post_entity $post The post the user wants to view
      * @return bool
      */
-    public function can_view_post(stdClass $user, discussion_entity $discussion, post_entity $post) : bool {
+    public function can_view_post(stdClass $user, discussion_entity $discussion, post_entity $post): bool {
         if (!$this->can_view_post_shell($user, $post)) {
             return false;
         }
 
+        // Return cached can view if possible.
+        if (isset($this->canviewpostcache[$user->id][$post->get_id()])) {
+            return $this->canviewpostcache[$user->id][$post->get_id()];
+        }
+
+        // Otherwise, check if the user can see this post.
         $forum = $this->get_forum();
         $forumrecord = $this->get_forum_record();
         $discussionrecord = $this->get_discussion_record($discussion);
         $postrecord = $this->get_post_record($post);
         $coursemodule = $forum->get_course_module_record();
-        return forum_user_can_see_post($forumrecord, $discussionrecord, $postrecord, $user, $coursemodule, false);
+        $canviewpost = forum_user_can_see_post($forumrecord, $discussionrecord, $postrecord, $user, $coursemodule, false);
+
+        // Then cache the result before returning.
+        $this->canviewpostcache[$user->id][$post->get_id()] = $canviewpost;
+
+        return $canviewpost;
     }
 
     /**
@@ -377,7 +404,11 @@ class capability {
      * @return  bool
      *
      */
-    public function can_view_post_shell(stdClass $user, post_entity $post) : bool {
+    public function can_view_post_shell(stdClass $user, post_entity $post): bool {
+        if ($post->is_owned_by_user($user)) {
+            return true;
+        }
+
         if (!$post->is_private_reply()) {
             return true;
         }
@@ -395,7 +426,7 @@ class capability {
      * @param   stdClass $user The user to check
      * @return  bool
      */
-    public function can_view_any_private_reply(stdClass $user) : bool {
+    public function can_view_any_private_reply(stdClass $user): bool {
         return has_capability('mod/forum:readprivatereplies', $this->get_context(), $user);
     }
 
@@ -407,12 +438,13 @@ class capability {
      * @param post_entity $post The post the user wants to edit
      * @return bool
      */
-    public function can_edit_post(stdClass $user, discussion_entity $discussion, post_entity $post) : bool {
+    public function can_edit_post(stdClass $user, discussion_entity $discussion, post_entity $post): bool {
         global $CFG;
 
         $context = $this->get_context();
         $ownpost = $post->is_owned_by_user($user);
         $ineditingtime = $post->get_age() < $CFG->maxeditingtime;
+        $mailnow = $post->should_mail_now();
 
         switch ($this->forum->get_type()) {
             case 'news':
@@ -426,32 +458,68 @@ class capability {
                 break;
         }
 
-        return ($ownpost && $ineditingtime) || has_capability('mod/forum:editanypost', $context, $user);
+        return ($ownpost && $ineditingtime && !$mailnow) || has_capability('mod/forum:editanypost', $context, $user);
     }
 
+    /**
+     * Verifies is the given user can delete a post.
+     *
+     * @param stdClass $user The user to check
+     * @param discussion_entity $discussion The discussion to check
+     * @param post_entity $post The post the user wants to delete
+     * @param bool $hasreplies Whether the post has replies
+     * @return bool
+     * @throws moodle_exception
+     */
+    public function validate_delete_post(stdClass $user, discussion_entity $discussion, post_entity $post,
+            bool $hasreplies = false): void {
+        global $CFG;
+
+        $forum = $this->get_forum();
+
+        if ($forum->get_type() == 'single' && $discussion->is_first_post($post)) {
+            // Do not allow deleting of first post in single simple type.
+            throw new moodle_exception('cannotdeletepost', 'forum');
+        }
+
+        $context = $this->get_context();
+        $ownpost = $post->is_owned_by_user($user);
+        $ineditingtime = $post->get_age() < $CFG->maxeditingtime;
+        $mailnow = $post->should_mail_now();
+
+        if (!($ownpost && $ineditingtime && has_capability('mod/forum:deleteownpost', $context, $user) && !$mailnow ||
+                has_capability('mod/forum:deleteanypost', $context, $user))) {
+
+            throw new moodle_exception('cannotdeletepost', 'forum');
+        }
+
+        if ($post->get_total_score()) {
+            throw new moodle_exception('couldnotdeleteratings', 'rating');
+        }
+
+        if ($hasreplies && !has_capability('mod/forum:deleteanypost', $context, $user)) {
+            throw new moodle_exception('couldnotdeletereplies', 'forum');
+        }
+    }
+
+
     /**
      * Can the user delete the post in this discussion?
      *
      * @param stdClass $user The user to check
      * @param discussion_entity $discussion The discussion to check
      * @param post_entity $post The post the user wants to delete
+     * @param bool $hasreplies Whether the post has replies
      * @return bool
      */
-    public function can_delete_post(stdClass $user, discussion_entity $discussion, post_entity $post) : bool {
-        global $CFG;
+    public function can_delete_post(stdClass $user, discussion_entity $discussion, post_entity $post,
+            bool $hasreplies = false): bool {
 
-        $forum = $this->get_forum();
-
-        if ($forum->get_type() == 'single' && $discussion->is_first_post($post)) {
-            // Do not allow deleting of first post in single simple type.
+        try {
+            $this->validate_delete_post($user, $discussion, $post, $hasreplies);
+            return true;
+        } catch (moodle_exception $e) {
             return false;
-        } else {
-            $context = $this->get_context();
-            $ownpost = $post->is_owned_by_user($user);
-            $ineditingtime = $post->get_age() < $CFG->maxeditingtime;
-
-            return ($ownpost && $ineditingtime && has_capability('mod/forum:deleteownpost', $context, $user)) ||
-                has_capability('mod/forum:deleteanypost', $context, $user);
         }
     }
 
@@ -463,7 +531,7 @@ class capability {
      * @param post_entity $post The post the user wants to split
      * @return bool
      */
-    public function can_split_post(stdClass $user, discussion_entity $discussion, post_entity $post) : bool {
+    public function can_split_post(stdClass $user, discussion_entity $discussion, post_entity $post): bool {
         if ($post->is_private_reply()) {
             // It is not possible to create a private discussion.
             return false;
@@ -480,10 +548,13 @@ class capability {
      * @param post_entity $post The post the user wants to reply to
      * @return bool
      */
-    public function can_reply_to_post(stdClass $user, discussion_entity $discussion, post_entity $post) : bool {
+    public function can_reply_to_post(stdClass $user, discussion_entity $discussion, post_entity $post): bool {
         if ($post->is_private_reply()) {
             // It is not possible to reply to a private reply.
             return false;
+        } else if (!$this->can_view_post($user, $discussion, $post)) {
+            // If the user cannot view the post in the first place, the user should not be able to reply to the post.
+            return false;
         }
 
         return $this->can_post_in_discussion($user, $discussion);
@@ -496,7 +567,7 @@ class capability {
      * @param post_entity $post The post the user wants to reply to
      * @return bool
      */
-    public function can_reply_privately_to_post(stdClass $user, post_entity $post) : bool {
+    public function can_reply_privately_to_post(stdClass $user, post_entity $post): bool {
         if ($post->is_private_reply()) {
             // You cannot reply privately to a post which is, itself, a private reply.
             return false;
@@ -512,7 +583,7 @@ class capability {
      * @param post_entity $post The post the user wants to export
      * @return bool
      */
-    public function can_export_post(stdClass $user, post_entity $post) : bool {
+    public function can_export_post(stdClass $user, post_entity $post): bool {
         global $CFG;
         $context = $this->get_context();
         return $CFG->enableportfolios  && (has_capability('mod/forum:exportpost', $context, $user) ||
@@ -524,7 +595,7 @@ class capability {
      *
      * @return forum_entity
      */
-    protected function get_forum() : forum_entity {
+    protected function get_forum(): forum_entity {
         return $this->forum;
     }
 
@@ -533,7 +604,7 @@ class capability {
      *
      * @return stdClass
      */
-    protected function get_forum_record() : stdClass {
+    protected function get_forum_record(): stdClass {
         return $this->forumrecord;
     }
 
@@ -542,7 +613,7 @@ class capability {
      *
      * @return context
      */
-    protected function get_context() : context {
+    protected function get_context(): context {
         return $this->context;
     }
 
@@ -552,7 +623,7 @@ class capability {
      * @param discussion_entity $discussion The discussion to convert
      * @return stdClass
      */
-    protected function get_discussion_record(discussion_entity $discussion) : stdClass {
+    protected function get_discussion_record(discussion_entity $discussion): stdClass {
         return $this->discussiondatamapper->to_legacy_object($discussion);
     }
 
@@ -562,7 +633,7 @@ class capability {
      * @param post_entity $post The post to convert
      * @return stdClass
      */
-    protected function get_post_record(post_entity $post) : stdClass {
+    protected function get_post_record(post_entity $post): stdClass {
         return $this->postdatamapper->to_legacy_object($post);
     }
 
@@ -573,7 +644,7 @@ class capability {
      * @param discussion_entity $discussion The discussion to check
      * @return bool
      */
-    public function can_view_participants(stdClass $user, discussion_entity $discussion) : bool {
+    public function can_view_participants(stdClass $user, discussion_entity $discussion): bool {
         return course_can_view_participants($this->get_context()) &&
             !$this->must_post_before_viewing_discussion($user, $discussion);
     }
@@ -584,7 +655,7 @@ class capability {
      * @param stdClass $user The user to check
      * @return bool
      */
-    public function can_view_hidden_posts(stdClass $user) : bool {
+    public function can_view_hidden_posts(stdClass $user): bool {
         return has_capability('mod/forum:viewhiddentimedposts', $this->get_context(), $user);
     }
 
@@ -604,7 +675,58 @@ class capability {
      * @param stdClass $user The user to check
      * @return bool
      */
-    public function can_manage_tags(stdClass $user) : bool {
+    public function can_manage_tags(stdClass $user): bool {
         return has_capability('moodle/tag:manage', context_system::instance(), $user);
     }
+
+    /**
+     * Checks whether the user can self enrol into the course.
+     * Mimics the checks on the add button in deprecatedlib/forum_print_latest_discussions
+     *
+     * @param stdClass $user
+     * @return bool
+     */
+    public function can_self_enrol(stdClass $user): bool {
+        $canstart = false;
+
+        if ($this->forum->get_type() != 'news') {
+            if (isguestuser($user) or !isloggedin()) {
+                $canstart = true;
+            }
+
+            if (!is_enrolled($this->context) and !is_viewing($this->context)) {
+                 // Allow guests and not-logged-in to see the button - they are prompted to log in after clicking the link,
+                 // Normal users with temporary guest access see this button too, they are asked to enrol instead,
+                 // Do not show the button to users with suspended enrolments here.
+                $canstart = enrol_selfenrol_available($this->forum->get_course_id());
+            }
+        }
+
+        return $canstart;
+    }
+
+    /**
+     * Checks whether the user can export the whole forum (discussions and posts).
+     *
+     * @param stdClass $user The user object.
+     * @return bool True if the user can export the forum or false otherwise.
+     */
+    public function can_export_forum(stdClass $user): bool {
+        return has_capability('mod/forum:exportforum', $this->get_context(), $user);
+    }
+
+    /**
+     * Check whether the supplied grader can grade the gradee.
+     *
+     * @param stdClass $grader The user grading
+     * @param stdClass $gradee The user being graded
+     * @return bool
+     */
+    public function can_grade(stdClass $grader, ?stdClass $gradee = null): bool {
+        if (!has_capability('mod/forum:grade', $this->get_context(), $grader)) {
+            return false;
+        }
+
+        return true;
+    }
 }
