@@ -207,10 +207,8 @@ class qtype_numerical_question extends question_graded_automatically {
         }
         foreach ($this->answers as $answer) {
             if ($answer->within_tolerance($scaledvalue)) {
-                $answer->unitisright = !is_null($multiplier);
                 return $answer;
             } else if ($answer->within_tolerance($value)) {
-                $answer->unitisright = false;
                 return $answer;
             }
         }
@@ -218,6 +216,23 @@ class qtype_numerical_question extends question_graded_automatically {
         return null;
     }
 
+    /**
+     * Checks if the provided $multiplier is appropriate for the unit of the given $value,
+     * ensuring that multiplying $value by the $multiplier yields the expected $answer.
+     *
+     * @param qtype_numerical_answer $answer The expected result when multiplying $value by the appropriate $multiplier.
+     * @param float $value The provided value
+     * @param float|null $multiplier The multiplier value for the unit of $value.
+     * @return bool Returns true if the $multiplier is correct for the unit of $value, false otherwise.
+     */
+    public function is_unit_right(qtype_numerical_answer $answer, float $value, ?float $multiplier): bool {
+        if (is_null($multiplier)) {
+            return false;
+        }
+
+        return $answer->within_tolerance($multiplier * $value);
+    }
+
     public function get_correct_answer() {
         foreach ($this->answers as $answer) {
             $state = question_state::graded_state_for_fraction($answer->fraction);
@@ -256,12 +271,14 @@ class qtype_numerical_question extends question_graded_automatically {
         list($value, $unit, $multiplier) = $this->ap->apply_units(
                 $response['answer'], $selectedunit);
 
+        /** @var qtype_numerical_answer $answer */
         $answer = $this->get_matching_answer($value, $multiplier);
         if (!$answer) {
             return array(0, question_state::$gradedwrong);
         }
 
-        $fraction = $this->apply_unit_penalty($answer->fraction, $answer->unitisright);
+        $unitisright = $this->is_unit_right($answer, $value, $multiplier);
+        $fraction = $this->apply_unit_penalty($answer->fraction, $unitisright);
         return array($fraction, question_state::graded_state_for_fraction($fraction));
     }
 
@@ -276,6 +293,7 @@ class qtype_numerical_question extends question_graded_automatically {
             $selectedunit = null;
         }
         list($value, $unit, $multiplier) = $this->ap->apply_units($response['answer'], $selectedunit);
+        /** @var qtype_numerical_answer $ans */
         $ans = $this->get_matching_answer($value, $multiplier);
 
         $resp = $response['answer'];
@@ -291,9 +309,10 @@ class qtype_numerical_question extends question_graded_automatically {
             return array($this->id => new question_classified_response(0, $resp, 0));
         }
 
-        return array($this->id => new question_classified_response($ans->id,
-                $resp,
-                $this->apply_unit_penalty($ans->fraction, $ans->unitisright)));
+        $unitisright = $this->is_unit_right($ans, $value, $multiplier);
+        return [
+            $this->id => new question_classified_response($ans->id, $resp, $this->apply_unit_penalty($ans->fraction, $unitisright))
+        ];
     }
 
     public function check_file_access($qa, $options, $component, $filearea, $args,
