@@ -25,9 +25,9 @@
 defined('MOODLE_INTERNAL') || die();
 
 
-$plugin->version   = 2020061500; // The current plugin version (Date: YYYYMMDDXX).
-$plugin->requires  = 2020060900; // Requires this Moodle version.
+$plugin->version   = 2025041400; // The current plugin version (Date: YYYYMMDDXX).
+$plugin->requires  = 2025040800; // Requires this Moodle version.
 $plugin->component = 'report_competency'; // Full name of the plugin (used for diagnostics).
-$plugin->dependencies = array(
-    'tool_lp' => ANY_VERSION
-);
+$plugin->dependencies = [
+    'tool_lp' => ANY_VERSION,
+];
