@@ -27,6 +27,9 @@ $string['configplugin'] = 'Configuration for recent files repository';
 $string['recentfilesnumber'] = 'Number of recent files';
 $string['emptyfilelist'] = 'There are no files to show';
 $string['notitle'] = 'notitle';
-$string['recent:view'] = 'View recent files repsitory plugin';
+$string['recent:view'] = 'View recent files repository plugin';
 $string['pluginname_help'] = 'Files recently used by current user';
 $string['pluginname'] = 'Recent files';
+$string['privacy:metadata'] = 'The Recent files repository plugin does not store or transmit any personal data.';
+$string['timelimit'] = 'Time limit';
+$string['timelimit_help'] = 'Only retrieve recent files within the time limit';
