@@ -16,6 +16,7 @@
 
 namespace tool_mfa\output;
 
+use core\context\system;
 use tool_mfa\local\factor\object_factor;
 use tool_mfa\local\form\login_form;
 use \html_writer;
@@ -41,23 +42,24 @@ class renderer extends \plugin_renderer_base {
 
         switch ($state) {
             case factor::STATE_PASS:
-                return html_writer::tag('span', get_string('state:pass', 'tool_mfa'), ['class' => 'badge badge-success']);
+                return html_writer::tag('span', get_string('state:pass', 'tool_mfa'), ['class' => 'badge bg-success text-white']);
 
             case factor::STATE_FAIL:
-                return html_writer::tag('span', get_string('state:fail', 'tool_mfa'), ['class' => 'badge badge-danger']);
+                return html_writer::tag('span', get_string('state:fail', 'tool_mfa'), ['class' => 'badge bg-danger text-white']);
 
             case factor::STATE_NEUTRAL:
-                return html_writer::tag('span', get_string('state:neutral', 'tool_mfa'), ['class' => 'badge badge-warning']);
+                return html_writer::tag('span', get_string('state:neutral', 'tool_mfa'),
+                    ['class' => 'badge bg-warning text-dark']);
 
             case factor::STATE_UNKNOWN:
                 return html_writer::tag('span', get_string('state:unknown', 'tool_mfa'),
-                        ['class' => 'badge badge-secondary']);
+                    ['class' => 'badge bg-secondary text-dark']);
 
             case factor::STATE_LOCKED:
-                return html_writer::tag('span', get_string('state:locked', 'tool_mfa'), ['class' => 'badge badge-error']);
+                return html_writer::tag('span', get_string('state:locked', 'tool_mfa'), ['class' => 'badge bg-danger text-white']);
 
             default:
-                return html_writer::tag('span', get_string('pending', 'tool_mfa'), ['class' => 'badge badge-secondary']);
+                return html_writer::tag('span', get_string('pending', 'tool_mfa'), ['class' => 'badge bg-secondary text-dark']);
         }
     }
 
@@ -67,93 +69,138 @@ class renderer extends \plugin_renderer_base {
      * @return string
      */
     public function available_factors(): string {
-        $html = $this->output->heading(get_string('preferences:availablefactors', 'tool_mfa'), 2);
-
+        global $USER;
         $factors = factor::get_enabled_factors();
+        $data = [];
+
         foreach ($factors as $factor) {
-            // TODO is_configured / is_ready.
-            if (!$factor->has_setup() || !$factor->show_setup_buttons()) {
+
+            // Allow all factors with setup and button.
+            // Make an exception for email factor as this is currently set up by admins only and required on this list.
+            if ((!$factor->has_setup() || !$factor->show_setup_buttons()) && !$factor instanceof \factor_email\factor) {
                 continue;
             }
-            $html .= $this->setup_factor($factor);
+
+            $userfactors = $factor->get_active_user_factors($USER);
+            $active = !empty($userfactors) ?? false;
+            $button = null;
+            $icon = $factor->get_icon();
+            $params = [
+                'action' => 'setup',
+                'factor' => $factor->name,
+            ];
+
+            if (!$active) {
+                // Not active yet and requires set up.
+                $info = $factor->get_info();
+
+                if ($factor->show_setup_buttons()) {
+                    $params['action'] = 'setup';
+                    $button = new \single_button(
+                        url: new \moodle_url('action.php', $params),
+                        label: $factor->get_setup_string(),
+                        method: 'post',
+                        type: \single_button::BUTTON_PRIMARY,
+                        attributes: [
+                            'aria-label' => get_string('setupfactor', 'factor_' . $factor->name),
+                        ],
+                    );
+                    $button = $button->export_for_template($this->output);
+                }
+
+            } else {
+                // Active and can be managed.
+                $factorid = reset($userfactors)->id;
+                $info = $factor->get_manage_info($factorid);
+
+                if ($factor->show_setup_buttons()) {
+                    $params['action'] = 'manage';
+                    $button = new \single_button(
+                        url: new \moodle_url('action.php', $params),
+                        label: $factor->get_manage_string(),
+                        method: 'post',
+                        type: \single_button::BUTTON_PRIMARY,
+                        attributes: [
+                            'aria-label' => get_string('managefactor', 'factor_' . $factor->name),
+                        ],
+                    );
+                    $button = $button->export_for_template($this->output);
+                }
+            }
+
+            // Prepare data for template.
+            $data['factors'][] = [
+                'active' => $active,
+                'label' => $factor->get_display_name(),
+                'name' => $factor->name,
+                'info' => $info,
+                'icon' => $icon,
+                'button' => $button,
+            ];
         }
 
-        return $html;
+        return $this->render_from_template('tool_mfa/mfa_selector', $data);
     }
 
     /**
-     * Returns the html section for factor setup
-     *
-     * @param object $factor object of the factor class
-     * @return string
+     * @deprecated since Moodle 4.4
      */
-    public function setup_factor(object $factor): string {
-        $html = '';
-
-        $html .= html_writer::start_tag('div', ['class' => 'card']);
-
-        $html .= html_writer::tag('h4', $factor->get_display_name(), ['class' => 'card-header']);
-        $html .= html_writer::start_tag('div', ['class' => 'card-body']);
-        $html .= $factor->get_info();
-
-        $setupparams = ['action' => 'setup', 'factor' => $factor->name, 'sesskey' => sesskey()];
-        $setupurl = new \moodle_url('action.php', $setupparams);
-        $html .= $this->output->single_button($setupurl, $factor->get_setup_string());
-        $html .= html_writer::end_tag('div');
-        $html .= html_writer::end_tag('div');
-        $html .= '<br>';
-
-        return $html;
+    #[\core\attribute\deprecated(null, reason: 'It is no longer used', since: '4.4', mdl: 'MDL-79920', final: true)]
+    public function setup_factor(): void {
+        \core\deprecation::emit_deprecation_if_present([self::class, __FUNCTION__]);
     }
 
     /**
-     * Defines section with active user's factors.
+     * Show a table displaying a users active factors.
      *
+     * @param string|null $filterfactor The factor name to filter on.
      * @return string $html
      * @throws \coding_exception
      */
-    public function active_factors(): string {
+    public function active_factors(?string $filterfactor = null): string {
         global $USER, $CFG;
 
         require_once($CFG->dirroot . '/iplookup/lib.php');
 
-        $html = $this->output->heading(get_string('preferences:activefactors', 'tool_mfa'), 2);
+        $html = '';
 
         $headers = get_strings([
-            'factor',
             'devicename',
-            'created',
-            'createdfromip',
-            'lastverified',
-            'revoke',
+            'added',
+            'lastused',
+            'replace',
+            'remove',
         ], 'tool_mfa');
 
         $table = new \html_table();
         $table->id = 'active_factors';
         $table->attributes['class'] = 'generaltable table table-bordered';
         $table->head  = [
-            $headers->factor,
             $headers->devicename,
-            $headers->created,
-            $headers->createdfromip,
-            $headers->lastverified,
-            $headers->revoke,
+            $headers->added,
+            $headers->lastused,
+            $headers->replace,
+            $headers->remove,
         ];
         $table->colclasses = [
-            'leftalign',
-            'leftalign',
-            'centeralign',
-            'centeralign',
-            'centeralign',
-            'centeralign',
-            'centeralign',
-            'centeralign',
+            'text-start',
+            'text-start',
+            'text-start',
+            'text-center',
+            'text-center',
         ];
         $table->data  = [];
 
         $factors = factor::get_enabled_factors();
+        $hasmorethanone = factor::user_has_more_than_one_active_factors();
 
         foreach ($factors as $factor) {
+
+            // Filter results to match the specified factor.
+            if (!empty($filterfactor) && $factor->name !== $filterfactor) {
+                continue;
+            }
+
             $userfactors = $factor->get_active_user_factors($USER);
 
             if (!$factor->has_setup()) {
@@ -161,15 +208,39 @@ class renderer extends \plugin_renderer_base {
             }
 
             foreach ($userfactors as $userfactor) {
-                if ($factor->has_revoke()) {
-                    $revokeparams = [
-                        'action' => 'revoke', 'factor' => $factor->name,
-                        'factorid' => $userfactor->id, 'sesskey' => sesskey(),
+
+                // Revoke option.
+                if ($factor->has_revoke() && $hasmorethanone) {
+                    $content = $headers->remove;
+                    $attributes = [
+                        'data-action' => 'revoke',
+                        'data-factor' => $factor->name,
+                        'data-factorid' => $userfactor->id,
+                        'data-factorname' => $factor->get_display_name(),
+                        'data-devicename' => $userfactor->label,
+                        'aria-label' => get_string('revokefactor', 'tool_mfa'),
+                        'class' => 'btn btn-primary mfa-action-button',
                     ];
-                    $revokeurl = new \moodle_url('action.php', $revokeparams);
-                    $revokelink = \html_writer::link($revokeurl, $headers->revoke);
+                    $revokebutton = \html_writer::tag('button', $content, $attributes);
                 } else {
-                    $revokelink = '';
+                    $revokebutton = get_string('statusna');
+                }
+
+                // Replace option.
+                if ($factor->has_replace()) {
+                    $content = $headers->replace;
+                    $attributes = [
+                        'data-action' => 'replace',
+                        'data-factor' => $factor->name,
+                        'data-factorid' => $userfactor->id,
+                        'data-factorname' => $factor->get_display_name(),
+                        'data-devicename' => $userfactor->label,
+                        'aria-label' => get_string('replacefactor', 'tool_mfa'),
+                        'class' => 'btn btn-primary mfa-action-button',
+                    ];
+                    $replacebutton = \html_writer::tag('button', $content, $attributes);
+                } else {
+                    $replacebutton = get_string('statusna');
                 }
 
                 $timecreated  = $userfactor->timecreated == '-' ? '-'
@@ -183,17 +254,12 @@ class renderer extends \plugin_renderer_base {
                     $lastverified .= get_string('ago', 'core_message', format_time(time() - $userfactor->lastverified));
                 }
 
-                $info = iplookup_find_location($userfactor->createdfromip);
-                $ip = $userfactor->createdfromip;
-                $ip .= '<br>' . $info['country'] . ' - ' . $info['city'];
-
                 $row = new \html_table_row([
-                    $factor->get_display_name(),
                     $userfactor->label,
                     $timecreated,
-                    $ip,
                     $lastverified,
-                    $revokelink,
+                    $replacebutton,
+                    $revokebutton,
                 ]);
                 $table->data[] = $row;
             }
@@ -222,7 +288,8 @@ class renderer extends \plugin_renderer_base {
         // Support link.
         $supportemail = $CFG->supportemail;
         if (!empty($supportemail)) {
-            $subject = get_string('email:subject', 'tool_mfa', $SITE->fullname);
+            $subject = get_string('email:subject', 'tool_mfa',
+                format_string($SITE->fullname, true, ['context' => system::instance()]));
             $maillink = \html_writer::link("mailto:$supportemail?Subject=$subject", $supportemail);
             $notification .= get_string('error:support', 'tool_mfa');
             $notification .= \html_writer::tag('p', $maillink);
@@ -635,6 +702,7 @@ class renderer extends \plugin_renderer_base {
                 'hasadditionalfactors' => $hasadditionalfactors,
                 'additionalfactors' => $alladitionalfactors,
                 'authurl' => $authurl->out(),
+                'sesskey' => sesskey(),
                 'supportlink' => $this->get_support_link(),
                 'disablefactor' => $disablefactor
         ];
