@@ -213,6 +213,7 @@ class api {
             'tool_mobile_qrcodetype' => clean_param(get_config('tool_mobile', 'qrcodetype'), PARAM_INT),
             'supportpage' => $sitesupportavailable ? clean_param($CFG->supportpage, PARAM_URL) : '',
             'supportavailability' => clean_param($CFG->supportavailability, PARAM_INT),
+            'showloginform' => (int) get_config('core', 'showloginform'),
         );
 
         $typeoflogin = get_config('tool_mobile', 'typeoflogin');
@@ -319,7 +320,14 @@ class api {
             $settings->tool_mobile_customlangstrings = get_config('tool_mobile', 'customlangstrings');
             $settings->tool_mobile_disabledfeatures = get_config('tool_mobile', 'disabledfeatures');
             $settings->tool_mobile_filetypeexclusionlist = get_config('tool_mobile', 'filetypeexclusionlist');
-            $settings->tool_mobile_custommenuitems = get_config('tool_mobile', 'custommenuitems');
+            $custommenuitems = get_config('tool_mobile', 'custommenuitems');
+            // If filtering of the primary custom menu is enabled, apply only the string filters.
+            if (!empty($CFG->navfilter && !empty($CFG->stringfilters))) {
+                // Apply filters that are enabled for Content and Headings.
+                $filtermanager = \filter_manager::instance();
+                $custommenuitems = $filtermanager->filter_string($custommenuitems, \context_system::instance());
+            }
+            $settings->tool_mobile_custommenuitems = $custommenuitems;
             $settings->tool_mobile_apppolicy = get_config('tool_mobile', 'apppolicy');
             // This setting could be not set in some edge cases such as bad upgrade.
             $mintimereq = get_config('tool_mobile', 'autologinmintimebetweenreq');
@@ -462,13 +470,13 @@ class api {
     public static function get_qrlogin_key(stdClass $mobilesettings) {
         global $USER;
         // Delete previous keys.
-        delete_user_key('tool_mobile', $USER->id);
+        delete_user_key('tool_mobile/qrlogin', $USER->id);
 
         // Create a new key.
         $iprestriction = !empty($mobilesettings->qrsameipcheck) ? getremoteaddr(null) : null;
         $qrkeyttl = !empty($mobilesettings->qrkeyttl) ? $mobilesettings->qrkeyttl : self::LOGIN_QR_KEY_TTL;
         $validuntil = time() + $qrkeyttl;
-        return create_user_key('tool_mobile', $USER->id, null, $iprestriction, $validuntil);
+        return create_user_key('tool_mobile/qrlogin', $USER->id, null, $iprestriction, $validuntil);
     }
 
     /**
@@ -495,8 +503,8 @@ class api {
         $availablemods = core_plugin_manager::instance()->get_plugins_of_type('mod');
         $coursemodules = array();
         $appsupportedmodules = array(
-            'assign', 'bigbluebuttonbn', 'book', 'chat', 'choice', 'data', 'feedback', 'folder', 'forum', 'glossary', 'h5pactivity',
-            'imscp', 'label', 'lesson', 'lti', 'page', 'quiz', 'resource', 'scorm', 'survey', 'url', 'wiki', 'workshop');
+            'assign', 'bigbluebuttonbn', 'book', 'choice', 'data', 'feedback', 'folder', 'forum', 'glossary', 'h5pactivity',
+            'imscp', 'label', 'lesson', 'lti', 'page', 'quiz', 'resource', 'scorm', 'url', 'wiki', 'workshop');
 
         foreach ($availablemods as $mod) {
             if (in_array($mod->name, $appsupportedmodules)) {
