@@ -35,13 +35,13 @@ require_once($CFG->dirroot . '/webservice/lib.php');
  * @license     http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  * @since       Moodle 3.1
  */
-class externallib_test extends externallib_advanced_testcase {
+final class externallib_test extends externallib_advanced_testcase {
 
     /**
      * Test get_plugins_supporting_mobile.
      * This is a very basic test because currently there aren't plugins supporting Mobile in core.
      */
-    public function test_get_plugins_supporting_mobile() {
+    public function test_get_plugins_supporting_mobile(): void {
         $result = external::get_plugins_supporting_mobile();
         $result = external_api::clean_returnvalue(external::get_plugins_supporting_mobile_returns(), $result);
         $this->assertCount(0, $result['warnings']);
@@ -49,7 +49,7 @@ class externallib_test extends externallib_advanced_testcase {
         $this->assertTrue(is_array($result['plugins']));
     }
 
-    public function test_get_public_config() {
+    public function test_get_public_config(): void {
         global $CFG, $SITE, $OUTPUT;
 
         $this->resetAfterTest(true);
@@ -102,7 +102,8 @@ class externallib_test extends externallib_advanced_testcase {
             'tool_mobile_qrcodetype' => get_config('tool_mobile', 'qrcodetype'),
             'supportpage' => $CFG->supportpage,
             'supportavailability' => $CFG->supportavailability,
-            'warnings' => array()
+            'warnings' => [],
+            'showloginform' => (int) get_config('core', 'showloginform'),
         );
         $this->assertEquals($expected, $result);
 
@@ -123,17 +124,14 @@ class externallib_test extends externallib_advanced_testcase {
         set_config('supportemail', 'test@test.com');
         set_config('supportavailability', CONTACT_SUPPORT_ANYONE);
 
-        // Enable couple of issuers.
+        // Enable an issuer.
         $issuer = \core\oauth2\api::create_standard_issuer('google');
         $irecord = $issuer->to_record();
         $irecord->clientid = 'mock';
         $irecord->clientsecret = 'mock';
         \core\oauth2\api::update_issuer($irecord);
 
-        set_config('hostname', 'localhost', 'auth_cas');
-        set_config('auth_logo', 'http://invalidurl.com//invalid/', 'auth_cas');
-        set_config('auth_name', 'CAS', 'auth_cas');
-        set_config('auth', 'oauth2,cas');
+        set_config('auth', 'oauth2');
 
         list($authinstructions, $notusedformat) = \core_external\util::format_text($authinstructions, FORMAT_MOODLE, $context->id);
         $expected['registerauth'] = 'email';
@@ -158,26 +156,15 @@ class externallib_test extends externallib_advanced_testcase {
 
         $result = external::get_public_config();
         $result = external_api::clean_returnvalue(external::get_public_config_returns(), $result);
-        // First check providers.
-        $identityproviders = $result['identityproviders'];
+
+        // First check the provider.
+        $identityprovider = $result['identityproviders'];
         unset($result['identityproviders']);
 
-        $this->assertEquals('Google', $identityproviders[0]['name']);
-        $this->assertEquals($irecord->image, $identityproviders[0]['iconurl']);
-        $this->assertStringContainsString($CFG->wwwroot, $identityproviders[0]['url']);
-
-        $this->assertEquals('CAS', $identityproviders[1]['name']);
-        $this->assertEmpty($identityproviders[1]['iconurl']);
-        $this->assertStringContainsString($CFG->wwwroot, $identityproviders[1]['url']);
-
+        $this->assertEquals('Google', $identityprovider[0]['name']);
+        $this->assertEquals($irecord->image, $identityprovider[0]['iconurl']);
+        $this->assertStringContainsString($CFG->wwwroot, $identityprovider[0]['url']);
         $this->assertEquals($expected, $result);
-
-        // Change providers img.
-        $newurl = 'validimage.png';
-        set_config('auth_logo', $newurl, 'auth_cas');
-        $result = external::get_public_config();
-        $result = external_api::clean_returnvalue(external::get_public_config_returns(), $result);
-        $this->assertStringContainsString($newurl, $result['identityproviders'][1]['iconurl']);
     }
 
     /**
@@ -305,7 +292,7 @@ class externallib_test extends externallib_advanced_testcase {
     /*
      * Test get_autologin_key.
      */
-    public function test_get_autologin_key() {
+    public function test_get_autologin_key(): void {
         global $DB, $CFG, $USER;
 
         $this->resetAfterTest(true);
@@ -349,7 +336,7 @@ class externallib_test extends externallib_advanced_testcase {
     /**
      * Test get_autologin_key missing ws.
      */
-    public function test_get_autologin_key_missing_ws() {
+    public function test_get_autologin_key_missing_ws(): void {
         global $CFG;
         $this->resetAfterTest(true);
 
@@ -370,7 +357,7 @@ class externallib_test extends externallib_advanced_testcase {
     /**
      * Test get_autologin_key missing https.
      */
-    public function test_get_autologin_key_missing_https() {
+    public function test_get_autologin_key_missing_https(): void {
         global $CFG;
 
         // Fake the app.
@@ -391,7 +378,7 @@ class externallib_test extends externallib_advanced_testcase {
     /**
      * Test get_autologin_key missing admin.
      */
-    public function test_get_autologin_key_missing_admin() {
+    public function test_get_autologin_key_missing_admin(): void {
         global $CFG;
 
         $this->resetAfterTest(true);
@@ -409,7 +396,7 @@ class externallib_test extends externallib_advanced_testcase {
     /**
      * Test get_autologin_key locked.
      */
-    public function test_get_autologin_key_missing_locked() {
+    public function test_get_autologin_key_missing_locked(): void {
         global $CFG, $DB, $USER;
 
         $this->resetAfterTest(true);
@@ -452,7 +439,7 @@ class externallib_test extends externallib_advanced_testcase {
     /**
      * Test get_autologin_key missing app_request.
      */
-    public function test_get_autologin_key_missing_app_request() {
+    public function test_get_autologin_key_missing_app_request(): void {
         global $CFG;
 
         $this->resetAfterTest(true);
@@ -466,7 +453,7 @@ class externallib_test extends externallib_advanced_testcase {
     /**
      * Test get_content.
      */
-    public function test_get_content() {
+    public function test_get_content(): void {
 
         $paramval = 16;
         $result = external::get_content('tool_mobile', 'test_view', array(array('name' => 'param1', 'value' => $paramval)));
@@ -489,7 +476,7 @@ class externallib_test extends externallib_advanced_testcase {
     /**
      * Test get_content disabled.
      */
-    public function test_get_content_disabled() {
+    public function test_get_content_disabled(): void {
 
         $paramval = 16;
         $result = external::get_content('tool_mobile', 'test_view_disabled',
@@ -501,7 +488,7 @@ class externallib_test extends externallib_advanced_testcase {
     /**
      * Test get_content non existent function in valid component.
      */
-    public function test_get_content_non_existent_function() {
+    public function test_get_content_non_existent_function(): void {
 
         $this->expectException('coding_exception');
         $result = external::get_content('tool_mobile', 'test_blahblah');
@@ -510,7 +497,7 @@ class externallib_test extends externallib_advanced_testcase {
     /**
      * Test get_content incorrect component.
      */
-    public function test_get_content_invalid_component() {
+    public function test_get_content_invalid_component(): void {
 
         $this->expectException('moodle_exception');
         $result = external::get_content('tool_mobile\hack', 'test_view');
@@ -519,13 +506,13 @@ class externallib_test extends externallib_advanced_testcase {
     /**
      * Test get_content non existent component.
      */
-    public function test_get_content_non_existent_component() {
+    public function test_get_content_non_existent_component(): void {
 
         $this->expectException('moodle_exception');
         $result = external::get_content('tool_blahblahblah', 'test_view');
     }
 
-    public function test_call_external_functions() {
+    public function test_call_external_functions(): void {
         global $SESSION;
 
         $this->resetAfterTest(true);
@@ -670,7 +657,7 @@ class externallib_test extends externallib_advanced_testcase {
     /*
      * Test get_tokens_for_qr_login.
      */
-    public function test_get_tokens_for_qr_login() {
+    public function test_get_tokens_for_qr_login(): void {
         global $DB, $CFG, $USER;
 
         $this->resetAfterTest(true);
@@ -706,7 +693,7 @@ class externallib_test extends externallib_advanced_testcase {
     /*
      * Test get_tokens_for_qr_login ignore ip check.
      */
-    public function test_get_tokens_for_qr_login_ignore_ip_check() {
+    public function test_get_tokens_for_qr_login_ignore_ip_check(): void {
         global $DB, $CFG, $USER;
 
         $this->resetAfterTest(true);
@@ -745,7 +732,7 @@ class externallib_test extends externallib_advanced_testcase {
     /*
      * Test get_tokens_for_qr_login ip check fails.
      */
-    public function test_get_tokens_for_qr_login_ip_check_mismatch() {
+    public function test_get_tokens_for_qr_login_ip_check_mismatch(): void {
         global $DB, $CFG, $USER;
 
         $this->resetAfterTest(true);
@@ -776,7 +763,7 @@ class externallib_test extends externallib_advanced_testcase {
     /**
      * Test get_tokens_for_qr_login missing QR code enabled.
      */
-    public function test_get_tokens_for_qr_login_missing_enableqr() {
+    public function test_get_tokens_for_qr_login_missing_enableqr(): void {
         global $CFG, $USER;
         $this->resetAfterTest(true);
         $this->setAdminUser();
@@ -790,7 +777,7 @@ class externallib_test extends externallib_advanced_testcase {
     /**
      * Test get_tokens_for_qr_login missing ws.
      */
-    public function test_get_tokens_for_qr_login_missing_ws() {
+    public function test_get_tokens_for_qr_login_missing_ws(): void {
         global $CFG;
         $this->resetAfterTest(true);
 
@@ -814,7 +801,7 @@ class externallib_test extends externallib_advanced_testcase {
     /**
      * Test get_tokens_for_qr_login missing https.
      */
-    public function test_get_tokens_for_qr_login_missing_https() {
+    public function test_get_tokens_for_qr_login_missing_https(): void {
         global $CFG, $USER;
 
         // Fake the app.
@@ -835,7 +822,7 @@ class externallib_test extends externallib_advanced_testcase {
     /**
      * Test get_tokens_for_qr_login missing admin.
      */
-    public function test_get_tokens_for_qr_login_missing_admin() {
+    public function test_get_tokens_for_qr_login_missing_admin(): void {
         global $CFG, $USER;
 
         $this->resetAfterTest(true);
@@ -853,7 +840,7 @@ class externallib_test extends externallib_advanced_testcase {
     /**
      * Test get_tokens_for_qr_login missing app_request.
      */
-    public function test_get_tokens_for_qr_login_missing_app_request() {
+    public function test_get_tokens_for_qr_login_missing_app_request(): void {
         global $CFG, $USER;
 
         $this->resetAfterTest(true);
@@ -867,7 +854,7 @@ class externallib_test extends externallib_advanced_testcase {
     /**
      * Test validate subscription key.
      */
-    public function test_validate_subscription_key_valid() {
+    public function test_validate_subscription_key_valid(): void {
         $this->resetAfterTest(true);
 
         $sitesubscriptionkey = ['validuntil' => time() + MINSECS, 'key' => complex_random_string(32)];
@@ -882,7 +869,7 @@ class externallib_test extends externallib_advanced_testcase {
     /**
      * Test validate subscription key invalid first and then a valid one.
      */
-    public function test_validate_subscription_key_invalid_key_first() {
+    public function test_validate_subscription_key_invalid_key_first(): void {
         $this->resetAfterTest(true);
 
         $sitesubscriptionkey = ['validuntil' => time() + MINSECS, 'key' => complex_random_string(32)];
@@ -903,7 +890,7 @@ class externallib_test extends externallib_advanced_testcase {
     /**
      * Test validate subscription key invalid.
      */
-    public function test_validate_subscription_key_invalid_key() {
+    public function test_validate_subscription_key_invalid_key(): void {
         $this->resetAfterTest(true);
 
         $result = external::validate_subscription_key('fakekey');
@@ -915,7 +902,7 @@ class externallib_test extends externallib_advanced_testcase {
     /**
      * Test validate subscription key invalid.
      */
-    public function test_validate_subscription_key_outdated() {
+    public function test_validate_subscription_key_outdated(): void {
         $this->resetAfterTest(true);
 
         $sitesubscriptionkey = ['validuntil' => time() - MINSECS, 'key' => complex_random_string(32)];
