@@ -25,6 +25,9 @@
 
 defined('MOODLE_INTERNAL') || die;
 
-if ($hassiteconfig) {
-    $ADMIN->add('roles', new admin_externalpage('toolunsuproles', get_string('pluginname', 'tool_unsuproles'), "$CFG->wwwroot/$CFG->admin/tool/unsuproles/index.php", array('moodle/site:config', 'moodle/role:assign')));
+$capabilities = ['moodle/role:assign'];
+
+if ($hassiteconfig || has_any_capability($capabilities, core\context\system::instance())) {
+    $ADMIN->add('roles', new admin_externalpage('toolunsuproles', get_string('pluginname', 'tool_unsuproles'),
+        "$CFG->wwwroot/$CFG->admin/tool/unsuproles/index.php", ['moodle/site:config', 'moodle/role:assign']));
 }
