@@ -20,13 +20,12 @@ namespace core_badges\reportbuilder\datasource;
 
 use core_badges_generator;
 use core_reportbuilder_generator;
-use core_reportbuilder_testcase;
 use core_reportbuilder\local\filters\{boolean_select, date, select, tags, text};
+use core_reportbuilder\tests\core_reportbuilder_testcase;
 
 defined('MOODLE_INTERNAL') || die();
 
 global $CFG;
-require_once("{$CFG->dirroot}/reportbuilder/tests/helpers.php");
 require_once("{$CFG->libdir}/badgeslib.php");
 
 /**
@@ -37,7 +36,7 @@ require_once("{$CFG->libdir}/badgeslib.php");
  * @copyright   2023 Paul Holden <paulh@moodle.com>
  * @license     http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
-class users_test extends core_reportbuilder_testcase {
+final class users_test extends core_reportbuilder_testcase {
 
     /**
      * Test default datasource
@@ -92,6 +91,9 @@ class users_test extends core_reportbuilder_testcase {
         $course = $this->getDataGenerator()->create_course();
         $user = $this->getDataGenerator()->create_and_enrol($course, 'student', ['firstname' => 'Zoe', 'lastname' => 'Zebra']);
 
+        $cohort = $this->getDataGenerator()->create_cohort(['name' => 'My cohort']);
+        cohort_add_member($cohort->id, $user->id);
+
         /** @var core_badges_generator $generator */
         $generator = $this->getDataGenerator()->get_plugin_generator('core_badges');
 
@@ -137,14 +139,17 @@ class users_test extends core_reportbuilder_testcase {
 
         $generator->create_column(['reportid' => $report->get('id'), 'uniqueidentifier' => 'course:fullname']);
 
+        $generator->create_column(['reportid' => $report->get('id'), 'uniqueidentifier' => 'cohort:name']);
+
         $content = $this->get_custom_report_content($report->get('id'));
         $this->assertCount(3, $content);
 
         // Admin user, no badge issued.
-        [$fullname, $badgename, $criteria, $image, $language, $version, $status, $expiry, $tag, $expires, $visible, $coursename]
-            = array_values($content[0]);
+        [$fullname, $badgename, $namewithlink, $criteria, $image, $language, $version, $status, $expiry, $tag, $expires,
+            $visible, $coursename, $cohortname] = array_values($content[0]);
         $this->assertEquals('Admin User', $fullname);
         $this->assertEmpty($badgename);
+        $this->assertEmpty($namewithlink);
         $this->assertEmpty($criteria);
         $this->assertEmpty($image);
         $this->assertEmpty($language);
@@ -155,13 +160,14 @@ class users_test extends core_reportbuilder_testcase {
         $this->assertEmpty($expires);
         $this->assertEmpty($visible);
         $this->assertEmpty($coursename);
+        $this->assertEmpty($cohortname);
 
         $expectedbadgesitelink = \html_writer::link(new \moodle_url('/badges/overview.php',
             ['id' => $badgesite->id]), ($badgesite->name));
 
         // User issued site badge.
         [$fullname, $badgename, $namewithlink, $criteria, $image, $language, $version, $status, $expiry, $tag, $expires,
-            $visible, $coursename] = array_values($content[1]);
+            $visible, $coursename, $cohortname] = array_values($content[1]);
         $this->assertEquals(fullname($user), $fullname);
         $this->assertEquals($badgesite->name, $badgename);
         $this->assertEquals($expectedbadgesitelink, $namewithlink);
@@ -175,26 +181,28 @@ class users_test extends core_reportbuilder_testcase {
         $this->assertNotEmpty($expires);
         $this->assertEquals('Yes', $visible);
         $this->assertEquals('PHPUnit test site', $coursename);
+        $this->assertEquals($cohort->name, $cohortname);
 
         $expectedbadgecourselink = \html_writer::link(new \moodle_url('/badges/overview.php',
             ['id' => $badgecourse->id]), ($badgecourse->name));
 
         // User issued course badge.
         [$fullname, $badgename, $namewithlink, $criteria, $image, $language, $version, $status, $expiry, $tag, $expires,
-            $visible, $coursename] = array_values($content[2]);
+            $visible, $coursename, $cohortname] = array_values($content[2]);
         $this->assertEquals(fullname($user), $fullname);
         $this->assertEquals($badgecourse->name, $badgename);
         $this->assertEquals($expectedbadgecourselink, $namewithlink);
-        $this->assertEquals('Criteria for this badge have not been set up yet.', $criteria);
+        $this->assertStringContainsString('no-criteria-set', $criteria);
         $this->assertStringContainsString('Image caption', $image);
         $this->assertEquals('English', $language);
         $this->assertEquals(2, $version);
         $this->assertEquals('Available (criteria locked)', $status);
-        $this->assertEquals('Never', $expiry);
+        $this->assertEquals('Never: this badge does not expire.', $expiry);
         $this->assertEmpty($tag);
         $this->assertEmpty($expires);
         $this->assertEquals('Yes', $visible);
         $this->assertEquals($course->fullname, $coursename);
+        $this->assertEquals($cohort->name, $cohortname);
     }
 
     /**
@@ -202,7 +210,7 @@ class users_test extends core_reportbuilder_testcase {
      *
      * @return array[]
      */
-    public function datasource_filters_provider(): array {
+    public static function datasource_filters_provider(): array {
         return [
             // User.
             'Filter user fullname' => ['user:fullname', [
@@ -282,6 +290,16 @@ class users_test extends core_reportbuilder_testcase {
                 'course:fullname_operator' => text::IS_EQUAL_TO,
                 'course:fullname_value' => 'Course 2',
             ], false],
+
+            // Cohort.
+            'Filter cohort name' => ['cohort:name', [
+                'cohort:name_operator' => text::IS_EQUAL_TO,
+                'cohort:name_value' => 'My cohort',
+            ], true],
+            'Filter cohort name (no match)' => ['cohort:name', [
+                'cohort:name_operator' => text::IS_EQUAL_TO,
+                'cohort:name_value' => 'Not my cohort',
+            ], false],
         ];
     }
 
@@ -300,6 +318,9 @@ class users_test extends core_reportbuilder_testcase {
         $course = $this->getDataGenerator()->create_course(['fullname' => 'Course 1']);
         $user = $this->getDataGenerator()->create_and_enrol($course, 'student', ['firstname' => 'Zoe', 'lastname' => 'Zebra']);
 
+        $cohort = $this->getDataGenerator()->create_cohort(['name' => 'My cohort']);
+        cohort_add_member($cohort->id, $user->id);
+
         /** @var core_badges_generator $generator */
         $generator = $this->getDataGenerator()->get_plugin_generator('core_badges');
         $badge = $generator->create_badge([
@@ -315,7 +336,7 @@ class users_test extends core_reportbuilder_testcase {
         $generator = $this->getDataGenerator()->get_plugin_generator('core_reportbuilder');
 
         // Create report containing single username column, and given filter.
-        $report = $generator->create_report(['name' => 'My report', 'source' => users::class, 'default' => 0]);
+        $report = $generator->create_report(['name' => 'Badges', 'source' => users::class, 'default' => 0]);
         $generator->create_column(['reportid' => $report->get('id'), 'uniqueidentifier' => 'user:username']);
 
         // Add filter, set it's values.
