@@ -37,13 +37,15 @@ require_once($CFG->dirroot . '/calendar/lib.php');
  * @package core_calendar
  * @copyright 2017 Cameron Ball <cameron@cameron1729.xyz>
  * @license http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
+ * @covers \core_calendar\local\event\container
  */
-class container_test extends \advanced_testcase {
+final class container_test extends \advanced_testcase {
 
     /**
      * Test setup.
      */
     public function setUp(): void {
+        parent::setUp();
         $this->resetAfterTest();
         $this->setAdminUser();
     }
@@ -67,7 +69,7 @@ class container_test extends \advanced_testcase {
     /**
      * Test that the event factory correctly creates instances of events.
      *
-     * @dataProvider get_event_factory_testcases()
+     * @dataProvider get_event_factory_testcases
      * @param \stdClass $dbrow Row from the "database".
      */
     public function test_event_factory_create_instance($dbrow): void {
@@ -128,7 +130,7 @@ class container_test extends \advanced_testcase {
     /**
      * Test that the event factory deals with invisible modules properly as admin.
      *
-     * @dataProvider get_event_factory_testcases()
+     * @dataProvider get_event_factory_testcases
      * @param \stdClass $dbrow Row from the "database".
      */
     public function test_event_factory_when_module_visibility_is_toggled_as_admin($dbrow): void {
@@ -154,7 +156,7 @@ class container_test extends \advanced_testcase {
     /**
      * Test that the event factory deals with invisible modules properly as a guest.
      *
-     * @dataProvider get_event_factory_testcases()
+     * @dataProvider get_event_factory_testcases
      * @param \stdClass $dbrow Row from the "database".
      */
     public function test_event_factory_when_module_visibility_is_toggled_as_guest($dbrow): void {
@@ -183,7 +185,7 @@ class container_test extends \advanced_testcase {
     /**
      * Test that the event factory deals with invisible courses as an admin.
      *
-     * @dataProvider get_event_factory_testcases()
+     * @dataProvider get_event_factory_testcases
      * @param \stdClass $dbrow Row from the "database".
      */
     public function test_event_factory_when_course_visibility_is_toggled_as_admin($dbrow): void {
@@ -208,7 +210,7 @@ class container_test extends \advanced_testcase {
     /**
      * Test that the event factory deals with invisible courses as a student.
      *
-     * @dataProvider get_event_factory_testcases()
+     * @dataProvider get_event_factory_testcases
      * @param \stdClass $dbrow Row from the "database".
      */
     public function test_event_factory_when_course_visibility_is_toggled_as_student($dbrow): void {
@@ -369,7 +371,6 @@ class container_test extends \advanced_testcase {
 
     /**
      * Checks that completed activities events do not show.
-     * @covers \core_calendar\local\event::init
      */
     public function test_event_factory_with_completed_module_related_event(): void {
         global $CFG, $DB;
@@ -504,7 +505,7 @@ class container_test extends \advanced_testcase {
     /**
      * Test cases for the get event factory test.
      */
-    public function get_event_factory_testcases() {
+    public static function get_event_factory_testcases(): array {
         return [
             'Data set 1' => [
                 'dbrow' => (object)[
