@@ -23,7 +23,7 @@
  * @covers     \core_courseformat\base
  * @coversDefaultClass \core_courseformat\base
  */
-class base_test extends advanced_testcase {
+final class base_test extends advanced_testcase {
 
     /**
      * Setup to ensure that fixtures are loaded.
@@ -40,9 +40,8 @@ class base_test extends advanced_testcase {
      * Tests the save and load functionality.
      *
      * @author Jason den Dulk
-     * @covers \core_courseformat
      */
-    public function test_courseformat_saveandload() {
+    public function test_courseformat_saveandload(): void {
         $this->resetAfterTest();
 
         $courseformatoptiondata = (object) [
@@ -65,7 +64,7 @@ class base_test extends advanced_testcase {
         $this->assertEqualsCanonicalizing($courseformatoptiondata, (object) $savedcourseformatoptiondata);
     }
 
-    public function test_available_hook() {
+    public function test_available_hook(): void {
         global $DB;
         $this->resetAfterTest();
 
@@ -184,7 +183,7 @@ class base_test extends advanced_testcase {
     /**
      * Test for supports_news() with a course format plugin that doesn't define 'news_items' in default blocks.
      */
-    public function test_supports_news() {
+    public function test_supports_news(): void {
         $this->resetAfterTest();
         $format = course_get_format((object)['format' => 'testformat']);
         $this->assertFalse($format->supports_news());
@@ -193,7 +192,7 @@ class base_test extends advanced_testcase {
     /**
      * Test for supports_news() for old course format plugins that defines 'news_items' in default blocks.
      */
-    public function test_supports_news_legacy() {
+    public function test_supports_news_legacy(): void {
         $this->resetAfterTest();
         $format = course_get_format((object)['format' => 'testlegacy']);
         $this->assertTrue($format->supports_news());
@@ -263,7 +262,7 @@ class base_test extends advanced_testcase {
      * @param string $result the expected result classname
      * @param bool $exception if the method will raise an exception
      */
-    public function test_get_output_classname($find, $result, $exception) {
+    public function test_get_output_classname($find, $result, $exception): void {
         $this->resetAfterTest();
 
         $course = $this->getDataGenerator()->create_course(['format' => 'theunittest']);
@@ -282,7 +281,7 @@ class base_test extends advanced_testcase {
      *
      * @return array the testing scenarios
      */
-    public function get_output_classname_provider(): array {
+    public static function get_output_classname_provider(): array {
         return [
             'overridden class' => [
                 'find' => 'state\\course',
@@ -307,7 +306,7 @@ class base_test extends advanced_testcase {
      *
      * @covers ::get_sections_preferences
      */
-    public function test_get_sections_preferences() {
+    public function test_get_sections_preferences(): void {
         $this->resetAfterTest();
         $generator = $this->getDataGenerator();
         $course = $generator->create_course();
@@ -341,7 +340,7 @@ class base_test extends advanced_testcase {
      *
      * @covers ::set_sections_preference
      */
-    public function test_set_sections_preference() {
+    public function test_set_sections_preference(): void {
         $this->resetAfterTest();
         $generator = $this->getDataGenerator();
         $course = $generator->create_course();
@@ -466,7 +465,7 @@ class base_test extends advanced_testcase {
      * @dataProvider delete_format_data_provider
      * @param bool $usehook if it should use course_delete to trigger $format->delete_format_data as a hook
      */
-    public function test_delete_format_data(bool $usehook) {
+    public function test_delete_format_data(bool $usehook): void {
         global $DB;
 
         $this->resetAfterTest();
@@ -517,7 +516,7 @@ class base_test extends advanced_testcase {
      *
      * @return array the testing scenarios
      */
-    public function delete_format_data_provider(): array {
+    public static function delete_format_data_provider(): array {
         return [
             'direct call' => [
                 'usehook' => false
@@ -532,7 +531,7 @@ class base_test extends advanced_testcase {
      * Test duplicate_section()
      * @covers ::duplicate_section
      */
-    public function test_duplicate_section() {
+    public function test_duplicate_section(): void {
         global $DB;
 
         $this->setAdminUser();
@@ -542,13 +541,14 @@ class base_test extends advanced_testcase {
         $course = $generator->create_course();
         $format = course_get_format($course);
 
-        $originalsection = $DB->get_record('course_sections', ['course' => $course->id, 'section' => 1], '*', MUST_EXIST);
+        $originalsection = $DB->get_record('course_sections', ['course' => $course->id, 'section' => 0], '*', MUST_EXIST);
         $generator->create_module('page', ['course' => $course, 'section' => $originalsection->section]);
         $generator->create_module('page', ['course' => $course, 'section' => $originalsection->section]);
         $generator->create_module('page', ['course' => $course, 'section' => $originalsection->section]);
+        $generator->create_module('qbank', ['course' => $course, 'section' => $originalsection->section]);
 
         $originalmodcount = $DB->count_records('course_modules', ['course' => $course->id, 'section' => $originalsection->id]);
-        $this->assertEquals(3, $originalmodcount);
+        $this->assertEquals(4, $originalmodcount);
 
         $modinfo = get_fast_modinfo($course);
         $sectioninfo = $modinfo->get_section_info($originalsection->section, MUST_EXIST);
@@ -564,7 +564,49 @@ class base_test extends advanced_testcase {
         }
 
         $newmodcount = $DB->count_records('course_modules', ['course' => $course->id, 'section' => $newsection->id]);
-        $this->assertEquals($originalmodcount, $newmodcount);
+        $modinfo = course_modinfo::instance($course);
+        $qbankinstances = $modinfo->get_instances_of('qbank');
+        $this->assertCount(1, $qbankinstances);
+        $this->assertEquals($originalmodcount - 1, $newmodcount);
+    }
+
+    /**
+     * Test duplicate_section() with delegated section
+     * @covers     ::duplicate_section
+     */
+    public function test_duplicate_section_with_delegated_sections(): void {
+        global $DB;
+
+        $this->setAdminUser();
+        $this->resetAfterTest();
+        // Add subsection.
+        $manager = \core_plugin_manager::resolve_plugininfo_class('mod');
+        $manager::enable_plugin('subsection', 1);
+        $course = $this->getDataGenerator()->create_course(['format' => 'topics', 'numsections' => 1]);
+        $subsection1 = $this->getDataGenerator()->create_module(
+            'subsection', ['course' => $course, 'section' => 1, 'name' => 'subsection1']);
+        $subsection2 = $this->getDataGenerator()->create_module(
+            'subsection', ['course' => $course, 'section' => 1, 'name' => 'subsection2']);
+        $format = course_get_format($course);
+
+        $modinfo = get_fast_modinfo($course);
+        $sectioninfo = $modinfo->get_section_info(1, MUST_EXIST);
+        $originalsectioncount = $DB->count_records('course_sections', ['course' => $course->id]);
+        $this->assertEquals(4, $originalsectioncount);
+
+        $originalsection = $DB->get_record('course_sections',
+            ['course' => $course->id, 'section' => 0],
+            '*',
+            MUST_EXIST);
+        $newsection = $format->duplicate_section($sectioninfo);
+        foreach ($originalsection as $prop => $value) {
+            if ($prop == 'id' || $prop == 'sequence' || $prop == 'section' || $prop == 'timemodified') {
+                continue;
+            }
+            $this->assertEquals($value, $newsection->$prop);
+        }
+        $sectioncount = $DB->count_records('course_sections', ['course' => $course->id]);
+        $this->assertEquals(7, $sectioncount);
     }
 
     /**
@@ -605,7 +647,7 @@ class base_test extends advanced_testcase {
      *
      * @return array the testing scenarios
      */
-    public function get_format_string_provider(): array {
+    public static function get_format_string_provider(): array {
         return [
             'Existing in format lang' => [
                 'key' => 'addsection',
@@ -645,7 +687,7 @@ class base_test extends advanced_testcase {
      * @param string $destination the reference of the destination section
      * @param string[] $order the references of the final section order
      */
-    public function test_move_section_after(string $movesection, string $destination, array $order) {
+    public function test_move_section_after(string $movesection, string $destination, array $order): void {
         global $DB;
 
         $this->resetAfterTest();
@@ -684,7 +726,7 @@ class base_test extends advanced_testcase {
      *
      * @return array the testing scenarios
      */
-    public function move_section_after_provider(): array {
+    public static function move_section_after_provider(): array {
         return [
             'Move top' => [
                 'movesection' => 'section3',
@@ -770,7 +812,7 @@ class base_test extends advanced_testcase {
      * @param string $expectedparam the expected param to check
      * @param string $exception if an exception is expected
      */
-    public function test_get_non_ajax_cm_action_url(string $action, string $expectedparam, bool $exception) {
+    public function test_get_non_ajax_cm_action_url(string $action, string $expectedparam, bool $exception): void {
         global $DB;
 
         $this->resetAfterTest();
@@ -787,7 +829,10 @@ class base_test extends advanced_testcase {
             $this->expectException(\coding_exception::class);
         }
         $result = $format->get_non_ajax_cm_action_url($action, $cminfo);
-        $this->assertEquals($assign0->cmid, $result->param($expectedparam));
+        if (!$exception) {
+            $this->assertDebuggingCalled();
+        }
+        $this->assertEquals($assign0->cmid, $result->param('id'));
     }
 
     /**
@@ -795,7 +840,7 @@ class base_test extends advanced_testcase {
      *
      * @return array the testing scenarios
      */
-    public function get_non_ajax_cm_action_url_provider(): array {
+    public static function get_non_ajax_cm_action_url_provider(): array {
         return [
             'duplicate' => [
                 'action' => 'cmDuplicate',
@@ -965,6 +1010,141 @@ class base_test extends advanced_testcase {
         $format = course_get_format($course);
         $this->assertTrue($format->can_sections_be_removed_from_navigation());
     }
+
+    public function test_is_section_visible(): void {
+        $this->resetAfterTest();
+
+        $generator = $this->getDataGenerator();
+        $course = $generator->create_course(['format' => 'testformatsections'], ['hiddensections' => 1]);
+        course_create_sections_if_missing($course, [0, 1, 2]);
+
+        // Students cannot view hidden sections.
+        $sectioninfo = get_fast_modinfo($course)->get_section_info(1);
+        \core_courseformat\formatactions::section($course)->update($sectioninfo, ['visible' => false]);
+
+        $format = course_get_format($course);
+
+        // Force max sections to 1 to detect section 2 as orphan.
+        $format->forcemaxsections = 1;
+
+        $teacher = $this->getDataGenerator()->create_and_enrol($course, 'editingteacher');
+        $student = $this->getDataGenerator()->create_and_enrol($course, 'student');
+
+        $this->setUser($teacher);
+        $modinfoteacher = get_fast_modinfo($course, $teacher->id);
+        $this->assertTrue($format->is_section_visible($modinfoteacher->get_section_info(0)));
+        $this->assertTrue($format->is_section_visible($modinfoteacher->get_section_info(1)));
+        $this->assertTrue($format->is_section_visible($modinfoteacher->get_section_info(2)));
+
+        $this->setUser($student);
+        $modinfostudent = get_fast_modinfo($course, $student->id);
+        $this->assertTrue($format->is_section_visible($modinfostudent->get_section_info(0)));
+        $this->assertFalse($format->is_section_visible($modinfostudent->get_section_info(1)));
+        $this->assertFalse($format->is_section_visible($modinfostudent->get_section_info(2)));
+    }
+
+    /**
+     * Test for the get_generic_section_name method.
+     *
+     * @covers ::get_generic_section_name
+     */
+    public function test_get_generic_section_name(): void {
+        $this->resetAfterTest();
+
+        $generator = $this->getDataGenerator();
+        $course1 = $generator->create_course(['format' => 'topics']);
+        $course2 = $generator->create_course(['format' => 'theunittest']);
+
+        $format = course_get_format($course1);
+        $this->assertEquals(
+            get_string('sectionname', 'format_topics'),
+            $format->get_generic_section_name()
+        );
+
+        $format = course_get_format($course2);
+        $this->assertEquals(
+            get_string('section'),
+            $format->get_generic_section_name()
+        );
+    }
+
+    /**
+     * Test can_sections_be_removed_from_navigation().
+     *
+     * @covers ::session_cache
+     * @covers ::session_cache_reset
+     * @covers ::session_cache_reset_all
+     * @covers ::invalidate_all_session_caches_for_course
+     */
+    public function test_session_caches_methods(): void {
+        global $DB;
+
+        $this->resetAfterTest();
+        $generator = $this->getDataGenerator();
+
+        $course1 = $generator->create_course(['format' => 'topics']);
+        $course2 = $generator->create_course(['format' => 'topics']);
+
+        // Force some cacherev to the course.
+        $course1->cacherev = 12345;
+        $course2->cacherev = 67890;
+        $DB->set_field('course', 'cacherev', $course1->cacherev, ['id' => $course1->id]);
+        $DB->set_field('course', 'cacherev', $course2->cacherev, ['id' => $course2->id]);
+
+        $teacher = $generator->create_and_enrol($course1, 'editingteacher');
+        $generator->enrol_user($teacher->id, $course2->id, 'editingteacher');
+        $this->setUser($teacher);
+
+        // The cache key uses time() as hash. To not wait a second between calls we fake an initial value.
+        $statecache = cache::make('core', 'courseeditorstate');
+        $statecache->set($course1->id, $course1->cacherev . '_11111');
+        $statecache->set($course2->id, $course2->cacherev . '_22222');
+
+        $course1cachekey = \core_courseformat\base::session_cache($course1);
+
+        // Validate the method returns the same value when called twice.
+        $course1cachekeyagain = \core_courseformat\base::session_cache($course1);
+        $this->assertEquals($course1cachekey, $course1cachekeyagain);
+
+        // Validate other course has a diferent cache key.
+        $course2cachekey = \core_courseformat\base::session_cache($course2);
+        $this->assertNotEquals($course1cachekey, $course2cachekey);
+
+        // Reset the specific course cache.
+        \core_courseformat\base::session_cache_reset($course1);
+
+        $resetcachekey = \core_courseformat\base::session_cache($course1);
+        $this->assertNotEquals($course1cachekey, $resetcachekey);
+
+        $reset2cachekey = \core_courseformat\base::session_cache($course2);
+        $this->assertEquals($course2cachekey, $reset2cachekey);
+
+        // Return to the initial value.
+        $statecache->set($course1->id, $course1->cacherev . '_11111');
+        $statecache->set($course2->id, $course2->cacherev . '_22222');
+
+        // Reset all user course caches.
+        \core_courseformat\base::session_cache_reset_all();
+
+        $resetallcachekey = \core_courseformat\base::session_cache($course1);
+        $this->assertNotEquals($course1cachekey, $resetallcachekey);
+
+        $resetall2cachekey = \core_courseformat\base::session_cache($course2);
+        $this->assertNotEquals($course2cachekey, $resetall2cachekey);
+
+        // Return to the initial value.
+        $statecache->set($course1->id, $course1->cacherev . '_11111');
+        $statecache->set($course2->id, $course2->cacherev . '_22222');
+
+        // Invalidate cache on course 1.
+        \core_courseformat\base::invalidate_all_session_caches_for_course($course1);
+
+        $invalidatecachekey = \core_courseformat\base::session_cache($course1);
+        $this->assertNotEquals($course1cachekey, $invalidatecachekey);
+
+        $invalidate2cachekey = \core_courseformat\base::session_cache($course2);
+        $this->assertEquals($course2cachekey, $invalidate2cachekey);
+    }
 }
 
 /**
@@ -1000,6 +1180,10 @@ class format_testformat extends core_courseformat\base {
  * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
 class format_testformatsections extends core_courseformat\base {
+    /**
+     * @var int|null $forcemaxsections The maximum number of sections.
+     */
+    public ?int $forcemaxsections = null;
     /**
      * Returns if this course format uses sections.
      *
@@ -1012,6 +1196,13 @@ class format_testformatsections extends core_courseformat\base {
     public function can_sections_be_removed_from_navigation(): bool {
         return true;
     }
+
+    public function get_last_section_number(): int {
+        if ($this->forcemaxsections !== null) {
+            return $this->forcemaxsections;
+        }
+        return parent::get_last_section_number();
+    }
 }
 
 /**
