@@ -829,7 +829,7 @@ class grade_edit_tree_column_name extends grade_edit_tree_column {
         if ($this->deepest_level > 1) {
             if (empty($params['eid'])) {
                 throw new Exception('Array key (eid) missing from 3rd param of ' .
-                    'grade_edit_tree_column_select::get_category_cell($category, $levelclass, $params)');
+                    'grade_edit_tree_column_name::get_category_cell($category, $levelclass, $params)');
             }
 
             // Get toggle group for this master checkbox.
@@ -856,7 +856,7 @@ class grade_edit_tree_column_name extends grade_edit_tree_column {
         $categorycell = parent::get_category_cell($category, $levelclass, $params);
         $categorycell->colspan = ($this->deepest_level + 2) - $params['level'];
         $rowtitle = html_writer::div($params['name'], 'rowtitle');
-        $categorycell->text = html_writer::div($mastercheckbox . $visibilitytoggle . $moveaction . $rowtitle, 'font-weight-bold');
+        $categorycell->text = html_writer::div($mastercheckbox . $visibilitytoggle . $moveaction . $rowtitle, 'fw-bold');
         return $categorycell;
     }
 
@@ -865,7 +865,7 @@ class grade_edit_tree_column_name extends grade_edit_tree_column {
             throw new Exception('Array key (name, level or element) missing from 2nd param of grade_edit_tree_column_name::get_item_cell($item, $params)');
         }
 
-        $itemicon = \html_writer::div($params['icon'], 'mr-1');
+        $itemicon = \html_writer::div($params['icon'], 'me-1');
         $itemtype = \html_writer::span($params['type'], 'd-block text-uppercase small dimmed_text');
         $itemtitle = html_writer::div($params['name'], 'rowtitle');
         $content = \html_writer::div($itemtype . $itemtitle);
@@ -1163,131 +1163,3 @@ class grade_edit_tree_column_actions extends grade_edit_tree_column {
         return $itemcell;
     }
 }
-
-/**
- * Class grade_edit_tree_column_select
- *
- * @deprecated Since Moodle 4.3.
- * @todo Final deprecation on Moodle 4.7 MDL-77668
- *
- * @package   core_grades
- * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
- */
-class grade_edit_tree_column_select extends grade_edit_tree_column {
-
-    /**
-     * @deprecated Since Moodle 4.3.
-     * @todo Final deprecation on Moodle 4.7 MDL-77668
-     */
-    public function get_header_cell() {
-        debugging('Method grade_edit_tree_column_select::get_header_cell() is deprecated, ' .
-            'please do not use it anymore.', DEBUG_DEVELOPER);
-
-        $headercell = clone($this->headercell);
-        $headercell->text = get_string('select');
-        return $headercell;
-    }
-
-    /**
-     * @deprecated Since Moodle 4.3.
-     * @todo Final deprecation on Moodle 4.7 MDL-77668
-     */
-    public function get_category_cell($category, $levelclass, $params) {
-        global $OUTPUT;
-
-        debugging('Method grade_edit_tree_column_select::get_category_cell() is deprecated, ' .
-            'please do not use it anymore.', DEBUG_DEVELOPER);
-
-        if (empty($params['eid'])) {
-            throw new Exception('Array key (eid) missing from 3rd param of grade_edit_tree_column_select::get_category_cell($category, $levelclass, $params)');
-        }
-
-        // Get toggle group for this master checkbox.
-        $togglegroup = $this->get_checkbox_togglegroup($category);
-        // Set label for this master checkbox.
-        $masterlabel = get_string('all');
-        // Use category name if available.
-        if ($category->fullname !== '?') {
-            $masterlabel = format_string($category->fullname, true, ['escape' => false]);
-            // Limit the displayed category name to prevent the Select column from getting too wide.
-            if (core_text::strlen($masterlabel) > 20) {
-                $masterlabel = get_string('textellipsis', 'core', core_text::substr($masterlabel, 0, 12));
-            }
-        }
-        // Build the master checkbox.
-        $mastercheckbox = new \core\output\checkbox_toggleall($togglegroup, true, [
-            'id' => 'select_category_' . $category->id,
-            'name' => $togglegroup,
-            'value' => 1,
-            'classes' => 'itemselect ignoredirty',
-            'label' => $masterlabel,
-            // Consistent label to prevent the select column from resizing.
-            'selectall' => $masterlabel,
-            'deselectall' => $masterlabel,
-            'labelclasses' => 'm-0',
-        ]);
-
-        $categorycell = parent::get_category_cell($category, $levelclass, $params);
-        $categorycell->text = $OUTPUT->render($mastercheckbox);
-        return $categorycell;
-    }
-
-    /**
-     * @deprecated Since Moodle 4.3.
-     * @todo Final deprecation on Moodle 4.7 MDL-77668
-     */
-    public function get_item_cell($item, $params) {
-        debugging('Method grade_edit_tree_column_select::get_item_cell() is deprecated, ' .
-            'please do not use it anymore.', DEBUG_DEVELOPER);
-
-        if (empty($params['itemtype']) || empty($params['eid'])) {
-            throw new \moodle_exception('missingitemtypeoreid', 'core_grades');
-        }
-        $itemcell = parent::get_item_cell($item, $params);
-
-        if ($params['itemtype'] != 'course' && $params['itemtype'] != 'category') {
-            global $OUTPUT;
-
-            // Fetch the grade item's category.
-            $category = grade_category::fetch(['id' => $item->categoryid]);
-            $togglegroup = $this->get_checkbox_togglegroup($category);
-
-            $checkboxid = 'select_' . $params['eid'];
-            $checkbox = new \core\output\checkbox_toggleall($togglegroup, false, [
-                'id' => $checkboxid,
-                'name' => $checkboxid,
-                'label' => get_string('select', 'grades', $item->itemname),
-                'labelclasses' => 'accesshide',
-                'classes' => 'itemselect ignoredirty',
-            ]);
-            $itemcell->text = $OUTPUT->render($checkbox);
-        }
-        return $itemcell;
-    }
-
-    /**
-     * Generates a toggle group name for a bulk-action checkbox based on the given grade category.
-     *
-     * @deprecated Since Moodle 4.3.
-     * @todo Final deprecation on Moodle 4.7 MDL-77668
-     *
-     * @param grade_category $category The grade category.
-     * @return string
-     */
-    protected function get_checkbox_togglegroup(grade_category $category): string {
-        debugging('Method grade_edit_tree_column_select::get_checkbox_togglegroup() is deprecated, ' .
-            'please do not use it anymore.', DEBUG_DEVELOPER);
-
-        $levels = [];
-        $categories = explode('/', $category->path);
-        foreach ($categories as $categoryid) {
-            $level = 'category' . $categoryid;
-            if (!in_array($level, $levels)) {
-                $levels[] = 'category' . $categoryid;
-            }
-        }
-        $togglegroup = implode(' ', $levels);
-
-        return $togglegroup;
-    }
-}
