@@ -36,15 +36,17 @@ $string['configenrolplugins'] = 'Please select all required plugins and arrange
 $string['custominstancename'] = 'Custom instance name';
 $string['customwelcomemessage'] = 'Custom welcome message';
 $string['customwelcomemessage_help'] = 'Accepted formats: Plain text or Moodle-auto format. HTML tags and multi-lang tags are also accepted, as well as the following placeholders:
-<br>
-* Course name {$a->coursename}<br>
-* Link to user\'s profile page {$a->profileurl}<br>
-* User email {$a->email}<br>
-* User fullname {$a->fullname}<br>
-* User first name {$a->firstname}<br>
-* User last name {$a->lastname}<br>
-* User course role {$a->courserole}<br>';
-$string['customwelcomemessageplaceholder'] = 'Dear {$a->fullname}, you have successfully been enrolled to course {$a->coursename}';
+<ul>
+<li>Course name {$a->coursename}</li>
+<li>Link to course page {$a->courselink}</li>
+<li>Link to user\'s profile page {$a->profileurl}</li>
+<li>User email {$a->email}</li>
+<li>User fullname {$a->fullname}</li>
+<li>User first name {$a->firstname}</li>
+<li>User last name {$a->lastname}</li>
+<li>User course role {$a->courserole}</li>
+</ul>';
+$string['customwelcomemessageplaceholder'] = 'Hi {$a->firstname}, you are enrolled in the course {$a->coursename}.';
 $string['defaultenrol'] = 'Add instance to new courses';
 $string['defaultenrol_desc'] = 'It is possible to add this plugin to all new courses by default.';
 $string['deleteinstanceconfirm'] = 'You are about to delete the enrolment method "{$a->name}". All {$a->users} users currently enrolled using this method will be unenrolled and any course-related data such as users\' grades, group membership or forum subscriptions will be deleted.
@@ -132,6 +134,7 @@ $string['periodend'] = 'until {$a}';
 $string['periodnone'] = 'enrolled {$a}';
 $string['periodstart'] = 'from {$a}';
 $string['periodstartend'] = 'from {$a->start} until {$a->end}';
+$string['plugindisabled'] = '{$a} enrol plugin is disabled';
 $string['recovergrades'] = 'Recover user\'s old grades if possible';
 $string['rolefromthiscourse'] = '{$a->role} (Assigned in this course)';
 $string['rolefrommetacourse'] = '{$a->role} (Inherited from parent course)';
