@@ -645,7 +645,6 @@ trait behat_session_trait {
      * @return void Throws an exception if it times out without the element being visible
      */
     protected function ensure_node_is_visible($node) {
-
         if (!$this->running_javascript()) {
             return;
         }
@@ -715,7 +714,6 @@ trait behat_session_trait {
      * @return NodeElement Throws an exception if it times out without being visible
      */
     protected function ensure_element_is_visible($element, $selectortype) {
-
         if (!$this->running_javascript()) {
             return;
         }
@@ -746,9 +744,14 @@ trait behat_session_trait {
      *
      * @param string $windowsize size of window.
      * @param bool $viewport If true, changes viewport rather than window size
+     * @param bool $scalesize Whether to scale the size by the WINDOWSCALE environment variable
      * @throws ExpectationException
      */
-    protected function resize_window($windowsize, $viewport = false) {
+    protected function resize_window(
+        string $windowsize,
+        bool $viewport = false,
+        bool $scalesize = true,
+    ): void {
         global $CFG;
 
         // Non JS don't support resize window.
@@ -792,6 +795,16 @@ trait behat_session_trait {
             $height *= $CFG->behat_window_size_modifier;
         }
 
+        if ($scalesize) {
+            // Scale the window size by the WINDOWSCALE environment variable.
+            // This is intended to be used for Behat reruns to negate the impact of browser window size issues.
+            // This allows a per-run, runtime configuration of the scaling, unlike behat_window_size_modifier which
+            // typically applies to all runs.
+            $scalefactor = getenv('WINDOWSCALE') ? floatval(getenv('WINDOWSCALE')) : 1;
+            $width *= $scalefactor;
+            $height *= $scalefactor;
+        }
+
         if ($viewport) {
             // When setting viewport size, we set it so that the document width will be exactly
             // as specified, assuming that there is a vertical scrollbar. (In cases where there is
@@ -1025,14 +1038,58 @@ EOF;
             return;
         }
 
-        // Look for any DOM element with deprecated message in before pseudo-element.
+        // Look for DOM elements with deprecated message in before pseudo-element.
         $js = <<<EOF
-            [...document.querySelectorAll('*')].some(
-                el => window.getComputedStyle(el, ':before').content === '"Deprecated style in use"'
+            [...document.querySelectorAll('*')].flatMap(el => {
+                const beforeContent = window.getComputedStyle(el, ':before').content;
+                if (beforeContent.startsWith('"Deprecated style in use')) {
+                    const deprecatedClass = beforeContent.match(/\(([^)]+)\)/)?.[1] ?? 'unknown';
+                    return [deprecatedClass + ' (found in: ' + el.classList + ')'];
+                }
+                return [];
+            });
+        EOF;
+
+        $deprecations = $this->evaluate_script($js);
+        if ($deprecations) {
+            $deprecationdata = "Deprecated styles found:\n";
+            foreach ($deprecations as $deprecation) {
+                $deprecationdata .= "  {$deprecation}\n";
+            }
+            throw new \Exception(html_entity_decode($deprecationdata, ENT_COMPAT));
+        }
+    }
+
+
+    /**
+     * Internal step definition to find deprecated icons.
+     *
+     * Part of behat_hooks class as is part of the testing framework, is auto-executed
+     * after each step so no features will splicitly use it.
+     *
+     * @throws Exception Unknown type, depending on what we caught in the hook or basic \Exception.
+     * @see Moodle\BehatExtension\Tester\MoodleStepTester
+     */
+    public function look_for_deprecated_icons() {
+        if (behat_config_manager::get_behat_run_config_value('no-icon-deprecations')) {
+            return;
+        }
+
+        if (!$this->running_javascript()) {
+            return;
+        }
+
+        // Look for any DOM element with deprecated icon.
+        $js = <<<EOF
+            [...document.querySelectorAll('.icon.deprecated')].some(
+                deprecatedicon => true
             );
         EOF;
         if ($this->evaluate_script($js)) {
-            throw new \Exception(html_entity_decode("Deprecated style in use", ENT_COMPAT));
+            throw new \Exception(html_entity_decode(
+                "Deprecated icon in use. Enable \$CFG->debugdisplay for detailed debugging information in the console",
+                ENT_COMPAT,
+            ));
         }
     }
 
@@ -1077,6 +1134,9 @@ EOF;
 
         // Look for deprecated styles.
         $this->look_for_deprecated_styles();
+
+        // Look for deprecated icons.
+        $this->look_for_deprecated_icons();
     }
 
     /**
@@ -1114,11 +1174,11 @@ EOF;
         if (empty($sid)) {
             throw new coding_exception('failed to get moodle session');
         }
-        $userid = $DB->get_field('sessions', 'userid', ['sid' => $sid]);
-        if (empty($userid)) {
-            throw new coding_exception('failed to get user from seession id '.$sid);
+        $session = \core\session\manager::get_session_by_sid($sid);
+        if (empty($session->userid)) {
+            throw new coding_exception('failed to get user from session id: '.$sid);
         }
-        return $DB->get_record('user', ['id' => $userid]);
+        return $DB->get_record('user', ['id' => $session->userid]);
     }
 
     /**
@@ -1629,7 +1689,7 @@ EOF;
      *    // Note: phpDoc beforeStep attribution not shown.
      *    public function before_step(StepScope $scope) {
      *        $callback = function (string $tag): bool {
-     *            return $tag === 'editor_atto' || substr($tag, 0, 5) === 'atto_';
+     *            return $tag === 'editor_tiny' || substr($tag, 0, 5) === 'tiny_';
      *        };
      *
      *        if (!self::scope_tags_match($scope, $callback)) {
@@ -1670,6 +1730,32 @@ EOF;
         return !empty($matches);
     }
 
+    /**
+     * Get the user object from an identifier.
+     *
+     * The user username and email fields are checked.
+     *
+     * @param string $identifier The user's username or email.
+     * @return stdClass|null The user id or null if not found.
+     */
+    protected function get_user_by_identifier(string $identifier): ?stdClass {
+        global $DB;
+
+        $sql = <<<EOF
+            SELECT *
+              FROM {user}
+             WHERE username = :username
+                OR email = :email
+        EOF;
+
+        $result = $DB->get_record_sql($sql, [
+            'username' => $identifier,
+            'email' => $identifier,
+        ]);
+
+        return $result ?: null;
+    }
+
     /**
      * Get the user id from an identifier.
      *
@@ -1695,4 +1781,19 @@ EOF;
 
         return $result ?: null;
     }
+
+    /**
+     * Prepare an xpath for insertion into Selenium JavaScript.
+     *
+     * @param string $xpath
+     * @return string
+     */
+    protected function prepare_xpath_for_javascript(string $xpath): string {
+        $newlines = [
+            "\r\n",
+            "\r",
+            "\n",
+        ];
+        return str_replace($newlines, ' ', $xpath);
+    }
 }
