@@ -113,19 +113,6 @@ class mysql_sql_generator extends sql_generator {
         return array("ALTER TABLE $this->prefix$tablename AUTO_INCREMENT = $value");
     }
 
-    /**
-     * Calculate proximate row size when using InnoDB
-     * tables in Antelope row format.
-     *
-     * Note: the returned value is a bit higher to compensate for
-     *       errors and changes of column data types.
-     *
-     * @deprecated since Moodle 2.9 MDL-49723 - please do not use this function any more.
-     */
-    public function guess_antolope_row_size(array $columns) {
-        throw new coding_exception('guess_antolope_row_size() can not be used any more, please use guess_antelope_row_size() instead.');
-    }
-
     /**
      * Calculate proximate row size when using InnoDB tables in Antelope row format.
      *
@@ -638,7 +625,10 @@ class mysql_sql_generator extends sql_generator {
             '_filename', 'admin', 'cume_dist', 'dense_rank', 'empty', 'except', 'first_value', 'grouping', 'groups',
             'json_table', 'lag', 'last_value', 'lead', 'nth_value', 'ntile',
             'of', 'over', 'percent_rank', 'persist', 'persist_only', 'rank', 'recursive', 'row_number',
-            'system', 'window'
+            'system', 'window',
+            // Added in Amazon Aurora MySQL version 3.06.0:
+            // https://docs.aws.amazon.com/AmazonRDS/latest/AuroraMySQLReleaseNotes/AuroraMySQL.Updates.3060.html .
+            'accept', 'aws_bedrock_invoke_model', 'aws_sagemaker_invoke_endpoint', 'content_type', 'timeout_ms',
         );
         return $reserved_words;
     }
