@@ -195,7 +195,7 @@ abstract class test_moodle_database extends \moodle_database {
      * @return bool true
      * @throws Exception
      */
-    public function execute($sql, array $params = null) {
+    public function execute($sql, ?array $params = null) {
         throw new Exception("execute() not implemented");
     }
 
@@ -208,7 +208,7 @@ abstract class test_moodle_database extends \moodle_database {
      * @return bool true
      * @throws Exception
      */
-    public function get_recordset_sql($sql, array $params = null, $limitfrom = 0, $limitnum = 0) {
+    public function get_recordset_sql($sql, ?array $params = null, $limitfrom = 0, $limitnum = 0) {
         throw new Exception("get_recordset_sql() not implemented");
     }
 
@@ -221,7 +221,7 @@ abstract class test_moodle_database extends \moodle_database {
      * @return bool true
      * @throws Exception
      */
-    public function get_records_sql($sql, array $params = null, $limitfrom = 0, $limitnum = 0) {
+    public function get_records_sql($sql, ?array $params = null, $limitfrom = 0, $limitnum = 0) {
         throw new Exception("get_records_sql() not implemented");
     }
 
@@ -232,7 +232,7 @@ abstract class test_moodle_database extends \moodle_database {
      * @return bool true
      * @throws Exception
      */
-    public function get_fieldset_sql($sql, array $params = null) {
+    public function get_fieldset_sql($sql, ?array $params = null) {
         throw new Exception("get_fieldset_sql() not implemented");
     }
 
@@ -308,7 +308,7 @@ abstract class test_moodle_database extends \moodle_database {
      * @return bool true
      * @throws Exception
      */
-    public function set_field_select($table, $newfield, $newvalue, $select, array $params = null) {
+    public function set_field_select($table, $newfield, $newvalue, $select, ?array $params = null) {
         throw new Exception("set_field_select() not implemented");
     }
 
@@ -320,7 +320,7 @@ abstract class test_moodle_database extends \moodle_database {
      * @return bool true
      * @throws Exception
      */
-    public function delete_records_select($table, $select, array $params = null) {
+    public function delete_records_select($table, $select, ?array $params = null) {
         throw new Exception("delete_records_select() not implemented");
     }
 
