@@ -778,7 +778,7 @@ class grade_item extends grade_object {
      * @param string $groupwheresql Where conditions for $groupsql
      * @return int The number of hidden grades
      */
-    public function has_hidden_grades($groupsql="", array $params=null, $groupwheresql="") {
+    public function has_hidden_grades($groupsql="", ?array $params=null, $groupwheresql="") {
         global $DB;
         $params = (array)$params;
         $params['itemid'] = $this->id;
@@ -2086,6 +2086,11 @@ class grade_item extends grade_object {
         }
         // end of hack alert
 
+        // Only reset the deducted mark if the grade has changed.
+        if ($grade->timemodified !== $oldgrade->timemodified) {
+            $grade->deductedmark = 0;
+        }
+
         $gradechanged = false;
         if (empty($grade->id)) {
             $result = (bool)$grade->insert($source, $isbulkupdate);
@@ -2146,6 +2151,21 @@ class grade_item extends grade_object {
         return $result;
     }
 
+    /**
+     * Update penalty value for given user
+     *
+     * @param int $userid The graded user
+     * @param float $deductedmark The mark deducted from final grade
+     */
+    public function update_deducted_mark(int $userid, float $deductedmark): void {
+        $grade = new grade_grade([
+                'itemid' => $this->id,
+                'userid' => $userid,
+            ]);
+        $grade->deductedmark = $deductedmark;
+        $grade->update();
+    }
+
     /**
      * Calculates final grade values using the formula in the calculation property.
      * The parameters are taken from final grades of grade items in current course only.
@@ -2211,25 +2231,21 @@ class grade_item extends grade_object {
         list($usql, $params) = $DB->get_in_or_equal($gis);
 
         if ($userid) {
-            $usersql = "AND g.userid=?";
+            $usersql = "AND userid=?";
             $params[] = $userid;
         } else {
             $usersql = "";
         }
 
-        $grade_inst = new grade_grade();
-        $fields = 'g.'.implode(',g.', $grade_inst->required_fields);
+        $gradeinst = new grade_grade();
+        $fields = implode(',', $gradeinst->required_fields);
 
         $params[] = $this->courseid;
-        $sql = "SELECT $fields
-                  FROM {grade_grades} g, {grade_items} gi
-                 WHERE gi.id = g.itemid AND gi.id $usql $usersql AND gi.courseid=?
-                 ORDER BY g.userid";
 
         $return = true;
 
         // group the grades by userid and use formula on the group
-        $rs = $DB->get_recordset_sql($sql, $params);
+        $rs = $DB->get_recordset_select('grade_grades', "itemid $usql $usersql", $params, 'userid', $fields);
         if ($rs->valid()) {
             $prevuser = 0;
             $grade_records   = array();
