@@ -142,6 +142,11 @@ class restore_assign_activity_structure_step extends restore_activity_structure_
             $data->grade = -($this->get_mappingid('scale', abs($data->grade)));
         }
 
+        // Grade penalty.
+        if (!isset($data->gradepenalty)) {
+            $data->gradepenalty = 0;
+        }
+
         $newitemid = $DB->insert_record('assign', $data);
 
         $this->apply_activity_instance($newitemid);
@@ -421,11 +426,11 @@ class restore_assign_activity_structure_step extends restore_activity_structure_
 
     /**
      * Once the database tables have been fully restored, restore the files
-     * @return void
      */
     protected function after_execute() {
         $this->add_related_files('mod_assign', 'intro', null);
         $this->add_related_files('mod_assign', 'introattachment', null);
+        $this->add_related_files('mod_assign', 'activityattachment', null);
 
         $this->add_plugin_config_files('assignsubmission');
         $this->add_plugin_config_files('assignfeedback');
