@@ -407,6 +407,27 @@ class assign_submission_onlinetext extends assign_submission_plugin {
         return '';
     }
 
+
+    #[\Override]
+    public function submission_summary_for_messages(stdClass $submission): array {
+        global $PAGE;
+
+        $onlinetextsubmission = $this->get_onlinetext_submission($submission->id);
+        if (!$onlinetextsubmission || !$onlinetextsubmission->onlinetext) {
+            return ['', ''];
+        }
+
+        $renderer = $PAGE->get_renderer('mod_assign');
+        $templatecontext = ['wordcount' => count_words($onlinetextsubmission->onlinetext)];
+        return [
+            // Mustache strips off all trailing whitespace, but we want a newline at the end.
+            $renderer->render_from_template(
+               'assignsubmission_onlinetext/email_summary_text', $templatecontext) . "\n",
+            $renderer->render_from_template(
+               'assignsubmission_onlinetext/email_summary_html', $templatecontext),
+        ];
+    }
+
     /**
      * Produce a list of files suitable for export that represent this submission.
      *
@@ -422,7 +443,7 @@ class assign_submission_onlinetext extends assign_submission_plugin {
 
         // Note that this check is the same logic as the result from the is_empty function but we do
         // not call it directly because we already have the submission record.
-        if ($onlinetextsubmission) {
+        if ($onlinetextsubmission && !html_is_blank($onlinetextsubmission->onlinetext)) {
             // Do not pass the text through format_text. The result may not be displayed in Moodle and
             // may be passed to external services such as document conversion or portfolios.
             $formattedtext = $this->assignment->download_rewrite_pluginfile_urls($onlinetextsubmission->onlinetext, $user, $this);
