@@ -14,68 +14,22 @@
 // You should have received a copy of the GNU General Public License
 // along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
 
-/**
- * Base class for unit tests for mod_assign.
- *
- * @package    mod_assign
- * @copyright  2018 Adrian Greeve <adrian@moodle.com>
- * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
- */
-
 namespace mod_assign\privacy;
 
-defined('MOODLE_INTERNAL') || die();
-
-global $CFG;
-require_once($CFG->dirroot . '/mod/assign/locallib.php');
-
-use core_privacy\tests\provider_testcase;
 use core_privacy\local\request\writer;
 use core_privacy\local\request\approved_contextlist;
 use mod_assign\privacy\provider;
+use mod_assign\tests\provider_testcase;
 
 /**
  * Unit tests for mod/assign/classes/privacy/
  *
+ * @package    mod_assign
  * @copyright  2018 Adrian Greeve <adrian@moodle.com>
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
+ * @covers \mod_assign\privacy\provider
  */
-class provider_test extends provider_testcase {
-
-    /**
-     * Convenience method for creating a submission.
-     *
-     * @param  assign  $assign The assign object
-     * @param  stdClass  $user The user object
-     * @param  string  $submissiontext Submission text
-     * @param  integer $attemptnumber The attempt number
-     * @return object A submission object.
-     */
-    protected function create_submission($assign, $user, $submissiontext, $attemptnumber = 0) {
-        $submission = $assign->get_user_submission($user->id, true, $attemptnumber);
-        $submission->onlinetext_editor = ['text' => $submissiontext,
-                                         'format' => FORMAT_MOODLE];
-
-        $this->setUser($user);
-        $notices = [];
-        $assign->save_submission($submission, $notices);
-        return $submission;
-    }
-
-    /**
-     * Convenience function to create an instance of an assignment.
-     *
-     * @param array $params Array of parameters to pass to the generator
-     * @return assign The assign class.
-     */
-    protected function create_instance($params = array()) {
-        $generator = $this->getDataGenerator()->get_plugin_generator('mod_assign');
-        $instance = $generator->create_instance($params);
-        $cm = get_coursemodule_from_instance('assign', $instance->id);
-        $context = \context_module::instance($cm->id);
-        return new \assign($context, $cm, $params['course']);
-    }
-
+final class provider_test extends provider_testcase {
     /**
      * Test that getting the contexts for a user works.
      */
