@@ -15,7 +15,7 @@
 // along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
 
 /**
- * Language File.
+ * Strings for component 'mod_bigbluebuttonbn', language 'en'.
  *
  * @package   mod_bigbluebuttonbn
  * @copyright 2010 onwards, Blindside Networks Inc
@@ -43,6 +43,7 @@ $string['bigbluebuttonbn:protectrecordings'] = 'Protect recordings';
 $string['bigbluebuttonbn:unprotectrecordings'] = 'Unprotect recordings';
 $string['bigbluebuttonbn:deleterecordings'] = 'Delete recordings';
 $string['bigbluebuttonbn:importrecordings'] = 'Import recordings';
+$string['bigbluebuttonbn:seepresentation'] = 'Always see presentation file on activity page';
 $string['bigbluebuttonbn:viewallrecordingformats'] = 'View all recording formats';
 $string['bigbluebuttonbn'] = 'BigBlueButton';
 $string['cannotperformaction'] = 'Cannot perform action {$a} on this recording';
@@ -206,14 +207,19 @@ $string['config_voicebridge_description'] = 'These settings enable or disable op
 $string['config_voicebridge_editable'] = 'Conference voice bridge can be edited';
 $string['config_voicebridge_editable_description'] = 'A conference voice bridge number can be permanently assigned to a room. When assigned, the number cannot be used for any other room.';
 
-$string['config_preuploadpresentation'] = 'Pre-upload presentation';
+$string['config_preuploadpresentation'] = 'Presentation file';
 $string['config_preuploadpresentation_description'] = 'These settings enable or disable options in the UI and also define default values for these options. The feature works only if the Moodle server is accessible to BigBlueButton.';
-$string['config_preuploadpresentation_editable'] = 'Pre-uploading presentation editable';
-$string['config_preuploadpresentation_editable_description'] = 'Preupload presentation feature is editable in the UI when the room or conference is added or updated.';
+$string['config_preuploadpresentation_editable'] = 'Presentation file on activity page';
+$string['config_preuploadpresentation_editable_description'] = 'Option in activity settings to upload presentation file to show on activity page.';
 
 $string['config_presentation_default'] = 'Default presentation file';
 $string['config_presentation_default_description'] = 'A file may be provided for use in all rooms.';
 
+$string['config_showpresentation_default'] = 'Show presentation file on activity page';
+$string['config_showpresentation_default_description'] = 'Should students have access to the presentation file before the session? This setting has no effect on users with the capability \'Always see presentation file on activity page\' .';
+$string['config_showpresentation_editable'] = '\'Show presentation file on activity page\' option';
+$string['config_showpresentation_editable_description'] = 'Option in activity settings for teachers to choose whether to show the presentation file to students.';
+
 $string['config_participant'] = 'Participants';
 $string['config_participant_description'] = 'These settings define the default role for participants.';
 $string['config_participant_moderator_default'] = 'Moderator';
@@ -253,7 +259,7 @@ $string['config_muteonstart_default_description'] = 'If enabled the session will
 $string['config_muteonstart_editable'] = 'Mute on start can be edited';
 $string['config_muteonstart_editable_description'] = 'Mute on start by default can be edited when the instance is added or updated.';
 $string['config_welcome_default'] = 'Default welcome message';
-$string['config_welcome_default_description'] = 'The welcome message is displayed when participants enter the room. If the field is left blank, then a message set on the BigBlueButton server is displayed.';
+$string['config_welcome_default_description'] = 'The welcome message is displayed when participants enter the room. If the field is left blank, the language string \'mod_form_field_welcome_default\' is used.';
 $string['config_welcome_editable'] = 'Default welcome message is editable by teachers';
 $string['config_welcome_editable_description'] = 'Welcome message can be edited when the instance is added or updated';
 $string['config_default_messages'] = 'Default messages';
@@ -396,6 +402,7 @@ $string['mod_form_field_participant_list_action_add'] = 'Add';
 $string['mod_form_field_participant_list_action_remove'] = 'Remove';
 $string['mod_form_field_participant_bbb_role_moderator'] = 'Moderator';
 $string['mod_form_field_participant_bbb_role_viewer'] = 'Viewer';
+$string['mod_form_field_showpresentation'] = 'Show presentation file on activity page';
 $string['mod_form_field_instanceprofiles'] = 'Instance type';
 $string['mod_form_field_instanceprofiles_help'] = 'If a session is to be recorded, select \'Room with recordings\', otherwise \'Room only\'. After a session is recorded, if there are to be no more sessions, select \'Recordings only\'.';
 $string['mod_form_field_muteonstart'] = 'Mute on start';
@@ -626,9 +633,6 @@ $string['view_room'] = 'View room';
 $string['index_error_noinstances'] = 'There are no instances of BigBlueButton rooms';
 $string['index_error_bbtn'] = 'BigBlueButton ID {$a} is incorrect';
 
-$string['view_mobile_message_reload_page_creation_time_meeting'] = 'You exceeded 45 seconds on this page. Please refresh the page to join the session.';
-$string['view_mobile_message_groups_not_supported'] = 'This instance is enabled to work with groups but the mobile app doesn\'t yet support it. Please use the web version.';
-
 $string['end_session_confirm_title'] = 'Really end session?';
 $string['end_session_confirm'] = 'Are you sure you want to end the session?';
 $string['end_session_notification'] = 'The session has ended.';
@@ -642,28 +646,12 @@ $string['taskname:check_dismissed_recordings'] = 'Check for recordings that have
 $string['userlimitreached'] = 'The number of users allowed in a session has been reached.';
 $string['waitformoderator'] = 'Waiting for a moderator to join.';
 
+$string['recordingnotfound'] = 'The recording was not found.';
 $string['recordingurlnotfound'] = 'The recording URL is invalid.';
 
 $string['subplugintype_bbbext'] = 'BigBlueButton activity extension';
 $string['subplugintype_bbbext_plural'] = 'BigBlueButton activity extensions';
 
-// Deprecated since Moodle 4.3.
-$string['completionview'] = 'Require view';
-$string['completionview_desc'] = 'View the room';
-$string['completionattendancegroup_help'] = 'Attending the meeting for (n) minutes is required for completion.';
-$string['completionengagementgroup_help'] = 'Active participation during the session is required for completion.';
-// Deprecated since Moodle 4.4.
-$string['acceptdpa'] = 'I understand and accept the data processing agreement';
-$string['bigbluebuttondisablednotification_subject'] = 'BigBlueButton activity module disabled.';
-$string['bigbluebuttondisablednotification'] = 'The BigBlueButton activity module has been disabled and any existing BigBlueButton course activities are currently not accessible. Prior to re-enabling this plugin, please ensure that you have read and accepted the <a href="{$a}" target="_blank">data processing agreement</a> with Blindside Networks Inc.';
-$string['enablingbigbluebutton'] = 'Enabling BigBlueButton activity';
-$string['enablingbigbluebuttondpainfo'] = 'In order to meet your data protection obligations, prior to enabling this plugin, you may need to ensure that you have read and accepted the <a href="{$a}" target="_blank">Blindside Networks data processing agreement</a>. Please consult with your own privacy professionals for advice.';
-$string['dpainfonotsigned'] = 'Before enabling this plugin, you must confirm that you have read and accepted the <a href="{$a}">Blindside Networks data processing agreement</a>.';
-$string['config_dpa_note'] = 'Note: In order to meet your data protection obligations, before using a service provider for this plugin, you must ensure that you have read and accepted the service provider\'s data processing agreement. For the default free BigBlueButton service, this is the <a href="{$a}" target="_blank">Blindside Networks data processing agreement</a>. Please consult with your own privacy professionals for advice.';
-$string['view_warning_default_server'] = 'This site is using a <a href="https://bigbluebutton.org/free-bigbluebutton-service-for-moodle/" target="_blank">free BigBlueButton service for Moodle (opens in new window)</a> provided by Blindside Networks with restrictions as follows:
-<ol>
-<li>The maximum length for each session is 60 minutes</li>
-<li>The maximum number of concurrent users per session is 25</li>
-<li>Recordings expire after seven (7) days and are not downloadable</li>
-<li>Student webcams are only visible to the moderator.</li>
-</ol>';
+// Deprecated since Moodle 4.5.
+$string['view_mobile_message_reload_page_creation_time_meeting'] = 'You exceeded 45 seconds on this page. Please refresh the page to join the session.';
+$string['view_mobile_message_groups_not_supported'] = 'This instance is enabled to work with groups but the mobile app doesn\'t yet support it. Please use the web version.';
