@@ -43,7 +43,7 @@ require_once($CFG->dirroot . '/rating/lib.php');
  * @copyright  2019 Ryan Wyllie <ryan@moodle.com>
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
-class exporters_post_test extends \advanced_testcase {
+final class exporters_post_test extends \advanced_testcase {
     // Make use of the test generator trait.
     use mod_forum_tests_generator_trait;
 
@@ -201,7 +201,7 @@ class exporters_post_test extends \advanced_testcase {
      *
      * @return array
      */
-    public function export_post_provider(): array {
+    public static function export_post_provider(): array {
         return [
             'Simple export' => [
             ],
