@@ -31,7 +31,7 @@ require_once("{$CFG->dirroot}/mod/forum/lib.php");
  * @copyright  2013 Frédéric Massart
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
-class subscriptions_test extends \advanced_testcase {
+final class subscriptions_test extends \advanced_testcase {
     // Include the mod_forum test helpers.
     // This includes functions to create forums, users, discussions, and posts.
     use mod_forum_tests_generator_trait;
@@ -41,6 +41,7 @@ class subscriptions_test extends \advanced_testcase {
      */
     public function setUp(): void {
         global $DB;
+        parent::setUp();
 
         // We must clear the subscription caches. This has to be done both before each test, and after in case of other
         // tests using these functions.
@@ -56,6 +57,7 @@ class subscriptions_test extends \advanced_testcase {
         // tests using these functions.
         \mod_forum\subscriptions::reset_forum_cache();
         \mod_forum\subscriptions::reset_discussion_cache();
+        parent::tearDown();
     }
 
     /**
@@ -63,7 +65,7 @@ class subscriptions_test extends \advanced_testcase {
      *
      * @covers \mod_forum\event\subscription_mode_updated
      */
-    public function test_subscription_modes() {
+    public function test_subscription_modes(): void {
         global $DB;
 
         $this->resetAfterTest(true);
@@ -149,7 +151,7 @@ class subscriptions_test extends \advanced_testcase {
     /**
      * Test fetching unsubscribable forums.
      */
-    public function test_unsubscribable_forums() {
+    public function test_unsubscribable_forums(): void {
         global $DB;
 
         $this->resetAfterTest(true);
@@ -210,7 +212,7 @@ class subscriptions_test extends \advanced_testcase {
      * Test that toggling the forum-level subscription for a different user does not affect their discussion-level
      * subscriptions.
      */
-    public function test_forum_subscribe_toggle_as_other() {
+    public function test_forum_subscribe_toggle_as_other(): void {
         global $DB;
 
         $this->resetAfterTest(true);
@@ -399,7 +401,7 @@ class subscriptions_test extends \advanced_testcase {
     /**
      * Test that a user unsubscribed from a forum is not subscribed to it's discussions by default.
      */
-    public function test_forum_discussion_subscription_forum_unsubscribed() {
+    public function test_forum_discussion_subscription_forum_unsubscribed(): void {
         $this->resetAfterTest(true);
 
         // Create a course, with a forum.
@@ -424,7 +426,7 @@ class subscriptions_test extends \advanced_testcase {
     /**
      * Test that the act of subscribing to a forum subscribes the user to it's discussions by default.
      */
-    public function test_forum_discussion_subscription_forum_subscribed() {
+    public function test_forum_discussion_subscription_forum_subscribed(): void {
         $this->resetAfterTest(true);
 
         // Create a course, with a forum.
@@ -456,7 +458,7 @@ class subscriptions_test extends \advanced_testcase {
     /**
      * Test that a user unsubscribed from a forum can be subscribed to a discussion.
      */
-    public function test_forum_discussion_subscription_forum_unsubscribed_discussion_subscribed() {
+    public function test_forum_discussion_subscription_forum_unsubscribed_discussion_subscribed(): void {
         $this->resetAfterTest(true);
 
         // Create a course, with a forum.
@@ -490,7 +492,7 @@ class subscriptions_test extends \advanced_testcase {
     /**
      * Test that a user subscribed to a forum can be unsubscribed from a discussion.
      */
-    public function test_forum_discussion_subscription_forum_subscribed_discussion_unsubscribed() {
+    public function test_forum_discussion_subscription_forum_subscribed_discussion_unsubscribed(): void {
         $this->resetAfterTest(true);
 
         // Create a course, with a forum.
@@ -524,7 +526,7 @@ class subscriptions_test extends \advanced_testcase {
     /**
      * Test the effect of toggling the discussion subscription status when subscribed to the forum.
      */
-    public function test_forum_discussion_toggle_forum_subscribed() {
+    public function test_forum_discussion_toggle_forum_subscribed(): void {
         global $DB;
 
         $this->resetAfterTest(true);
@@ -704,7 +706,7 @@ class subscriptions_test extends \advanced_testcase {
     /**
      * Test the effect of toggling the discussion subscription status when unsubscribed from the forum.
      */
-    public function test_forum_discussion_toggle_forum_unsubscribed() {
+    public function test_forum_discussion_toggle_forum_unsubscribed(): void {
         global $DB;
 
         $this->resetAfterTest(true);
@@ -795,7 +797,7 @@ class subscriptions_test extends \advanced_testcase {
      * Test that the correct users are returned when fetching subscribed users from a forum where users can choose to
      * subscribe and unsubscribe.
      */
-    public function test_fetch_subscribed_users_subscriptions() {
+    public function test_fetch_subscribed_users_subscriptions(): void {
         global $DB, $CFG;
 
         $this->resetAfterTest(true);
@@ -833,7 +835,7 @@ class subscriptions_test extends \advanced_testcase {
      * Test that the correct users are returned hwen fetching subscribed users from a forum where users are forcibly
      * subscribed.
      */
-    public function test_fetch_subscribed_users_forced() {
+    public function test_fetch_subscribed_users_forced(): void {
         global $DB;
 
         $this->resetAfterTest(true);
@@ -855,7 +857,7 @@ class subscriptions_test extends \advanced_testcase {
     /**
      * Test that unusual combinations of discussion subscriptions do not affect the subscribed user list.
      */
-    public function test_fetch_subscribed_users_discussion_subscriptions() {
+    public function test_fetch_subscribed_users_discussion_subscriptions(): void {
         global $DB;
 
         $this->resetAfterTest(true);
@@ -927,7 +929,7 @@ class subscriptions_test extends \advanced_testcase {
     /**
      * Test whether a user is force-subscribed to a forum.
      */
-    public function test_force_subscribed_to_forum() {
+    public function test_force_subscribed_to_forum(): void {
         global $DB;
 
         $this->resetAfterTest(true);
@@ -959,7 +961,7 @@ class subscriptions_test extends \advanced_testcase {
     /**
      * Test that the subscription cache can be pre-filled.
      */
-    public function test_subscription_cache_prefill() {
+    public function test_subscription_cache_prefill(): void {
         global $DB;
 
         $this->resetAfterTest(true);
@@ -994,7 +996,7 @@ class subscriptions_test extends \advanced_testcase {
     /**
      * Test that the subscription cache can filled user-at-a-time.
      */
-    public function test_subscription_cache_fill() {
+    public function test_subscription_cache_fill(): void {
         global $DB;
 
         $this->resetAfterTest(true);
@@ -1025,7 +1027,7 @@ class subscriptions_test extends \advanced_testcase {
     /**
      * Test that the discussion subscription cache can filled course-at-a-time.
      */
-    public function test_discussion_subscription_cache_fill_for_course() {
+    public function test_discussion_subscription_cache_fill_for_course(): void {
         global $DB;
 
         $this->resetAfterTest(true);
@@ -1073,7 +1075,7 @@ class subscriptions_test extends \advanced_testcase {
     /**
      * Test that the discussion subscription cache can be forcibly updated for a user.
      */
-    public function test_discussion_subscription_cache_prefill() {
+    public function test_discussion_subscription_cache_prefill(): void {
         global $DB;
 
         $this->resetAfterTest(true);
@@ -1161,7 +1163,7 @@ class subscriptions_test extends \advanced_testcase {
     /**
      * Test that the discussion subscription cache can filled user-at-a-time.
      */
-    public function test_discussion_subscription_cache_fill() {
+    public function test_discussion_subscription_cache_fill(): void {
         global $DB;
 
         $this->resetAfterTest(true);
@@ -1220,7 +1222,7 @@ class subscriptions_test extends \advanced_testcase {
      * Test that after toggling the forum subscription as another user,
      * the discussion subscription functionality works as expected.
      */
-    public function test_forum_subscribe_toggle_as_other_repeat_subscriptions() {
+    public function test_forum_subscribe_toggle_as_other_repeat_subscriptions(): void {
         global $DB;
 
         $this->resetAfterTest(true);
@@ -1308,7 +1310,7 @@ class subscriptions_test extends \advanced_testcase {
      * Test that providing a context_module instance to is_subscribed does not result in additional lookups to retrieve
      * the context_module.
      */
-    public function test_is_subscribed_cm() {
+    public function test_is_subscribed_cm(): void {
         global $DB;
 
         $this->resetAfterTest(true);
@@ -1353,7 +1355,7 @@ class subscriptions_test extends \advanced_testcase {
         $this->assertGreaterThan($suppliedcmcount, $calculatedcmcount);
     }
 
-    public function is_subscribable_forums() {
+    public static function is_subscribable_forums(): array {
         return [
             [
                 'forcesubscribe' => FORUM_DISALLOWSUBSCRIBE,
@@ -1370,9 +1372,9 @@ class subscriptions_test extends \advanced_testcase {
         ];
     }
 
-    public function is_subscribable_provider() {
+    public static function is_subscribable_provider(): array {
         $data = [];
-        foreach ($this->is_subscribable_forums() as $forum) {
+        foreach (self::is_subscribable_forums() as $forum) {
             $data[] = [$forum];
         }
 
@@ -1382,7 +1384,7 @@ class subscriptions_test extends \advanced_testcase {
     /**
      * @dataProvider is_subscribable_provider
      */
-    public function test_is_subscribable_logged_out($options) {
+    public function test_is_subscribable_logged_out($options): void {
         $this->resetAfterTest(true);
 
         // Create a course, with a forum.
@@ -1396,7 +1398,7 @@ class subscriptions_test extends \advanced_testcase {
     /**
      * @dataProvider is_subscribable_provider
      */
-    public function test_is_subscribable_is_guest($options) {
+    public function test_is_subscribable_is_guest($options): void {
         global $DB;
         $this->resetAfterTest(true);
 
@@ -1411,7 +1413,7 @@ class subscriptions_test extends \advanced_testcase {
         $this->assertFalse(\mod_forum\subscriptions::is_subscribable($forum));
     }
 
-    public function is_subscribable_loggedin_provider() {
+    public static function is_subscribable_loggedin_provider(): array {
         return [
             [
                 ['forcesubscribe' => FORUM_DISALLOWSUBSCRIBE],
@@ -1435,7 +1437,7 @@ class subscriptions_test extends \advanced_testcase {
     /**
      * @dataProvider is_subscribable_loggedin_provider
      */
-    public function test_is_subscribable_loggedin($options, $expect) {
+    public function test_is_subscribable_loggedin($options, $expect): void {
         $this->resetAfterTest(true);
 
         // Create a course, with a forum.
@@ -1450,7 +1452,7 @@ class subscriptions_test extends \advanced_testcase {
         $this->assertEquals($expect, \mod_forum\subscriptions::is_subscribable($forum));
     }
 
-    public function test_get_user_default_subscription() {
+    public function test_get_user_default_subscription(): void {
         global $DB;
         $this->resetAfterTest(true);
 
