@@ -25,10 +25,10 @@
 namespace mod_quiz\output;
 
 use core_question\local\bank\question_version_status;
-use mod_quiz\question\bank\qbank_helper;
 use \mod_quiz\structure;
 use \html_writer;
 use qbank_previewquestion\question_preview_options;
+use question_bank;
 use renderable;
 
 /**
@@ -53,8 +53,14 @@ class edit_renderer extends \plugin_renderer_base {
      * @param array $pagevars the variables from {@link question_edit_setup()}.
      * @return string HTML to output.
      */
-    public function edit_page(\mod_quiz\quiz_settings $quizobj, structure $structure,
-        \core_question\local\bank\question_edit_contexts $contexts, \moodle_url $pageurl, array $pagevars) {
+    public function edit_page(
+        \mod_quiz\quiz_settings $quizobj,
+        structure $structure,
+        \core_question\local\bank\question_edit_contexts $contexts,
+        \moodle_url $pageurl,
+        array $pagevars,
+    ) {
+
         $output = '';
 
         // Page title.
@@ -66,7 +72,7 @@ class edit_renderer extends \plugin_renderer_base {
         $output .= html_writer::start_div('mod_quiz-edit-top-controls');
 
         $output .= html_writer::start_div('d-flex justify-content-between flex-wrap mb-1');
-        $output .= html_writer::start_div('d-flex flex-column justify-content-around');
+        $output .= html_writer::start_div('d-flex align-items-center justify-content-around');
         $output .= $this->quiz_information($structure);
         $output .= html_writer::end_tag('div');
         $output .= $this->maximum_grade_input($structure, $pageurl);
@@ -112,11 +118,14 @@ class edit_renderer extends \plugin_renderer_base {
         if ($structure->can_be_edited()) {
             $thiscontext = $contexts->lowest();
             $this->page->requires->js_call_amd('mod_quiz/modal_quiz_question_bank', 'init', [
-                $thiscontext->id
+                $thiscontext->id,
+                $quizobj->get_cm()->id,
+                $quizobj->get_cm()->id,
             ]);
 
             $this->page->requires->js_call_amd('mod_quiz/modal_add_random_question', 'init', [
                 $thiscontext->id,
+                $quizobj->get_cm()->id,
                 $pagevars['cat'],
                 $pageurl->out_as_local_url(true),
                 $pageurl->param('cmid'),
@@ -179,19 +188,19 @@ class edit_renderer extends \plugin_renderer_base {
      */
     public function maximum_grade_input($structure, \moodle_url $pageurl) {
         $output = '';
-        $output .= html_writer::start_div('maxgrade');
+        $output .= html_writer::start_div('maxgrade', ['class' => 'mt-2 mt-sm-0']);
         $output .= html_writer::start_tag('form', ['method' => 'post', 'action' => 'edit.php',
-                'class' => 'quizsavegradesform form-inline']);
-        $output .= html_writer::start_tag('fieldset', ['class' => 'invisiblefieldset']);
+                'class' => 'quizsavegradesform']);
+        $output .= html_writer::start_tag('fieldset', ['class' => 'invisiblefieldset d-flex align-items-center']);
         $output .= html_writer::empty_tag('input', ['type' => 'hidden', 'name' => 'sesskey', 'value' => sesskey()]);
         $output .= html_writer::input_hidden_params($pageurl);
         $output .= html_writer::tag('label', get_string('maximumgrade') . ' ',
-                ['for' => 'inputmaxgrade']);
+                ['for' => 'inputmaxgrade', 'class' => 'd-inline-block w-auto mb-0']);
         $output .= html_writer::empty_tag('input', ['type' => 'text', 'id' => 'inputmaxgrade',
                 'name' => 'maxgrade', 'size' => ($structure->get_decimal_places_for_grades() + 2),
                 'value' => $structure->formatted_quiz_grade(),
-                'class' => 'form-control']);
-        $output .= html_writer::empty_tag('input', ['type' => 'submit', 'class' => 'btn btn-secondary ml-1',
+                'class' => 'form-control d-inline-block align-middle w-auto ms-1']);
+        $output .= html_writer::empty_tag('input', ['type' => 'submit', 'class' => 'btn btn-secondary ms-1 d-inline-block w-auto ',
                 'name' => 'savechanges', 'value' => get_string('save', 'quiz')]);
         $output .= html_writer::end_tag('fieldset');
         $output .= html_writer::end_tag('form');
@@ -214,7 +223,7 @@ class edit_renderer extends \plugin_renderer_base {
             'name'  => 'repaginate',
             'id'    => 'repaginatecommand',
             'value' => get_string('repaginatecommand', 'quiz'),
-            'class' => 'btn btn-secondary mr-1',
+            'class' => 'btn btn-secondary me-1',
             'data-header' => $header,
             'data-form'   => $form,
         ];
@@ -327,13 +336,13 @@ class edit_renderer extends \plugin_renderer_base {
         $hiddenurl->param('sesskey', sesskey());
 
         $select = html_writer::select($perpage, 'questionsperpage',
-                $structure->get_questions_per_page(), false, ['class' => 'custom-select']);
+                $structure->get_questions_per_page(), false, ['class' => 'form-select']);
 
         $buttonattributes = [
             'type' => 'submit',
             'name' => 'repaginate',
             'value' => get_string('go'),
-            'class' => 'btn btn-secondary ml-1'
+            'class' => 'btn btn-secondary ms-1'
         ];
 
         $formcontent = html_writer::tag('form', html_writer::div(
@@ -402,9 +411,9 @@ class edit_renderer extends \plugin_renderer_base {
             $sectionheadingtext = format_string($section->heading);
             $sectionheading = html_writer::span($sectionheadingtext, 'instancesection');
         } else {
-            // Use a sr-only default section heading, so we don't end up with an empty section heading.
+            // Use a visually-hidden default section heading, so we don't end up with an empty section heading.
             $sectionheadingtext = get_string('sectionnoname', 'quiz');
-            $sectionheading = html_writer::span($sectionheadingtext, 'instancesection sr-only');
+            $sectionheading = html_writer::span($sectionheadingtext, 'instancesection visually-hidden');
         }
 
         $output .= html_writer::start_tag('li', ['id' => 'section-'.$section->id,
@@ -521,6 +530,8 @@ class edit_renderer extends \plugin_renderer_base {
         foreach ($structure->get_slots_in_section($section->id) as $slot) {
             $output .= $this->question_row($structure, $slot, $contexts, $pagevars, $pageurl);
         }
+
+        $this->page->requires->js_call_amd('mod_quiz/question_slot', 'init');
         return html_writer::tag('ul', $output, ['class' => 'section img-text']);
     }
 
@@ -743,8 +754,23 @@ class edit_renderer extends \plugin_renderer_base {
      * @return string HTML to output.
      */
     public function question(structure $structure, int $slot, \moodle_url $pageurl) {
+        global $DB;
+
         // Get the data required by the question_slot template.
         $slotid = $structure->get_slot_id_for_slot($slot);
+        $question = $structure->get_question_in_slot($slot);
+        $bank = $structure->get_source_bank($slot);
+
+        if ($bank?->issharedbank) {
+            $bankurl = (new \moodle_url('/question/edit.php',
+                [
+                    'cmid' => $bank->cminfo->id,
+                    'cat' => "{$question->category},{$question->contextid}",
+                ]
+            ))->out(false);
+        } else {
+            $bankurl = '';
+        }
 
         $output = '';
         $output .= html_writer::start_tag('div');
@@ -770,13 +796,15 @@ class edit_renderer extends \plugin_renderer_base {
             'questiondependencyicon' => ($structure->can_be_edited() ? $this->question_dependency_icon($structure, $slot) : ''),
             'versionselection' => false,
             'draftversion' => $structure->get_question_in_slot($slot)->status == question_version_status::QUESTION_STATUS_DRAFT,
+            'bankname' => $bank?->cminfo->get_formatted_name(),
+            'issharedbank' => $bank?->issharedbank,
+            'bankurl' => $bankurl,
         ];
 
         $data['versionoptions'] = [];
         if ($structure->get_slot_by_number($slot)->qtype !== 'random') {
             $data['versionselection'] = true;
             $data['versionoption'] = $structure->get_version_choices_for_slot($slot);
-            $this->page->requires->js_call_amd('mod_quiz/question_slot', 'init', [$slotid]);
         }
 
         // Render the question slot template.
@@ -794,7 +822,7 @@ class edit_renderer extends \plugin_renderer_base {
      * @param int $slot the slot on the page we are outputting.
      * @return string HTML to output.
      */
-    public function get_checkbox_render(structure $structure, int $slot) : string {
+    public function get_checkbox_render(structure $structure, int $slot): string {
         $questionslot = $structure->get_displayed_number_for_slot($slot);
         $checkbox = new \core\output\checkbox_toggleall($this->togglegroup, false,
             [
@@ -802,7 +830,7 @@ class edit_renderer extends \plugin_renderer_base {
                 'name' => 'selectquestion[]',
                 'classes' => 'select-multiple-checkbox',
                 'label' => get_string('selectquestionslot', 'quiz', $questionslot),
-                'labelclasses' => 'sr-only',
+                'labelclasses' => 'visually-hidden',
             ]);
 
         return $this->render($checkbox);
@@ -816,7 +844,7 @@ class edit_renderer extends \plugin_renderer_base {
      * @param \moodle_url $pageurl the canonical URL of this page.
      * @return string HTML to output.
      */
-    public function get_question_name_for_slot(structure $structure, int $slot, \moodle_url $pageurl) : string {
+    public function get_question_name_for_slot(structure $structure, int $slot, \moodle_url $pageurl): string {
         // Display the link to the question (or do nothing if question has no url).
         if ($structure->get_question_type_for_slot($slot) === 'random') {
             $questionname = $this->random_question($structure, $slot, $pageurl);
@@ -835,12 +863,12 @@ class edit_renderer extends \plugin_renderer_base {
      * @param \moodle_url $pageurl the canonical URL of this page.
      * @return string HTML to output.
      */
-    public function get_action_icon(structure $structure, int $slot, \moodle_url $pageurl) : string {
+    public function get_action_icon(structure $structure, int $slot, \moodle_url $pageurl): string {
         // Action icons.
         $qtype = $structure->get_question_type_for_slot($slot);
         $slotinfo = $structure->get_slot_by_number($slot);
         $questionicons = '';
-        if ($qtype !== 'random') {
+        if ($qtype !== 'random' && question_bank::is_qtype_usable($qtype)) {
             $questionicons .= $this->question_preview_icon($structure->get_quiz(),
                     $structure->get_question_in_slot($slot),
                     null, null, $slotinfo->requestedversion ?: question_preview_options::ALWAYS_LATEST);
@@ -896,6 +924,11 @@ class edit_renderer extends \plugin_renderer_base {
      */
     public function question_preview_icon($quiz, $questiondata, $label = null, $variant = null, $restartversion = null) {
         $question = clone($questiondata);
+
+        if (!question_bank::is_qtype_usable($question->qtype)) {
+            return '';
+        }
+
         if (isset($question->questionid)) {
 
             $question->id = $question->questionid;
@@ -1030,21 +1063,32 @@ class edit_renderer extends \plugin_renderer_base {
 
         $instancename = quiz_question_tostring($question);
 
-        $qtype = \question_bank::get_qtype($question->qtype, false);
+        $qtype = question_bank::get_qtype($question->qtype, false);
         $namestr = $qtype->local_name();
 
         $icon = $this->pix_icon('icon', $namestr, $qtype->plugin_name(), ['title' => $namestr,
-                'class' => 'activityicon', 'alt' => ' ', 'role' => 'presentation']);
+                'class' => 'activityicon', 'alt' => $namestr]);
 
         $editicon = $this->pix_icon('t/edit', '', 'moodle', ['title' => '']);
 
         // Need plain question name without html tags for link title.
         $title = shorten_text(format_string($question->name), 100);
 
-        // Display the link itself.
-        $activitylink = $icon . html_writer::tag('span', $editicon . $instancename, ['class' => 'instancename']);
-        $output .= html_writer::link($editurl, $activitylink,
-                ['title' => get_string('editquestion', 'quiz').' '.$title]);
+        // If the question is invalid, don't show the link as it won't work.
+        if (!question_bank::is_qtype_usable($question->qtype)) {
+            $output .= html_writer::span($title);
+            $output .= html_writer::span(
+                get_string('invalidquestiontype', 'question', $question->originalqtype),
+                'badge bg-danger text-white ms-3'
+            );
+        } else {
+
+            // Display the link itself.
+            $activitylink = $icon . html_writer::tag('span', $editicon . $instancename, ['class' => 'instancename']);
+            $output .= html_writer::link($editurl, $activitylink,
+                ['title' => get_string('editquestion', 'quiz') . ' ' . $title]);
+
+        }
 
         return $output;
     }
@@ -1060,36 +1104,34 @@ class edit_renderer extends \plugin_renderer_base {
      */
     public function random_question(structure $structure, $slotnumber, $pageurl) {
         $question = $structure->get_question_in_slot($slotnumber);
+        $bankcontext = \context::instance_by_id($question->contextid);
         $slot = $structure->get_slot_by_number($slotnumber);
         $editurl = new \moodle_url('/mod/quiz/editrandom.php',
-                ['returnurl' => $pageurl->out_as_local_url(), 'slotid' => $slot->id]);
+                ['returnurl' => $pageurl->out_as_local_url(), 'slotid' => $slot->id, 'bankcmid' => $bankcontext->instanceid]);
 
         $temp = clone($question);
         $temp->questiontext = '';
-        $temp->name = qbank_helper::describe_random_question($slot);
+        $temp->name = $structure->describe_random_slot($slot->id);
         $instancename = quiz_question_tostring($temp);
+        if (strpos($instancename, structure::MISSING_QUESTION_CATEGORY_PLACEHOLDER) !== false) {
+            $label = html_writer::span(
+                get_string('missingcategory', 'mod_quiz'),
+                'badge bg-danger text-white h-50'
+            );
+            $instancename = str_replace(structure::MISSING_QUESTION_CATEGORY_PLACEHOLDER, $label, $instancename);
+        }
 
         $configuretitle = get_string('configurerandomquestion', 'quiz');
-        $qtype = \question_bank::get_qtype($question->qtype, false);
+        $qtype = question_bank::get_qtype($question->qtype, false);
         $namestr = $qtype->local_name();
-        $icon = $this->pix_icon('icon', $namestr, $qtype->plugin_name(), ['title' => $namestr,
-                'class' => 'icon activityicon', 'alt' => ' ', 'role' => 'presentation']);
+        $icon = $this->pix_icon('icon', $namestr, $qtype->plugin_name(), ['class' => 'icon activityicon']);
 
         $editicon = $this->pix_icon('t/edit', $configuretitle, 'moodle', ['title' => '']);
         $qbankurlparams = [
-            'cmid' => $structure->get_cmid(),
-            'cat' => $slot->category . ',' . $slot->contextid,
+            'cmid' => $structure->get_source_bank($slotnumber)->cminfo->id,
+            'filter' => json_encode($slot->filtercondition['filter']),
         ];
 
-        $slottags = [];
-        if (isset($slot->randomtags)) {
-            $slottags = $slot->randomtags;
-        }
-        foreach ($slottags as $index => $slottag) {
-            $slottag = explode(',', $slottag);
-            $qbankurlparams["qtagids[{$index}]"] = $slottag[0];
-        }
-
         // If this is a random question, display a link to show the questions
         // selected from in the question bank.
         $qbankurl = new \moodle_url('/question/edit.php', $qbankurlparams);
@@ -1234,8 +1276,6 @@ class edit_renderer extends \plugin_renderer_base {
                 'edittitleinstructions',
                 'emptydragdropregion',
                 'hide',
-                'markedthistopic',
-                'markthistopic',
                 'move',
                 'movecontent',
                 'moveleft',
@@ -1266,7 +1306,7 @@ class edit_renderer extends \plugin_renderer_base {
                 'questiondependsonprevious',
         ], 'quiz');
 
-        foreach (\question_bank::get_all_qtypes() as $qtype => $notused) {
+        foreach (question_bank::get_all_qtypes() as $qtype => $notused) {
             $this->page->requires->string_for_js('pluginname', 'qtype_' . $qtype);
         }
 
