@@ -25,7 +25,7 @@
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
 
-
+use core_question\local\bank\question_bank_helper;
 use qbank_managecategories\helper;
 
 defined('MOODLE_INTERNAL') || die();
@@ -254,7 +254,7 @@ function quiz_update_effective_access($quiz, $userid) {
 
     if (!empty($groupings[0])) {
         // Select all overrides that apply to the User's groups.
-        list($extra, $params) = $DB->get_in_or_equal(array_values($groupings[0]));
+        [$extra, $params] = $DB->get_in_or_equal(array_values($groupings[0]));
         $sql = "SELECT * FROM {quiz_overrides}
                 WHERE groupid $extra AND quiz = ?";
         $params[] = $quiz->id;
@@ -524,9 +524,10 @@ function quiz_get_user_attempts($quizids, $userid, $status = 'finished', $includ
             break;
 
         case 'finished':
-            $statuscondition = ' AND state IN (:state1, :state2)';
+            $statuscondition = ' AND state IN (:state1, :state2, :state3)';
             $params['state1'] = quiz_attempt::FINISHED;
             $params['state2'] = quiz_attempt::ABANDONED;
+            $params['state3'] = quiz_attempt::SUBMITTED;
             break;
 
         case 'unfinished':
@@ -537,7 +538,7 @@ function quiz_get_user_attempts($quizids, $userid, $status = 'finished', $includ
     }
 
     $quizids = (array) $quizids;
-    list($insql, $inparams) = $DB->get_in_or_equal($quizids, SQL_PARAMS_NAMED);
+    [$insql, $inparams] = $DB->get_in_or_equal($quizids, SQL_PARAMS_NAMED);
     $params += $inparams;
     $params['userid'] = $userid;
 
@@ -1415,6 +1416,7 @@ function quiz_questions_in_use($questionids) {
  */
 function quiz_reset_course_form_definition($mform) {
     $mform->addElement('header', 'quizheader', get_string('modulenameplural', 'quiz'));
+    $mform->addElement('static', 'quizdelete', get_string('delete'));
     $mform->addElement('advcheckbox', 'reset_quiz_attempts',
             get_string('removeallquizattempts', 'quiz'));
     $mform->addElement('advcheckbox', 'reset_quiz_user_overrides',
@@ -1482,7 +1484,7 @@ function quiz_reset_userdata($data) {
                 'quiz IN (SELECT id FROM {quiz} WHERE course = ?)', [$data->courseid]);
         $status[] = [
             'component' => $componentstr,
-            'item' => get_string('attemptsdeleted', 'quiz'),
+            'item' => get_string('removeallquizattempts', 'quiz'),
             'error' => false];
 
         // Remove all grades from gradebook.
@@ -1493,7 +1495,7 @@ function quiz_reset_userdata($data) {
         }
         $status[] = [
             'component' => $componentstr,
-            'item' => get_string('gradesdeleted', 'quiz'),
+            'item' => get_string('grades'),
             'error' => false];
     }
 
@@ -1505,7 +1507,7 @@ function quiz_reset_userdata($data) {
                 'quiz IN (SELECT id FROM {quiz} WHERE course = ?) AND userid IS NOT NULL', [$data->courseid]);
         $status[] = [
             'component' => $componentstr,
-            'item' => get_string('useroverridesdeleted', 'quiz'),
+            'item' => get_string('useroverrides', 'quiz'),
             'error' => false];
         $purgeoverrides = true;
     }
@@ -1515,7 +1517,7 @@ function quiz_reset_userdata($data) {
                 'quiz IN (SELECT id FROM {quiz} WHERE course = ?) AND groupid IS NOT NULL', [$data->courseid]);
         $status[] = [
             'component' => $componentstr,
-            'item' => get_string('groupoverridesdeleted', 'quiz'),
+            'item' => get_string('groupoverrides', 'quiz'),
             'error' => false];
         $purgeoverrides = true;
     }
@@ -1580,7 +1582,7 @@ function quiz_num_attempt_summary($quiz, $cm, $returnzero = false, $currentgroup
                         [$quiz->id, $currentgroup]);
                 return get_string('attemptsnumthisgroup', 'quiz', $a);
             } else if ($groups = groups_get_all_groups($cm->course, $USER->id, $cm->groupingid)) {
-                list($usql, $params) = $DB->get_in_or_equal(array_keys($groups));
+                [$usql, $params] = $DB->get_in_or_equal(array_keys($groups));
                 $a->group = $DB->count_records_sql('SELECT COUNT(DISTINCT qa.id) FROM ' .
                         '{quiz_attempts} qa JOIN ' .
                         '{groups_members} gm ON qa.userid = gm.userid ' .
@@ -1872,7 +1874,7 @@ function quiz_check_updates_since(cm_info $cm, $from, $filter = []) {
     $quizobj->preload_questions();
     $questionids = array_keys($quizobj->get_questions(null, false));
     if (!empty($questionids)) {
-        list($questionsql, $params) = $DB->get_in_or_equal($questionids, SQL_PARAMS_NAMED);
+        [$questionsql, $params] = $DB->get_in_or_equal($questionids, SQL_PARAMS_NAMED);
         $select = 'id ' . $questionsql . ' AND (timemodified > :time1 OR timecreated > :time2)';
         $params['time1'] = $from;
         $params['time2'] = $from;
@@ -1910,7 +1912,7 @@ function quiz_check_updates_since(cm_info $cm, $from, $filter = []) {
             if (empty($groupusers)) {
                 return $updates;
             }
-            list($insql, $inparams) = $DB->get_in_or_equal($groupusers);
+            [$insql, $inparams] = $DB->get_in_or_equal($groupusers);
             $select .= ' AND userid ' . $insql;
             $params = array_merge($params, $inparams);
         }
@@ -2328,15 +2330,23 @@ function mod_quiz_output_fragment_quiz_question_bank($args): string {
     $querystring = parse_url($args['querystring'], PHP_URL_QUERY);
     parse_str($querystring, $params);
 
+    // Load the bank we are looking at rather than always the quiz module itself.
+    $params['cmid'] = clean_param($args['bankcmid'], PARAM_INT);
+
     $viewclass = \mod_quiz\question\bank\custom_view::class;
     $extraparams['view'] = $viewclass;
 
+    // We need the quiz modid to POST back to.
+    $extraparams['quizcmid'] = clean_param($args['quizcmid'], PARAM_INT);
+
     // Build required parameters.
     [$contexts, $thispageurl, $cm, $pagevars, $extraparams] =
             build_required_parameters_for_custom_view($params, $extraparams);
 
     $course = get_course($cm->course);
-    require_capability('mod/quiz:manage', $contexts->lowest());
+    if (!has_capability('moodle/question:useall', $contexts->lowest())) {
+        require_capability('moodle/question:usemine', $contexts->lowest());
+    }
 
     // Custom View.
     $questionbank = new $viewclass($contexts, $thispageurl, $course, $cm, $pagevars, $extraparams);
@@ -2346,15 +2356,31 @@ function mod_quiz_output_fragment_quiz_question_bank($args): string {
     return $renderer->question_bank_contents($questionbank, $pagevars);
 }
 
+/**
+ * Build and return the output for the question bank and category chooser.
+ *
+ * @param array $args provided by the AJAX request.
+ * @return string html to render to the modal.
+ */
+function mod_quiz_output_fragment_switch_question_bank($args): string {
+    global $USER, $COURSE, $OUTPUT;
+
+    $quizcmid = clean_param($args['quizcmid'], PARAM_INT);
+
+    $switchbankwidget = new \core_question\output\switch_question_bank($quizcmid, $COURSE->id, $USER->id);
+
+    return $OUTPUT->render($switchbankwidget);
+}
+
 /**
  * Generates the add random question in a fragment output. This allows the
  * form to be rendered in javascript, for example inside a modal.
  *
  * The required arguments as keys in the $args array are:
- *      cat {string} The category and category context ids comma separated.
  *      addonpage {int} The page id to add this question to.
  *      returnurl {string} URL to return to after form submission.
- *      cmid {int} The course module id the questions are being added to.
+ *      quizcmid {int} The quiz course module id the questions are being added to.
+ *      bankcmid {int} The question bank course module id the questions are being added from.
  *
  * @param array $args The fragment arguments.
  * @return string The rendered mform fragment.
@@ -2363,6 +2389,8 @@ function mod_quiz_output_fragment_add_random_question_form($args) {
     global $PAGE, $OUTPUT;
 
     $extraparams = [];
+    $extraparams['quizcmid'] = clean_param($args['quizcmid'], PARAM_INT);
+    $extraparams['cmid'] = clean_param($args['bankcmid'], PARAM_INT);
 
     // Build required parameters.
     [$contexts, $thispageurl, $cm, $pagevars, $extraparams] =
@@ -2405,6 +2433,9 @@ function mod_quiz_output_fragment_add_random_question_form($args) {
         'questioncategoryoptions' => $catoptions,
     ];
 
+    $helpicon = new \help_icon('parentcategory', 'question');
+    $data['questioncategoryhelp'] = $helpicon->export_for_template($renderer);
+
     $result = $OUTPUT->render_from_template('mod_quiz/add_random_question_form', $data);
 
     return $result;
@@ -2434,24 +2465,24 @@ function mod_quiz_core_calendar_get_event_action_string(string $eventtype): stri
 }
 
 /**
- * Delete question reference data.
+ * Delete all question references for a quiz.
  *
  * @param int $quizid The id of quiz.
  */
 function quiz_delete_references($quizid): void {
     global $DB;
-    $slots = $DB->get_records('quiz_slots', ['quizid' => $quizid]);
-    foreach ($slots as $slot) {
-        $params = [
-            'itemid' => $slot->id,
-            'component' => 'mod_quiz',
-            'questionarea' => 'slot'
-        ];
-        // Delete any set references.
-        $DB->delete_records('question_set_references', $params);
-        // Delete any references.
-        $DB->delete_records('question_references', $params);
-    }
+
+    $cm = get_coursemodule_from_instance('quiz', $quizid);
+    $context = context_module::instance($cm->id);
+
+    $conditions = [
+        'usingcontextid' => $context->id,
+        'component' => 'mod_quiz',
+        'questionarea' => 'slot',
+    ];
+
+    $DB->delete_records('question_references', $conditions);
+    $DB->delete_records('question_set_references', $conditions);
 }
 
 /**
@@ -2474,12 +2505,14 @@ function mod_quiz_output_fragment_question_data(array $args): string {
     $thispageurl = new \moodle_url('/mod/quiz/edit.php', ['cmid' => $cmid]);
     $thiscontext = \context_module::instance($cmid);
     $contexts = new \core_question\local\bank\question_edit_contexts($thiscontext);
-    $defaultcategory = question_make_default_categories($contexts->all());
+    $defaultcategory = question_get_default_category($contexts->lowest()->id, true);
     $params['cat'] = implode(',', [$defaultcategory->id, $defaultcategory->contextid]);
 
     $course = get_course($params['courseid']);
-    [, $cm] = get_module_from_cmid($cmid);
+    // The viewing bank mod id.
+    [, $cm] = get_module_from_cmid(clean_param($args['cmid'], PARAM_INT));
     $params['tabname'] = 'questions';
+    $extraparams['quizcmid'] = clean_param($args['quizcmid'], PARAM_INT);
 
     // Custom question bank View.
     $viewclass = clean_param($args['view'], PARAM_NOTAGS);
@@ -2513,6 +2546,8 @@ function build_required_parameters_for_custom_view(array $params, array $extrapa
     // Add cmid so we can retrieve later in extra params.
     $extraparams['cmid'] = $cmid;
 
+    $extraparams['requirebankswitch'] = !empty(question_bank_helper::get_activity_types_with_shareable_questions());
+
     return [$contexts, $thispageurl, $cm, $pagevars, $extraparams];
 }
 
