@@ -35,13 +35,19 @@ require_once($CFG->libdir . '/completionlib.php');
 require_once($CFG->libdir . '/filelib.php');
 require_once($CFG->libdir . '/questionlib.php');
 
+use core\di;
+use core\hook;
+use core_question\local\bank\condition;
 use mod_quiz\access_manager;
 use mod_quiz\event\attempt_submitted;
 use mod_quiz\grade_calculator;
+use mod_quiz\hook\attempt_state_changed;
+use mod_quiz\local\override_manager;
 use mod_quiz\question\bank\qbank_helper;
 use mod_quiz\question\display_options;
 use mod_quiz\quiz_attempt;
 use mod_quiz\quiz_settings;
+use mod_quiz\structure;
 use qbank_previewquestion\question_preview_options;
 
 /**
@@ -122,27 +128,19 @@ function quiz_create_attempt(quiz_settings $quizobj, $attemptnumber, $lastattemp
     }
 
     $attempt->attempt = $attemptnumber;
-    $attempt->timestart = $timenow;
     $attempt->timefinish = 0;
     $attempt->timemodified = $timenow;
     $attempt->timemodifiedoffline = 0;
-    $attempt->state = quiz_attempt::IN_PROGRESS;
     $attempt->currentpage = 0;
     $attempt->sumgrades = null;
     $attempt->gradednotificationsenttime = null;
+    $attempt->timecheckstate = null;
 
     // If this is a preview, mark it as such.
     if ($ispreview) {
         $attempt->preview = 1;
     }
 
-    $timeclose = $quizobj->get_access_manager($timenow)->get_end_time($attempt);
-    if ($timeclose === false || $ispreview) {
-        $attempt->timecheckstate = null;
-    } else {
-        $attempt->timecheckstate = $timeclose;
-    }
-
     return $attempt;
 }
 /**
@@ -258,7 +256,7 @@ function quiz_start_new_attempt($quizobj, $quba, $attempt, $attemptnumber, $time
             $forcedvariantsbyseed, $variantstrategy);
     }
 
-    $quba->start_all_questions($variantstrategy, $timenow, $attempt->userid);
+    $quba->start_all_questions($variantstrategy, question_attempt_step::TIMECREATED_ON_FIRST_RENDER, $attempt->userid);
 
     // Work out the attempt layout.
     $sections = $quizobj->get_sections();
@@ -346,19 +344,50 @@ function quiz_start_attempt_built_on_last($quba, $attempt, $lastattempt) {
 }
 
 /**
- * The save started question usage and quiz attempt in db and log the started attempt.
+ * Create or update the quiz attempt record, and the question usage.
+ *
+ * If the attempt already exists in the database with the NOT_STARTED state, it will be transitioned
+ * to IN_PROGRESS and the timestart updated. If it does not already exist, a new record will be created
+ * already in the IN_PROGRESS state.
  *
  * @param quiz_settings $quizobj
  * @param question_usage_by_activity $quba
  * @param stdClass                     $attempt
+ * @param ?int $timenow The time to use for the attempt's timestart property. Defaults to time().
  * @return stdClass                    attempt object with uniqueid and id set.
  */
-function quiz_attempt_save_started($quizobj, $quba, $attempt) {
+function quiz_attempt_save_started(
+    quiz_settings $quizobj,
+    question_usage_by_activity $quba,
+    \stdClass $attempt,
+    ?int $timenow = null,
+): stdClass {
     global $DB;
-    // Save the attempt in the database.
-    question_engine::save_questions_usage_by_activity($quba);
-    $attempt->uniqueid = $quba->get_id();
-    $attempt->id = $DB->insert_record('quiz_attempts', $attempt);
+
+    $attempt->timestart = $timenow ?? time();
+
+    $timeclose = $quizobj->get_access_manager($attempt->timestart)->get_end_time($attempt);
+    if ($timeclose === false || $attempt->preview) {
+        $attempt->timecheckstate = null;
+    } else {
+        $attempt->timecheckstate = $timeclose;
+    }
+
+    $originalattempt = null;
+    if (isset($attempt->id) && $attempt->state === quiz_attempt::NOT_STARTED) {
+        $originalattempt = clone $attempt;
+        // In case questions have been edited since attempts were pre-created, update questions now.
+        quiz_attempt::create($attempt->id)->update_questions_to_new_version_if_changed();
+        // Update the attempt's state.
+        $attempt->state = quiz_attempt::IN_PROGRESS;
+        $DB->update_record('quiz_attempts', $attempt);
+    } else {
+        // Save the attempt in the database.
+        question_engine::save_questions_usage_by_activity($quba);
+        $attempt->uniqueid = $quba->get_id();
+        $attempt->state = quiz_attempt::IN_PROGRESS;
+        $attempt->id = $DB->insert_record('quiz_attempts', $attempt);
+    }
 
     // Params used by the events below.
     $params = [
@@ -383,6 +412,29 @@ function quiz_attempt_save_started($quizobj, $quba, $attempt) {
     $event->add_record_snapshot('quiz_attempts', $attempt);
     $event->trigger();
 
+    di::get(hook\manager::class)->dispatch(new attempt_state_changed($originalattempt, $attempt));
+
+    return $attempt;
+}
+
+/**
+ * Create the quiz attempt record, and the question usage.
+ *
+ * This saves an attempt in the NOT_STARTED state, and is designed for use when pre-creating attempts
+ * ahead of the quiz start time to spread out the processing load.
+ *
+ * @param question_usage_by_activity $quba
+ * @param stdClass $attempt
+ * @return stdClass attempt object with uniqueid and id set.
+ */
+function quiz_attempt_save_not_started(question_usage_by_activity $quba, stdClass $attempt): stdClass {
+    global $DB;
+    // Save the attempt in the database.
+    question_engine::save_questions_usage_by_activity($quba);
+    $attempt->uniqueid = $quba->get_id();
+    $attempt->state = quiz_attempt::NOT_STARTED;
+    $attempt->id = $DB->insert_record('quiz_attempts', $attempt);
+    di::get(hook\manager::class)->dispatch(new attempt_state_changed(null, $attempt));
     return $attempt;
 }
 
@@ -446,10 +498,14 @@ function quiz_delete_attempt($attempt, $quiz) {
         $event->add_record_snapshot('quiz_attempts', $attempt);
         $event->trigger();
 
+        // This class callback is deprecated, and will be removed in Moodle 4.8 (MDL-80327).
+        // Use the attempt_state_changed hook instead.
         $callbackclasses = \core_component::get_plugin_list_with_class('quiz', 'quiz_attempt_deleted');
         foreach ($callbackclasses as $callbackclass) {
-            component_class_callback($callbackclass, 'callback', [$quiz->id]);
+            component_class_callback($callbackclass, 'callback', [$quiz->id], null, true);
         }
+
+        di::get(hook\manager::class)->dispatch(new attempt_state_changed($attempt, null));
     }
 
     // Search quiz_attempts for other instances by this user.
@@ -788,7 +844,6 @@ function quiz_update_open_attempts(array $conditions) {
     * Each database handles updates with inner joins differently:
     *  - mysql does not allow a FROM clause
     *  - postgres and mssql allow FROM but handle table aliases differently
-    *  - oracle requires a subquery
     *
     * Different code for each database.
     */
@@ -815,15 +870,7 @@ function quiz_update_open_attempts(array $conditions) {
                         JOIN ( $quizausersql ) quizauser ON quizauser.id = quiza.id
                        WHERE $attemptselect";
     } else {
-        // oracle, sqlite and others
-        $updatesql = "UPDATE {quiz_attempts} quiza
-                         SET timecheckstate = (
-                           SELECT $timecheckstatesql
-                             FROM {quiz} quiz, ( $quizausersql ) quizauser
-                            WHERE quiz.id = quiza.quiz
-                              AND quizauser.id = quiza.id
-                         )
-                         WHERE $attemptselect";
+        throw new \core\exception\coding_exception("Unsupported database family: {$dbfamily}");
     }
 
     $DB->execute($updatesql, $params);
@@ -968,10 +1015,14 @@ function quiz_questions_per_page_options() {
  */
 function quiz_attempt_state_name($state) {
     switch ($state) {
+        case quiz_attempt::NOT_STARTED:
+            return get_string('statenotstarted', 'quiz');
         case quiz_attempt::IN_PROGRESS:
             return get_string('stateinprogress', 'quiz');
         case quiz_attempt::OVERDUE:
             return get_string('stateoverdue', 'quiz');
+        case quiz_attempt::SUBMITTED:
+            return get_string('statesubmitted', 'quiz');
         case quiz_attempt::FINISHED:
             return get_string('statefinished', 'quiz');
         case quiz_attempt::ABANDONED:
@@ -1379,23 +1430,21 @@ function quiz_send_notification_messages($course, $quiz, $attempt, $context, $cm
     $a = new stdClass();
     // Course info.
     $a->courseid        = $course->id;
-    $a->coursename      = $course->fullname;
-    $a->courseshortname = $course->shortname;
+    $a->coursename      = format_string($course->fullname, true, ['context' => $context]);
+    $a->courseshortname = format_string($course->shortname, true, ['context' => $context]);
     // Quiz info.
-    $a->quizname        = $quiz->name;
+    $a->quizname        = format_string($quiz->name, true, ['context' => $context]);
     $a->quizreporturl   = $CFG->wwwroot . '/mod/quiz/report.php?id=' . $cm->id;
-    $a->quizreportlink  = '<a href="' . $a->quizreporturl . '">' .
-            format_string($quiz->name) . ' report</a>';
+    $a->quizreportlink  = '<a href="' . $a->quizreporturl . '">' . $a->quizname . ' report</a>';
     $a->quizurl         = $CFG->wwwroot . '/mod/quiz/view.php?id=' . $cm->id;
-    $a->quizlink        = '<a href="' . $a->quizurl . '">' . format_string($quiz->name) . '</a>';
+    $a->quizlink        = '<a href="' . $a->quizurl . '">' . $a->quizname . '</a>';
     $a->quizid          = $quiz->id;
     $a->quizcmid        = $cm->id;
     // Attempt info.
     $a->submissiontime  = userdate($attempt->timefinish);
     $a->timetaken       = format_time($attempt->timefinish - $attempt->timestart);
     $a->quizreviewurl   = $CFG->wwwroot . '/mod/quiz/review.php?attempt=' . $attempt->id;
-    $a->quizreviewlink  = '<a href="' . $a->quizreviewurl . '">' .
-            format_string($quiz->name) . ' review</a>';
+    $a->quizreviewlink  = '<a href="' . $a->quizreviewurl . '">' . $a->quizname . ' review</a>';
     $a->attemptid       = $attempt->id;
     // Student who sat the quiz info.
     $a->studentidnumber = $submitter->idnumber;
@@ -1583,28 +1632,11 @@ function quiz_send_notify_manual_graded_message(quiz_attempt $attemptobj, object
  * @return void
  */
 function quiz_process_group_deleted_in_course($courseid) {
-    global $DB;
+    $affectedquizzes = override_manager::delete_orphaned_group_overrides_in_course($courseid);
 
-    // It would be nice if we got the groupid that was deleted.
-    // Instead, we just update all quizzes with orphaned group overrides.
-    $sql = "SELECT o.id, o.quiz, o.groupid
-              FROM {quiz_overrides} o
-              JOIN {quiz} quiz ON quiz.id = o.quiz
-         LEFT JOIN {groups} grp ON grp.id = o.groupid
-             WHERE quiz.course = :courseid
-               AND o.groupid IS NOT NULL
-               AND grp.id IS NULL";
-    $params = ['courseid' => $courseid];
-    $records = $DB->get_records_sql($sql, $params);
-    if (!$records) {
-        return; // Nothing to do.
+    if (!empty($affectedquizzes)) {
+        quiz_update_open_attempts(['quizid' => $affectedquizzes]);
     }
-    $DB->delete_records_list('quiz_overrides', 'id', array_keys($records));
-    $cache = cache::make('mod_quiz', 'overrides');
-    foreach ($records as $record) {
-        $cache->delete("{$record->quiz}_g_{$record->groupid}");
-    }
-    quiz_update_open_attempts(['quizid' => array_unique(array_column($records, 'quiz'))]);
 }
 
 /**
@@ -1642,10 +1674,11 @@ function quiz_get_js_module() {
  * @param bool $showidnumber If true, show the question's idnumber, if any. False by default.
  * @param core_tag_tag[]|bool $showtags if array passed, show those tags. Else, if true, get and show tags,
  *       else, don't show tags (which is the default).
+ * @param bool $displaytaglink Indicates whether the tag should be displayed as a link.
  * @return string HTML fragment.
  */
 function quiz_question_tostring($question, $showicon = false, $showquestiontext = true,
-        $showidnumber = false, $showtags = false) {
+        $showidnumber = false, $showtags = false, $displaytaglink = true) {
     global $OUTPUT;
     $result = '';
 
@@ -1660,7 +1693,7 @@ function quiz_question_tostring($question, $showicon = false, $showquestiontext
     if ($showidnumber && $question->idnumber !== null && $question->idnumber !== '') {
         $result .= ' ' . html_writer::span(
                 html_writer::span(get_string('idnumber', 'question'), 'accesshide') .
-                ' ' . s($question->idnumber), 'badge badge-primary');
+                ' ' . s($question->idnumber), 'badge bg-primary text-white');
     }
 
     // Question tags.
@@ -1672,7 +1705,7 @@ function quiz_question_tostring($question, $showicon = false, $showquestiontext
         $tags = [];
     }
     if ($tags) {
-        $result .= $OUTPUT->tag_list($tags, null, 'd-inline', 0, null, true);
+        $result .= $OUTPUT->tag_list($tags, null, 'd-inline', 0, null, true, $displaytaglink);
     }
 
     // Question text.
@@ -1729,6 +1762,12 @@ function quiz_add_quiz_question($questionid, $quiz, $page = 0, $maxmark = null)
         );
     }
 
+    // If the question type is invalid, we cannot add it to the quiz. It shouldn't be possible to get to this
+    // point without fiddling with the DOM so we can just throw an exception.
+    if (!\question_bank::is_qtype_installed($questiontype)) {
+        throw new coding_exception('Invalid question type: ' . $questiontype);
+    }
+
     $trans = $DB->start_delegated_transaction();
 
     $sql = "SELECT qbe.id
@@ -1904,8 +1943,16 @@ function quiz_add_random_questions(stdClass $quiz, int $addonpage, int $category
     );
 
     $settings = quiz_settings::create($quiz->id);
-    $structure = \mod_quiz\structure::create_for_quiz($settings);
-    $structure->add_random_questions($addonpage, $number, $categoryid);
+    $structure = structure::create_for_quiz($settings);
+    $structure->add_random_questions($addonpage, $number, [
+        'filter' => [
+            'category' => [
+                'jointype' => condition::JOINTYPE_DEFAULT,
+                'values' => [$categoryid],
+                'filteroptions' => ['includesubcategories' => false],
+            ],
+        ],
+    ]);
 }
 
 /**
@@ -1970,10 +2017,13 @@ function quiz_validate_new_attempt(quiz_settings $quizobj, access_manager $acces
     $lastattempt = end($attempts);
 
     $attemptnumber = null;
-    // If an in-progress attempt exists, check password then redirect to it.
-    if ($lastattempt && ($lastattempt->state == quiz_attempt::IN_PROGRESS ||
-            $lastattempt->state == quiz_attempt::OVERDUE)) {
+    if (
+        $lastattempt
+        && in_array($lastattempt->state, [quiz_attempt::NOT_STARTED, quiz_attempt::IN_PROGRESS, quiz_attempt::OVERDUE])
+    ) {
+        // If an in-progress or not-started attempt exists, check password then redirect to it.
         $currentattemptid = $lastattempt->id;
+
         $messages = $accessmanager->prevent_access();
 
         // If the attempt is now overdue, deal with that.
@@ -2048,15 +2098,33 @@ function quiz_prepare_and_start_new_attempt(quiz_settings $quizobj, $attemptnumb
     $quba = question_engine::make_questions_usage_by_activity('mod_quiz', $quizobj->get_context());
     $quba->set_preferred_behaviour($quizobj->get_quiz()->preferredbehaviour);
 
-    // Create the new attempt and initialize the question sessions
-    $timenow = time(); // Update time now, in case the server is running really slowly.
-    $attempt = quiz_create_attempt($quizobj, $attemptnumber, $lastattempt, $timenow, $ispreviewuser, $userid);
+    $attempt = $DB->get_record(
+        'quiz_attempts',
+        [
+            'quiz' => $quizobj->get_quizid(),
+            'userid' => $userid,
+            'preview' => 0,
+            'state' => quiz_attempt::NOT_STARTED,
+        ],
+    );
+    if (!$attempt) {
+        // Create the new attempt and initialize the question sessions.
+        $timenow = time(); // Update time now, in case the server is running really slowly.
+        $attempt = quiz_create_attempt($quizobj, $attemptnumber, $lastattempt, $timenow, $ispreviewuser, $userid);
 
-    if (!($quizobj->get_quiz()->attemptonlast && $lastattempt)) {
-        $attempt = quiz_start_new_attempt($quizobj, $quba, $attempt, $attemptnumber, $timenow,
-                $forcedrandomquestions, $forcedvariants);
-    } else {
-        $attempt = quiz_start_attempt_built_on_last($quba, $attempt, $lastattempt);
+        if (!($quizobj->get_quiz()->attemptonlast && $lastattempt)) {
+            $attempt = quiz_start_new_attempt(
+                $quizobj,
+                $quba,
+                $attempt,
+                $attemptnumber,
+                $timenow,
+                $forcedrandomquestions,
+                $forcedvariants,
+            );
+        } else {
+            $attempt = quiz_start_attempt_built_on_last($quba, $attempt, $lastattempt);
+        }
     }
 
     $transaction = $DB->start_delegated_transaction();
