@@ -36,7 +36,7 @@ require_once(__DIR__ . '/helpers.php');
  * @copyright  2012 The Open University
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
-class unitofwork_test extends \data_loading_method_test_base {
+final class unitofwork_test extends \data_loading_method_test_base {
     /** @var question_usage_by_activity the test question usage. */
     protected $quba;
 
@@ -47,6 +47,7 @@ class unitofwork_test extends \data_loading_method_test_base {
     protected $observer;
 
     protected function setUp(): void {
+        parent::setUp();
         // Create a usage in an initial state, with one shortanswer question added,
         // and attempted in interactive mode submitted responses 'toad' then 'frog'.
         // Then set it to use a new unit of work for any subsequent changes.
@@ -65,6 +66,7 @@ class unitofwork_test extends \data_loading_method_test_base {
 
     public function tearDown(): void {
         question_bank::end_unit_test();
+        parent::tearDown();
     }
 
     protected function setup_initial_test_state($testdata) {
@@ -95,7 +97,7 @@ class unitofwork_test extends \data_loading_method_test_base {
         );
     }
 
-    public function test_initial_state() {
+    public function test_initial_state(): void {
         $this->assertFalse($this->observer->get_modified());
         $this->assertEquals(0, count($this->observer->get_attempts_added()));
         $this->assertEquals(0, count($this->observer->get_attempts_modified()));
@@ -106,14 +108,14 @@ class unitofwork_test extends \data_loading_method_test_base {
         $this->assertEquals(0, count($this->observer->get_metadata_modified()));
     }
 
-    public function test_update_usage() {
+    public function test_update_usage(): void {
 
         $this->quba->set_preferred_behaviour('deferredfeedback');
 
         $this->assertTrue($this->observer->get_modified());
     }
 
-    public function test_add_question() {
+    public function test_add_question(): void {
 
         $slot = $this->quba->add_question(\test_question_maker::make_question('truefalse'));
 
@@ -126,7 +128,7 @@ class unitofwork_test extends \data_loading_method_test_base {
         $this->assertEquals(0, count($this->observer->get_metadata_modified()));
     }
 
-    public function test_add_and_start_question() {
+    public function test_add_and_start_question(): void {
 
         $slot = $this->quba->add_question(\test_question_maker::make_question('truefalse'));
                 $this->quba->start_question($slot);
@@ -145,7 +147,7 @@ class unitofwork_test extends \data_loading_method_test_base {
         $this->assertEquals(0, count($this->observer->get_metadata_modified()));
     }
 
-    public function test_process_action() {
+    public function test_process_action(): void {
 
         $this->quba->manual_grade($this->slot, 'Actually, that is not quite right', 0.5, FORMAT_HTML);
 
@@ -169,7 +171,7 @@ class unitofwork_test extends \data_loading_method_test_base {
         $this->assertEquals(0, count($this->observer->get_metadata_modified()));
     }
 
-    public function test_regrade_same_steps() {
+    public function test_regrade_same_steps(): void {
 
         // Change the question in a minor way and regrade.
         $this->quba->get_question($this->slot, false)->answers[14]->fraction = 0.5;
@@ -199,7 +201,7 @@ class unitofwork_test extends \data_loading_method_test_base {
         $this->assertEquals(0, count($this->observer->get_metadata_modified()));
     }
 
-    public function test_regrade_losing_steps() {
+    public function test_regrade_losing_steps(): void {
 
         // Change the question so that 'toad' is also right, and regrade. This
         // will mean that the try again, and second try states are no longer
@@ -238,7 +240,7 @@ class unitofwork_test extends \data_loading_method_test_base {
         $this->assertEquals(0, count($this->observer->get_metadata_modified()));
     }
 
-    public function test_tricky_regrade() {
+    public function test_tricky_regrade(): void {
 
         // The tricky thing here is that we take a half-complete question-attempt,
         // and then as one transaction, we submit some more responses, and then
@@ -279,7 +281,7 @@ class unitofwork_test extends \data_loading_method_test_base {
         $this->assertEquals(0, count($this->observer->get_metadata_modified()));
     }
 
-    public function test_move_question() {
+    public function test_move_question(): void {
 
         $q = \test_question_maker::make_question('truefalse');
         $newslot = $this->quba->add_question_in_place_of_other($this->slot, $q);
@@ -303,7 +305,7 @@ class unitofwork_test extends \data_loading_method_test_base {
         $this->assertEquals(0, count($this->observer->get_metadata_modified()));
     }
 
-    public function test_move_question_then_modify() {
+    public function test_move_question_then_modify(): void {
 
         $q = \test_question_maker::make_question('truefalse');
         $newslot = $this->quba->add_question_in_place_of_other($this->slot, $q);
@@ -333,7 +335,7 @@ class unitofwork_test extends \data_loading_method_test_base {
         $this->assertEquals(0, count($this->observer->get_metadata_modified()));
     }
 
-    public function test_move_question_then_move_again() {
+    public function test_move_question_then_move_again(): void {
         $originalqa = $this->quba->get_question_attempt($this->slot);
 
         $q1 = \test_question_maker::make_question('truefalse');
@@ -360,7 +362,7 @@ class unitofwork_test extends \data_loading_method_test_base {
         $this->assertEquals(0, count($this->observer->get_metadata_modified()));
     }
 
-    public function test_set_max_mark() {
+    public function test_set_max_mark(): void {
         $this->quba->set_max_mark($this->slot, 6.0);
         $this->assertEqualsWithDelta(4.0, $this->quba->get_total_mark(), 0.0000005);
 
@@ -379,7 +381,7 @@ class unitofwork_test extends \data_loading_method_test_base {
         $this->assertEquals(0, count($this->observer->get_metadata_modified()));
     }
 
-    public function test_set_question_attempt_metadata() {
+    public function test_set_question_attempt_metadata(): void {
         $this->quba->set_question_attempt_metadata($this->slot, 'metathingy', 'a value');
         $this->assertEquals('a value', $this->quba->get_question_attempt_metadata($this->slot, 'metathingy'));
 
@@ -395,7 +397,7 @@ class unitofwork_test extends \data_loading_method_test_base {
         $this->assertEquals(0, count($this->observer->get_metadata_modified()));
     }
 
-    public function test_set_question_attempt_metadata_then_change() {
+    public function test_set_question_attempt_metadata_then_change(): void {
         $this->quba->set_question_attempt_metadata($this->slot, 'metathingy', 'a value');
         $this->quba->set_question_attempt_metadata($this->slot, 'metathingy', 'different value');
         $this->assertEquals('different value', $this->quba->get_question_attempt_metadata($this->slot, 'metathingy'));
@@ -412,7 +414,7 @@ class unitofwork_test extends \data_loading_method_test_base {
         $this->assertEquals(0, count($this->observer->get_metadata_modified()));
     }
 
-    public function test_set_metadata_previously_set_but_dont_actually_change() {
+    public function test_set_metadata_previously_set_but_dont_actually_change(): void {
         $this->quba->set_question_attempt_metadata($this->slot, 'metathingy', 'a value');
         $this->observer = new testable_question_engine_unit_of_work($this->quba);
         $this->quba->set_observer($this->observer);
@@ -430,7 +432,7 @@ class unitofwork_test extends \data_loading_method_test_base {
         $this->assertEquals(0, count($this->observer->get_metadata_modified()));
     }
 
-    public function test_set_metadata_previously_set() {
+    public function test_set_metadata_previously_set(): void {
         $this->quba->set_question_attempt_metadata($this->slot, 'metathingy', 'a value');
         $this->observer = new testable_question_engine_unit_of_work($this->quba);
         $this->quba->set_observer($this->observer);
@@ -449,7 +451,7 @@ class unitofwork_test extends \data_loading_method_test_base {
                 $this->observer->get_metadata_modified());
     }
 
-    public function test_set_metadata_in_new_question() {
+    public function test_set_metadata_in_new_question(): void {
         $newslot = $this->quba->add_question(\test_question_maker::make_question('truefalse'));
         $this->quba->start_question($newslot);
         $this->quba->set_question_attempt_metadata($newslot, 'metathingy', 'a value');
@@ -467,7 +469,7 @@ class unitofwork_test extends \data_loading_method_test_base {
         $this->assertEquals(0, count($this->observer->get_metadata_modified()));
     }
 
-    public function test_set_metadata_then_move() {
+    public function test_set_metadata_then_move(): void {
         $this->quba->set_question_attempt_metadata($this->slot, 'metathingy', 'a value');
         $q = \test_question_maker::make_question('truefalse');
         $newslot = $this->quba->add_question_in_place_of_other($this->slot, $q);
@@ -488,7 +490,7 @@ class unitofwork_test extends \data_loading_method_test_base {
         $this->assertEquals(0, count($this->observer->get_metadata_modified()));
     }
 
-    public function test_move_then_set_metadata() {
+    public function test_move_then_set_metadata(): void {
         $q = \test_question_maker::make_question('truefalse');
         $newslot = $this->quba->add_question_in_place_of_other($this->slot, $q);
         $this->quba->start_question($this->slot);
