@@ -219,12 +219,12 @@ abstract class qtype_multianswer_subq_renderer_base extends qtype_renderer {
             'role' => 'button',
             'tabindex' => 0,
             'class' => 'feedbacktrigger btn btn-link p-0',
-            'data-toggle' => 'popover',
-            'data-container' => 'body',
-            'data-content' => $feedbackcontents,
-            'data-placement' => 'right',
-            'data-trigger' => 'hover focus',
-            'data-html' => 'true',
+            'data-bs-toggle' => 'popover',
+            'data-bs-container' => 'body',
+            'data-bs-content' => $feedbackcontents,
+            'data-bs-placement' => 'right',
+            'data-bs-trigger' => 'hover focus',
+            'data-bs-html' => 'true',
         ]);
     }
 
@@ -309,7 +309,7 @@ class qtype_multianswer_textfield_renderer extends qtype_multianswer_subq_render
             'value' => $response,
             'id' => $qa->get_qt_field_name($fieldname),
             'size' => $size,
-            'class' => 'form-control mb-1',
+            'class' => 'form-control d-inline mb-1',
         );
         if ($options->readonly) {
             $inputattributes['readonly'] = 'readonly';
@@ -332,7 +332,7 @@ class qtype_multianswer_textfield_renderer extends qtype_multianswer_subq_render
                         $qa, 'question', 'answerfeedback', $matchinganswer->id),
                 s($correctanswer->answer), $options);
 
-        $output = html_writer::start_tag('span', ['class' => 'subquestion d-flex flex-wrap align-items-center']);
+        $output = html_writer::start_tag('span', ['class' => 'subquestion']);
 
         $output .= html_writer::tag('label', $this->get_answer_label(),
                 array('class' => 'subq accesshide', 'for' => $inputattributes['id']));
@@ -377,6 +377,7 @@ class qtype_multianswer_multichoice_inline_renderer
 
         $inputattributes = array(
             'id' => $qa->get_qt_field_name($fieldname),
+            'class' => 'form-select d-inline-block mb-1',
         );
         if ($options->readonly) {
             $inputattributes['disabled'] = 'disabled';
@@ -532,7 +533,7 @@ class qtype_multianswer_multichoice_vertical_renderer extends qtype_multianswer_
     protected function all_choices_wrapper_start() {
         $wrapperstart = html_writer::start_tag('fieldset', array('class' => 'answer'));
         $legendtext = $this->get_answer_label('multichoicex', 'qtype_multianswer');
-        $wrapperstart .= html_writer::tag('legend', $legendtext, ['class' => 'sr-only']);
+        $wrapperstart .= html_writer::tag('legend', $legendtext, ['class' => 'visually-hidden']);
         return $wrapperstart;
     }
 
@@ -566,7 +567,7 @@ class qtype_multianswer_multichoice_horizontal_renderer
     protected function all_choices_wrapper_start() {
         $wrapperstart = html_writer::start_tag('fieldset', ['class' => 'answer']);
         $captiontext = $this->get_answer_label('multichoicex', 'qtype_multianswer');
-        $wrapperstart .= html_writer::tag('legend', $captiontext, ['class' => 'sr-only']);
+        $wrapperstart .= html_writer::tag('legend', $captiontext, ['class' => 'visually-hidden']);
         return $wrapperstart;
     }
 
@@ -616,6 +617,7 @@ class qtype_multianswer_multiresponse_vertical_renderer extends qtype_multianswe
         $inputattributes = array(
             'type' => 'checkbox',
             'value' => 1,
+            'class' => 'form-check-input',
         );
         if ($options->readonly) {
             $inputattributes['disabled'] = 'disabled';
@@ -648,7 +650,7 @@ class qtype_multianswer_multiresponse_vertical_renderer extends qtype_multianswe
                 unset($inputattributes['checked']);
             }
 
-            $class = 'r' . ($value % 2);
+            $class = 'form-check text-wrap text-break';
             if ($options->correctness && $isselected) {
                 $thisfrac = ($ans->fraction > 0) ? $answerfraction : 0;
                 $feedbackimg = $this->feedback_image($thisfrac);
@@ -661,7 +663,7 @@ class qtype_multianswer_multiresponse_vertical_renderer extends qtype_multianswe
             $result .= html_writer::empty_tag('input', $inputattributes);
             $result .= html_writer::tag('label', $subq->format_text($ans->answer,
                                                                     $ans->answerformat, $qa, 'question', 'answer', $ansid),
-                                        array('for' => $inputattributes['id']));
+                                        ['for' => $inputattributes['id'], 'class' => 'form-check-label text-body']);
             $result .= $feedbackimg;
 
             if ($options->feedback && $isselected && trim($ans->feedback)) {
@@ -741,7 +743,7 @@ class qtype_multianswer_multiresponse_horizontal_renderer
     extends qtype_multianswer_multiresponse_vertical_renderer {
 
     protected function choice_wrapper_start($class) {
-        return html_writer::start_tag('td', array('class' => $class));
+        return html_writer::start_tag('td', ['class' => $class . ' form-check-inline']);
     }
 
     protected function choice_wrapper_end() {
