@@ -25,6 +25,7 @@ use moodle_url;
 use core_form\dynamic_form;
 use core_reportbuilder\datasource;
 use core_reportbuilder\manager;
+use core_reportbuilder\customfield\report_handler;
 use core_reportbuilder\local\helpers\report as reporthelper;
 use core_tag_tag;
 
@@ -74,7 +75,7 @@ class report extends dynamic_form {
     /**
      * Ensure current user is able to use this form
      *
-     * A {@see \core_reportbuilder\report_access_exception} will be thrown if they can't
+     * A {@see \core_reportbuilder\exception\report_access_exception} will be thrown if they can't
      */
     protected function check_access_for_dynamic_submission(): void {
         $report = $this->get_custom_report();
@@ -92,6 +93,8 @@ class report extends dynamic_form {
     public function definition() {
         $mform = $this->_form;
 
+        $mform->addElement('header', 'general', get_string('general', 'form'));
+
         $mform->addElement('hidden', 'id');
         $mform->setType('id', PARAM_INT);
 
@@ -119,6 +122,10 @@ class report extends dynamic_form {
         $mform->addElement('tags', 'tags', get_string('tags'), [
             'component' => 'core_reportbuilder', 'itemtype' => 'reportbuilder_report',
         ]);
+
+        // Add custom fields to the form.
+        $reportid = empty($this->_customdata['id']) ? 0 : $this->_customdata['id'];
+        report_handler::create()->instance_form_definition($mform, $reportid);
     }
 
     /**
@@ -144,7 +151,9 @@ class report extends dynamic_form {
     public function set_data_for_dynamic_submission(): void {
         if ($persistent = $this->get_custom_report()?->get_report_persistent()) {
             $tags = core_tag_tag::get_item_tags_array('core_reportbuilder', 'reportbuilder_report', $persistent->get('id'));
-            $this->set_data(array_merge((array) $persistent->to_record(), ['tags' => $tags]));
+            $data = (object) array_merge((array) $persistent->to_record(), ['tags' => $tags]);
+            report_handler::create()->instance_form_before_set_data($data);
+            $this->set_data($data);
         }
     }
 
