@@ -109,7 +109,6 @@ class text extends base {
      */
     public function get_sql_filter(array $values): array {
         global $DB;
-        $name = database::generate_param_name();
 
         $operator = (int) ($values["{$this->name}_operator"] ?? self::ANY_VALUE);
         $value = trim($values["{$this->name}_value"] ?? '');
@@ -123,6 +122,8 @@ class text extends base {
             return ['', []];
         }
 
+        $name = database::generate_param_name();
+
         switch($operator) {
             case self::CONTAINS:
                 $res = $DB->sql_like($fieldsql, ":$name", false, false);
@@ -153,14 +154,10 @@ class text extends base {
                 $params[$name] = "%$value";
                 break;
             case self::IS_EMPTY:
-                $paramempty = database::generate_param_name();
-                $res = "COALESCE({$fieldsql}, :{$paramempty}) = :{$name}";
-                $params[$paramempty] = $params[$name] = '';
+                $res = "COALESCE({$fieldsql}, '') = ''";
                 break;
             case self::IS_NOT_EMPTY:
-                $paramempty = database::generate_param_name();
-                $res = "COALESCE({$fieldsql}, :{$paramempty}) != :{$name}";
-                $params[$paramempty] = $params[$name] = '';
+                $res = "COALESCE({$fieldsql}, '') != ''";
                 break;
             default:
                 // Filter configuration is invalid. Ignore the filter.
