@@ -28,9 +28,7 @@ import {debounce} from 'core/utils';
 import {isSmall, isLarge} from 'core/pagehelpers';
 import Pending from 'core/pending';
 import {setUserPreference} from 'core_user/repository';
-import * as FocusLock from 'core/local/aria/focuslock';
-// The jQuery module is only used for interacting with Boostrap 4. It can we removed when MDL-71979 is integrated.
-import jQuery from 'jquery';
+import Tooltip from './bootstrap/tooltip';
 
 let backdropPromise = null;
 
@@ -143,12 +141,11 @@ const disableDrawerTooltips = (drawerNode) => {
  */
 const disableButtonTooltip = (button, enableOnBlur) => {
     if (button.hasAttribute('data-original-title')) {
-        // The jQuery is still used in Boostrap 4. It can we removed when MDL-71979 is integrated.
-        jQuery(button).tooltip('disable');
+        Tooltip.getInstance(button).disable();
         button.setAttribute('title', button.dataset.originalTitle);
     } else {
         button.dataset.disabledToggle = button.dataset.toggle;
-        button.removeAttribute('data-toggle');
+        button.removeAttribute('data-bs-toggle');
     }
     if (enableOnBlur) {
         button.dataset.restoreTooltipOnBlur = true;
@@ -181,13 +178,12 @@ const enableDrawerTooltips = (drawerNode) => {
  * @private
  */
 const enableButtonTooltip = (button) => {
-    // The jQuery is still used in Boostrap 4. It can we removed when MDL-71979 is integrated.
-    if (button.hasAttribute('data-original-title')) {
-        jQuery(button).tooltip('enable');
+    if (button.hasAttribute('data-bs-original-title')) {
+        Tooltip.getInstance(button).enable();
         button.removeAttribute('title');
     } else if (button.dataset.disabledToggle) {
         button.dataset.toggle = button.dataset.disabledToggle;
-        jQuery(button).tooltip();
+        new Tooltip(button);
     }
     delete button.dataset.restoreTooltipOnBlur;
 };
@@ -433,8 +429,7 @@ export default class Drawers {
         // Remove open tooltip if still visible.
         let openButton = getDrawerOpenButton(this.drawerNode.id);
         if (openButton && openButton.hasAttribute('data-original-title')) {
-            // The jQuery is still used in Boostrap 4. It can we removed when MDL-71979 is integrated.
-            jQuery(openButton)?.tooltip('hide');
+            Tooltip.getInstance(openButton)?.hide();
         }
 
         Aria.unhide(this.drawerNode);
@@ -461,9 +456,7 @@ export default class Drawers {
                 pageWrapper.style.overflow = 'hidden';
                 return backdrop;
             })
-            .catch(() => {
-                return;
-            });
+            .catch();
         }
 
         // Show close button and header content once the drawer is fully opened.
@@ -478,11 +471,6 @@ export default class Drawers {
             if (focusOnCloseButton) {
                 closeButton.focus();
             }
-            // On small devices, the drawer must have a trap focus once the focus is inside
-            // to prevent the user from focussing on covered elements.
-            if (isSmall()) {
-                FocusLock.trapFocus(this.drawerNode);
-            }
             pendingPromise.resolve();
         }, 300);
 
@@ -496,7 +484,7 @@ export default class Drawers {
      * @param {boolean} [args.focusOnOpenButton=true] Whether to alter page focus when opening the drawer
      * @param {boolean} [args.updatePreferences=true] Whether to update the user prewference
      */
-        closeDrawer({focusOnOpenButton = true, updatePreferences = true} = {}) {
+    closeDrawer({focusOnOpenButton = true, updatePreferences = true} = {}) {
 
         const pendingPromise = new Pending('theme_boost/drawers:close');
 
@@ -512,8 +500,7 @@ export default class Drawers {
         headerContent?.classList.toggle('hidden', true);
         // Remove the close button tooltip if visible.
         if (closeButton.hasAttribute('data-original-title')) {
-            // The jQuery is still used in Boostrap 4. It can we removed when MDL-71979 is integrated.
-            jQuery(closeButton)?.tooltip('hide');
+            Tooltip.getInstance(closeButton)?.hide();
         }
 
         const preference = this.drawerNode.dataset.preference;
@@ -539,13 +526,8 @@ export default class Drawers {
             }
             return backdrop;
         })
-        .catch(() => {
-                return;
-        });
+        .catch();
 
-        if (isSmall()) {
-            FocusLock.untrapFocus();
-        }
         // Move focus to the open drawer (or toggler) button once the drawer is hidden.
         let openButton = getDrawerOpenButton(this.drawerNode.id);
         if (openButton) {
