@@ -17,6 +17,7 @@
 namespace core_user;
 
 use core_text;
+use core_user;
 
 /**
  * Class for retrieving information about user fields that are needed for displaying user identity.
@@ -377,19 +378,12 @@ class fields {
                 $allowed = false;
                 if ($allowcustom) {
                     require_once($CFG->dirroot . '/user/profile/lib.php');
-                    $fieldinfo = profile_get_custom_field_data_by_shortname($matches[1]);
-                    switch ($fieldinfo->visible ?? -1) {
-                        case PROFILE_VISIBLE_NONE:
-                        case PROFILE_VISIBLE_PRIVATE:
-                            $allowed = !$context || has_capability('moodle/user:viewalldetails', $context);
-                            break;
-                        case PROFILE_VISIBLE_TEACHERS:
-                            // This is actually defined (in user/profile/lib.php) based on whether
-                            // you have moodle/site:viewuseridentity in context. We already checked
-                            // that, so treat it as visible (fall through).
-                        case PROFILE_VISIBLE_ALL:
-                            $allowed = true;
-                            break;
+
+                    // Ensure the field exists (it may have been deleted since user identity was configured).
+                    $field = profile_get_custom_field_data_by_shortname($matches[1], false);
+                    if ($field !== null) {
+                        $fieldinstance = profile_get_user_field($field->datatype, $field->id, 0, $field);
+                        $allowed = $fieldinstance->is_visible($context);
                     }
                 }
                 if (!$allowed) {
@@ -543,7 +537,7 @@ class fields {
                                  $DB->sql_equal($fieldalias . '.shortname', $placeholder, false) . "
                        LEFT JOIN {user_info_data} $dataalias ON $dataalias.fieldid = $fieldalias.id
                                  AND $dataalias.userid = {$usertable}id";
-                // For Oracle we need to convert the field into a usable format.
+                // For sqlsrv we need to convert the field into a usable format.
                 $fieldsql = $DB->sql_compare_text($dataalias . '.data', 255);
                 $selects .= ", $fieldsql AS $prefix$field";
                 $mappings[$field] = $fieldsql;
@@ -592,10 +586,7 @@ class fields {
         $unique = self::$uniqueidentifier++;
 
         $namefields = self::get_name_fields();
-
-        // Create a dummy user object containing all name fields.
-        $dummyuser = (object) array_combine($namefields, $namefields);
-        $dummyfullname = fullname($dummyuser, $override);
+        $dummyfullname = core_user::get_dummy_fullname(null, ['override' => $override]);
 
         // Extract any name fields from the fullname format in the order that they appear.
         $matchednames = array_values(order_in_string($namefields, $dummyfullname));
@@ -620,7 +611,7 @@ class fields {
             }
 
             if (core_text::strlen($chunk) > 0) {
-                // If content is just whitespace, add to elements directly (also Oracle doesn't support passing ' ' as param).
+                // If content is just whitespace, add it directly to elements to handle it appropriately.
                 if (preg_match('/^\s+$/', $chunk)) {
                     $elements[] = "'$chunk'";
                 } else {
