@@ -50,7 +50,7 @@ final class externallib_test extends externallib_advanced_testcase {
     /**
      * Test get_users
      */
-    public function test_get_users() {
+    public function test_get_users(): void {
         global $USER, $CFG;
 
         $this->resetAfterTest(true);
@@ -315,6 +315,7 @@ final class externallib_test extends externallib_advanced_testcase {
                 }
                 // Default language and no theme were used for the user.
                 $this->assertEquals($CFG->lang, $returneduser['lang']);
+                $this->assertEquals($generateduser->trackforums, $returneduser['trackforums']);
                 $this->assertEmpty($returneduser['theme']);
 
                 if ($returneduser['id'] == $user1->id) {
@@ -422,7 +423,7 @@ final class externallib_test extends externallib_advanced_testcase {
     /**
      * Test get_course_user_profiles
      */
-    public function test_get_course_user_profiles() {
+    public function test_get_course_user_profiles(): void {
         global $USER, $CFG;
 
         $this->resetAfterTest(true);
@@ -440,7 +441,7 @@ final class externallib_test extends externallib_advanced_testcase {
         $this->assertEquals(1, count($enrolledusers));
     }
 
-    public function test_get_user_course_profile_as_admin() {
+    public function test_get_user_course_profile_as_admin(): void {
         global $USER, $CFG;
 
         global $USER, $CFG;
@@ -493,12 +494,12 @@ final class externallib_test extends externallib_advanced_testcase {
     /**
      * Test create_users
      */
-    public function test_create_users() {
+    public function test_create_users(): void {
         global $DB;
 
         $this->resetAfterTest(true);
 
-        $user1 = array(
+        $user1 = [
             'username' => 'usernametest1',
             'password' => 'Moodle2012!',
             'idnumber' => 'idnumbertest1',
@@ -512,20 +513,22 @@ final class externallib_test extends externallib_advanced_testcase {
             'description' => 'This is a description for user 1',
             'city' => 'Perth',
             'country' => 'AU',
-            'preferences' => [[
+            'preferences' => [
+                [
                     'type' => 'htmleditor',
-                    'value' => 'atto'
-                ], [
+                    'value' => 'tiny',
+                ],
+                [
                     'type' => 'invalidpreference',
-                    'value' => 'abcd'
-                ]
+                    'value' => 'abcd',
+                ],
             ],
             'department' => 'College of Science',
             'institution' => 'National Institute of Physics',
             'phone1' => '01 2345 6789',
             'maildisplay' => 1,
-            'interests' => 'badminton, basketball, cooking,  '
-        );
+            'interests' => 'badminton, basketball, cooking,  ',
+        ];
 
         // User with an authentication method done externally.
         $user2 = array(
@@ -554,7 +557,7 @@ final class externallib_test extends externallib_advanced_testcase {
 
             if ($createduser['username'] === $user1['username']) {
                 $usertotest = $user1;
-                $this->assertEquals('atto', get_user_preferences('htmleditor', null, $dbuser));
+                $this->assertEquals('tiny', get_user_preferences('htmleditor', null, $dbuser));
                 $this->assertEquals(null, get_user_preferences('invalidpreference', null, $dbuser));
                 // Confirm user interests have been saved.
                 $interests = \core_tag_tag::get_item_tags_array('core', 'user', $createduser['id'],
@@ -592,7 +595,7 @@ final class externallib_test extends externallib_advanced_testcase {
     /**
      * Test create_users with password and createpassword parameter not set.
      */
-    public function test_create_users_empty_password() {
+    public function test_create_users_empty_password(): void {
         $this->resetAfterTest();
         $this->setAdminUser();
 
@@ -611,7 +614,7 @@ final class externallib_test extends externallib_advanced_testcase {
     /**
      * Data provider for \core_user_externallib_testcase::test_create_users_with_same_emails().
      */
-    public function create_users_provider_with_same_emails() {
+    public static function create_users_provider_with_same_emails(): array {
         return [
             'Same emails allowed, same case' => [
                 1, false
@@ -635,7 +638,7 @@ final class externallib_test extends externallib_advanced_testcase {
      * @param int $sameemailallowed The value to set for $CFG->allowaccountssameemail.
      * @param boolean $differentcase Whether to user a different case for the other user.
      */
-    public function test_create_users_with_same_emails($sameemailallowed, $differentcase) {
+    public function test_create_users_with_same_emails($sameemailallowed, $differentcase): void {
         global $DB;
 
         $this->resetAfterTest();
@@ -680,7 +683,7 @@ final class externallib_test extends externallib_advanced_testcase {
      * @param array $data User data to attempt to register.
      * @param string $expectmessage Expected exception message.
      */
-    public function test_create_users_invalid_parameter(array $data, $expectmessage) {
+    public function test_create_users_invalid_parameter(array $data, $expectmessage): void {
         global $USER, $CFG, $DB;
 
         $this->resetAfterTest(true);
@@ -697,7 +700,7 @@ final class externallib_test extends externallib_advanced_testcase {
      *
      * @return array
      */
-    public function data_create_users_invalid_parameter() {
+    public static function data_create_users_invalid_parameter(): array {
         return [
             'blank_username' => [
                 'data' => [
@@ -754,7 +757,7 @@ final class externallib_test extends externallib_advanced_testcase {
     /**
      * Test delete_users
      */
-    public function test_delete_users() {
+    public function test_delete_users(): void {
         global $USER, $CFG, $DB;
 
         $this->resetAfterTest(true);
@@ -785,7 +788,7 @@ final class externallib_test extends externallib_advanced_testcase {
     /**
      * Test update_users
      */
-    public function test_update_users() {
+    public function test_update_users(): void {
         global $USER, $CFG, $DB;
 
         $this->resetAfterTest(true);
@@ -809,7 +812,7 @@ final class externallib_test extends externallib_advanced_testcase {
 
         $user1 = self::getDataGenerator()->create_user();
 
-        $user1 = array(
+        $user1 = [
             'id' => $user1->id,
             'username' => 'usernametest1',
             'password' => 'Moodle2012!',
@@ -825,20 +828,22 @@ final class externallib_test extends externallib_advanced_testcase {
             'city' => 'Perth',
             'userpicture' => $draftid,
             'country' => 'AU',
-            'preferences' => [[
+            'preferences' => [
+                [
                     'type' => 'htmleditor',
-                    'value' => 'atto'
-                ], [
+                    'value' => 'textarea',
+                ],
+                [
                     'type' => 'invialidpreference',
-                    'value' => 'abcd'
-                ]
+                    'value' => 'abcd',
+                ],
             ],
             'department' => 'College of Science',
             'institution' => 'National Institute of Physics',
             'phone1' => '01 2345 6789',
             'maildisplay' => 1,
-            'interests' => 'badminton, basketball, cooking,  '
-        );
+            'interests' => 'badminton, basketball, cooking,  ',
+        ];
 
         $context = \context_system::instance();
         $roleid = $this->assignUserCapability('moodle/user:update', $context->id);
@@ -891,7 +896,7 @@ final class externallib_test extends externallib_advanced_testcase {
         $this->assertEquals($dbuser->institution, $user1['institution']);
         $this->assertEquals($dbuser->phone1, $user1['phone1']);
         $this->assertEquals($dbuser->maildisplay, $user1['maildisplay']);
-        $this->assertEquals('atto', get_user_preferences('htmleditor', null, $dbuser));
+        $this->assertEquals('textarea', get_user_preferences('htmleditor', null, $dbuser));
         $this->assertEquals(null, get_user_preferences('invalidpreference', null, $dbuser));
 
         // Confirm user interests have been saved.
@@ -956,7 +961,7 @@ final class externallib_test extends externallib_advanced_testcase {
      *
      * @return array
      */
-    public function users_with_same_emails() {
+    public static function users_with_same_emails(): array {
         return [
             'Same emails not allowed: Update name using exactly the same email' => [
                 0, 'John', 's1@example.com', 'Johnny', 's1@example.com', false, true
@@ -992,7 +997,7 @@ final class externallib_test extends externallib_advanced_testcase {
      * @param boolean $successexpected Whether we expect that the target user's email/name will be updated.
      */
     public function test_update_users_emails_with_different_cases($allowsameemail, $currentname, $currentemail,
-                                                                  $newname, $newemail, $withanotheruser, $successexpected) {
+                                                                  $newname, $newemail, $withanotheruser, $successexpected): void {
         global $DB;
 
         $this->resetAfterTest();
@@ -1039,7 +1044,7 @@ final class externallib_test extends externallib_advanced_testcase {
     /**
      * Test add_user_private_files
      */
-    public function test_add_user_private_files() {
+    public function test_add_user_private_files(): void {
         global $USER, $CFG, $DB;
 
         $this->resetAfterTest(true);
@@ -1085,7 +1090,7 @@ final class externallib_test extends externallib_advanced_testcase {
     /**
      * Test add_user_private_files quota
      */
-    public function test_add_user_private_files_quota() {
+    public function test_add_user_private_files_quota(): void {
         global $USER, $CFG, $DB;
 
         $this->resetAfterTest(true);
@@ -1133,7 +1138,7 @@ final class externallib_test extends externallib_advanced_testcase {
     /**
      * Test add user device
      */
-    public function test_add_user_device() {
+    public function test_add_user_device(): void {
         global $USER, $CFG, $DB;
 
         $this->resetAfterTest(true);
@@ -1188,7 +1193,7 @@ final class externallib_test extends externallib_advanced_testcase {
     /**
      * Test remove user device
      */
-    public function test_remove_user_device() {
+    public function test_remove_user_device(): void {
         global $USER, $CFG, $DB;
 
         $this->resetAfterTest(true);
@@ -1243,7 +1248,7 @@ final class externallib_test extends externallib_advanced_testcase {
     /**
      * Test get_user_preferences
      */
-    public function test_get_user_preferences() {
+    public function test_get_user_preferences(): void {
         $this->resetAfterTest(true);
 
         $user = self::getDataGenerator()->create_user();
@@ -1306,7 +1311,7 @@ final class externallib_test extends externallib_advanced_testcase {
     /**
      * Test update_picture
      */
-    public function test_update_picture() {
+    public function test_update_picture(): void {
         global $DB, $USER;
 
         $this->resetAfterTest(true);
@@ -1359,7 +1364,7 @@ final class externallib_test extends externallib_advanced_testcase {
     /**
      * Test update_picture disabled
      */
-    public function test_update_picture_disabled() {
+    public function test_update_picture_disabled(): void {
         global $CFG;
         $this->resetAfterTest(true);
         $CFG->disableuserimages = true;
@@ -1372,7 +1377,7 @@ final class externallib_test extends externallib_advanced_testcase {
     /**
      * Test set_user_preferences
      */
-    public function test_set_user_preferences_save() {
+    public function test_set_user_preferences_save(): void {
         global $DB;
         $this->resetAfterTest(true);
 
@@ -1384,7 +1389,7 @@ final class externallib_test extends externallib_advanced_testcase {
         $preferences = array(
             array(
                 'name' => 'htmleditor',
-                'value' => 'atto',
+                'value' => 'textarea',
                 'userid' => $user1->id,
             ),
             array(
@@ -1400,7 +1405,7 @@ final class externallib_test extends externallib_advanced_testcase {
         $this->assertCount(2, $result['saved']);
 
         // Get preference from DB to avoid cache.
-        $this->assertEquals('atto', $DB->get_field('user_preferences', 'value',
+        $this->assertEquals('textarea', $DB->get_field('user_preferences', 'value',
             array('userid' => $user1->id, 'name' => 'htmleditor')));
         $this->assertEquals('tiny', $DB->get_field('user_preferences', 'value',
             array('userid' => $user2->id, 'name' => 'htmleditor')));
@@ -1409,7 +1414,7 @@ final class externallib_test extends externallib_advanced_testcase {
     /**
      * Test set_user_preferences
      */
-    public function test_set_user_preferences_save_invalid_pref() {
+    public function test_set_user_preferences_save_invalid_pref(): void {
         global $DB;
         $this->resetAfterTest(true);
 
@@ -1438,7 +1443,7 @@ final class externallib_test extends externallib_advanced_testcase {
     /**
      * Test set_user_preferences for an invalid user
      */
-    public function test_set_user_preferences_invalid_user() {
+    public function test_set_user_preferences_invalid_user(): void {
         $this->resetAfterTest(true);
 
         $this->setAdminUser();
@@ -1461,7 +1466,7 @@ final class externallib_test extends externallib_advanced_testcase {
     /**
      * Test set_user_preferences using an invalid preference
      */
-    public function test_set_user_preferences_invalid_preference() {
+    public function test_set_user_preferences_invalid_preference(): void {
         global $USER, $DB;
 
         $this->resetAfterTest(true);
@@ -1487,7 +1492,7 @@ final class externallib_test extends externallib_advanced_testcase {
     /**
      * Test set_user_preferences for other user not being admin
      */
-    public function test_set_user_preferences_capability() {
+    public function test_set_user_preferences_capability(): void {
         $this->resetAfterTest(true);
 
         $user1 = self::getDataGenerator()->create_user();
@@ -1513,7 +1518,7 @@ final class externallib_test extends externallib_advanced_testcase {
     /**
      * Test update_user_preferences unsetting an existing preference.
      */
-    public function test_update_user_preferences_unset() {
+    public function test_update_user_preferences_unset(): void {
         global $DB;
         $this->resetAfterTest(true);
 
@@ -1524,7 +1529,7 @@ final class externallib_test extends externallib_advanced_testcase {
         $preferences = array(
             array(
                 'name' => 'htmleditor',
-                'value' => 'atto',
+                'value' => 'textarea',
                 'userid' => $user->id,
             )
         );
@@ -1535,7 +1540,7 @@ final class externallib_test extends externallib_advanced_testcase {
         $this->assertCount(1, $result['saved']);
 
         // Get preference from DB to avoid cache.
-        $this->assertEquals('atto', $DB->get_field('user_preferences', 'value',
+        $this->assertEquals('textarea', $DB->get_field('user_preferences', 'value',
             array('userid' => $user->id, 'name' => 'htmleditor')));
 
         // Now, unset.
@@ -1547,7 +1552,7 @@ final class externallib_test extends externallib_advanced_testcase {
     /**
      * Test agree_site_policy
      */
-    public function test_agree_site_policy() {
+    public function test_agree_site_policy(): void {
         global $CFG, $DB, $USER;
         $this->resetAfterTest(true);
 
@@ -1598,7 +1603,7 @@ final class externallib_test extends externallib_advanced_testcase {
     /**
      * Test get_private_files_info
      */
-    public function test_get_private_files_info() {
+    public function test_get_private_files_info(): void {
 
         $this->resetAfterTest(true);
         $user = self::getDataGenerator()->create_user();
@@ -1638,7 +1643,7 @@ final class externallib_test extends externallib_advanced_testcase {
     /**
      * Test get_private_files_info missing permissions.
      */
-    public function test_get_private_files_info_missing_permissions() {
+    public function test_get_private_files_info_missing_permissions(): void {
 
         $this->resetAfterTest(true);
         $user1 = self::getDataGenerator()->create_user();
@@ -1653,7 +1658,7 @@ final class externallib_test extends externallib_advanced_testcase {
     /**
      * Test the functionality of the {@see \core_user\external\search_identity} class.
      */
-    public function test_external_search_identity() {
+    public function test_external_search_identity(): void {
         global $CFG;
 
         $this->resetAfterTest(true);
@@ -1762,7 +1767,7 @@ final class externallib_test extends externallib_advanced_testcase {
     /**
      * Test functionality of the {@see \core_user\external\search_identity} class with alternativefullnameformat defined.
      */
-    public function test_external_search_identity_with_alternativefullnameformat() {
+    public function test_external_search_identity_with_alternativefullnameformat(): void {
         global $CFG;
 
         $this->resetAfterTest(true);
@@ -1798,7 +1803,7 @@ final class externallib_test extends externallib_advanced_testcase {
     /**
      * Test verifying that update_user_preferences prevents changes to the default homepage for other users.
      */
-    public function test_update_user_preferences_homepage_permission_callback() {
+    public function test_update_user_preferences_homepage_permission_callback(): void {
         global $DB;
         $this->resetAfterTest();
 
