@@ -22,13 +22,14 @@ namespace core_user;
  * @package core
  * @copyright 2014 The Open University
  * @license http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
+ * @covers \core_user\fields
  */
-class fields_testcase extends \advanced_testcase {
+final class fields_test extends \advanced_testcase {
 
     /**
      * Tests getting the user picture fields.
      */
-    public function test_get_picture_fields() {
+    public function test_get_picture_fields(): void {
         $this->assertEquals(['id', 'picture', 'firstname', 'lastname', 'firstnamephonetic',
                 'lastnamephonetic', 'middlename', 'alternatename', 'imagealt', 'email'],
                 fields::get_picture_fields());
@@ -37,7 +38,7 @@ class fields_testcase extends \advanced_testcase {
     /**
      * Tests getting the user name fields.
      */
-    public function test_get_name_fields() {
+    public function test_get_name_fields(): void {
         $this->assertEquals(['firstnamephonetic', 'lastnamephonetic', 'middlename', 'alternatename',
                 'firstname', 'lastname'],
                 fields::get_name_fields());
@@ -50,19 +51,27 @@ class fields_testcase extends \advanced_testcase {
     /**
      * Tests getting the identity fields.
      */
-    public function test_get_identity_fields() {
-        global $DB;
+    public function test_get_identity_fields(): void {
+        global $DB, $CFG, $COURSE;
 
         $this->resetAfterTest();
 
-        // Create two custom profile fields, one of which is private.
+        require_once($CFG->dirroot . '/user/profile/lib.php');
+
+        // Create custom profile fields, one with each visibility option.
         $generator = self::getDataGenerator();
-        $generator->create_custom_profile_field(['datatype' => 'text', 'shortname' => 'a', 'name' => 'A']);
+        $generator->create_custom_profile_field(['datatype' => 'text', 'shortname' => 'a', 'name' => 'A',
+                'visible' => PROFILE_VISIBLE_ALL]);
         $generator->create_custom_profile_field(['datatype' => 'text', 'shortname' => 'b', 'name' => 'B',
                 'visible' => PROFILE_VISIBLE_PRIVATE]);
+        $generator->create_custom_profile_field(['datatype' => 'text', 'shortname' => 'c', 'name' => 'C',
+                'visible' => PROFILE_VISIBLE_NONE]);
+        $generator->create_custom_profile_field(['datatype' => 'text', 'shortname' => 'd', 'name' => 'D',
+                'visible' => PROFILE_VISIBLE_TEACHERS]);
 
-        // Set the extra user fields to include email, department, and both custom profile fields.
-        set_config('showuseridentity', 'email,department,profile_field_a,profile_field_b');
+        // Set the extra user fields to include email, department, and all custom profile fields.
+        set_config('showuseridentity', 'email,department,profile_field_a,profile_field_b,' .
+                'profile_field_c,profile_field_d');
         set_config('hiddenuserfields', 'email');
 
         // Create a test course and a student in the course.
@@ -73,8 +82,8 @@ class fields_testcase extends \advanced_testcase {
         $usercontext = \context_user::instance($anotheruser->id);
         $generator->enrol_user($user->id, $course->id, 'student');
 
-        // When no context is provided, it does no access checks and should return all specified.
-        $this->assertEquals(['email', 'department', 'profile_field_a', 'profile_field_b'],
+        // When no context is provided, it does no access checks and should return all specified (other than non-visible).
+        $this->assertEquals(['email', 'department', 'profile_field_a', 'profile_field_b', 'profile_field_d'],
                 fields::get_identity_fields(null));
 
         // If you turn off custom profile fields, you don't get those.
@@ -82,7 +91,8 @@ class fields_testcase extends \advanced_testcase {
 
         // Request in context as an administator.
         $this->setAdminUser();
-        $this->assertEquals(['email', 'department', 'profile_field_a', 'profile_field_b'],
+        $this->assertEquals(['email', 'department', 'profile_field_a', 'profile_field_b',
+                'profile_field_c', 'profile_field_d'],
                 fields::get_identity_fields($coursecontext));
         $this->assertEquals(['email', 'department'],
                 fields::get_identity_fields($coursecontext, false));
@@ -92,24 +102,27 @@ class fields_testcase extends \advanced_testcase {
         $this->setUser($user);
         $this->assertEquals([], fields::get_identity_fields($coursecontext));
 
-        // Give the student the basic identity fields permission.
+        // Give the student the basic identity fields permission (also makes them count as 'teacher'
+        // for the teacher-restricted field).
+        $COURSE = $course; // Horrible hack, because PROFILE_VISIBLE_TEACHERS relies on this global.
         $roleid = $DB->get_field('role', 'id', ['shortname' => 'student']);
         role_change_permission($roleid, $coursecontext, 'moodle/site:viewuseridentity', CAP_ALLOW);
-        $this->assertEquals(['department', 'profile_field_a'],
+        $this->assertEquals(['department', 'profile_field_a', 'profile_field_d'],
                 fields::get_identity_fields($coursecontext));
         $this->assertEquals(['department'],
                 fields::get_identity_fields($coursecontext, false));
 
         // Give them permission to view hidden user fields.
         role_change_permission($roleid, $coursecontext, 'moodle/course:viewhiddenuserfields', CAP_ALLOW);
-        $this->assertEquals(['email', 'department', 'profile_field_a'],
+        $this->assertEquals(['email', 'department', 'profile_field_a', 'profile_field_d'],
                 fields::get_identity_fields($coursecontext));
         $this->assertEquals(['email', 'department'],
                 fields::get_identity_fields($coursecontext, false));
 
         // Also give them permission to view all profile fields.
         role_change_permission($roleid, $coursecontext, 'moodle/user:viewalldetails', CAP_ALLOW);
-        $this->assertEquals(['email', 'department', 'profile_field_a', 'profile_field_b'],
+        $this->assertEquals(['email', 'department', 'profile_field_a', 'profile_field_b',
+                'profile_field_c', 'profile_field_d'],
                 fields::get_identity_fields($coursecontext));
         $this->assertEquals(['email', 'department'],
                 fields::get_identity_fields($coursecontext, false));
@@ -120,26 +133,48 @@ class fields_testcase extends \advanced_testcase {
 
         // Give them basic permission.
         role_change_permission($roleid, $usercontext, 'moodle/site:viewuseridentity', CAP_ALLOW);
-        $this->assertEquals(['department', 'profile_field_a'],
+        $this->assertEquals(['department', 'profile_field_a', 'profile_field_d'],
                 fields::get_identity_fields($usercontext));
         $this->assertEquals(['department'],
                 fields::get_identity_fields($usercontext, false));
 
         // Give them the hidden user fields permission (it's a different one).
         role_change_permission($roleid, $usercontext, 'moodle/user:viewhiddendetails', CAP_ALLOW);
-        $this->assertEquals(['email', 'department', 'profile_field_a'],
+        $this->assertEquals(['email', 'department', 'profile_field_a', 'profile_field_d'],
                 fields::get_identity_fields($usercontext));
         $this->assertEquals(['email', 'department'],
                 fields::get_identity_fields($usercontext, false));
 
         // Also give them permission to view all profile fields.
         role_change_permission($roleid, $usercontext, 'moodle/user:viewalldetails', CAP_ALLOW);
-        $this->assertEquals(['email', 'department', 'profile_field_a', 'profile_field_b'],
+        $this->assertEquals(['email', 'department', 'profile_field_a', 'profile_field_b',
+                'profile_field_c', 'profile_field_d'],
                 fields::get_identity_fields($usercontext));
         $this->assertEquals(['email', 'department'],
                 fields::get_identity_fields($usercontext, false));
     }
 
+    /**
+     * Test getting identity fields, when one of them refers to a non-existing custom profile field
+     */
+    public function test_get_identity_fields_invalid(): void {
+        $this->resetAfterTest();
+
+        $this->getDataGenerator()->create_custom_profile_field([
+            'datatype' => 'text',
+            'shortname' => 'real',
+            'name' => 'I\'m real',
+        ]);
+
+        // The "fake" profile field does not exist.
+        set_config('showuseridentity', 'email,profile_field_real,profile_field_fake');
+
+        $this->assertEquals([
+            'email',
+            'profile_field_real',
+        ], fields::get_identity_fields(null));
+    }
+
     /**
      * Tests the get_required_fields function.
      *
@@ -147,7 +182,7 @@ class fields_testcase extends \advanced_testcase {
      * to test the details of the identity permissions as that was already covered. Just how they
      * are included/combined.
      */
-    public function test_get_required_fields() {
+    public function test_get_required_fields(): void {
         $this->resetAfterTest();
 
         // Set up some profile fields.
@@ -197,7 +232,7 @@ class fields_testcase extends \advanced_testcase {
     /**
      * Tests the get_required_fields function when you use the $limitpurposes parameter.
      */
-    public function test_get_required_fields_limitpurposes() {
+    public function test_get_required_fields_limitpurposes(): void {
         $this->resetAfterTest();
 
         // Set up some profile fields.
@@ -229,7 +264,7 @@ class fields_testcase extends \advanced_testcase {
     /**
      * There should be an exception if you try to 'limit' purposes to one that wasn't even included.
      */
-    public function test_get_required_fields_limitpurposes_not_in_constructor() {
+    public function test_get_required_fields_limitpurposes_not_in_constructor(): void {
         $fields = fields::for_identity(null);
         $this->expectExceptionMessage('$limitpurposes can only include purposes defined in object');
         $fields->get_required_fields([fields::PURPOSE_USERPIC]);
@@ -261,7 +296,7 @@ class fields_testcase extends \advanced_testcase {
     /**
      * Tests getting SQL (and actually using it).
      */
-    public function test_get_sql_variations() {
+    public function test_get_sql_variations(): void {
         global $DB;
         $this->resetAfterTest();
 
@@ -356,12 +391,12 @@ class fields_testcase extends \advanced_testcase {
         $this->assertCount(2, $records);
 
         // User id was renamed.
-        $this->assertObjectNotHasAttribute('id', $records['XXX1']);
-        $this->assertObjectHasAttribute('userid', $records['XXX1']);
+        $this->assertObjectNotHasProperty('id', $records['XXX1']);
+        $this->assertObjectHasProperty('userid', $records['XXX1']);
 
         // Other fields are normal (just try a couple).
-        $this->assertObjectHasAttribute('firstname', $records['XXX1']);
-        $this->assertObjectHasAttribute('imagealt', $records['XXX1']);
+        $this->assertObjectHasProperty('firstname', $records['XXX1']);
+        $this->assertObjectHasProperty('imagealt', $records['XXX1']);
 
         // Check the user id is actually right.
         $this->assertEquals('XXX1',
@@ -380,13 +415,13 @@ class fields_testcase extends \advanced_testcase {
         $this->assertCount(2, $records);
 
         // User id was renamed.
-        $this->assertObjectNotHasAttribute('id', $records['XXX1']);
-        $this->assertObjectNotHasAttribute('u_id', $records['XXX1']);
-        $this->assertObjectHasAttribute('userid', $records['XXX1']);
+        $this->assertObjectNotHasProperty('id', $records['XXX1']);
+        $this->assertObjectNotHasProperty('u_id', $records['XXX1']);
+        $this->assertObjectHasProperty('userid', $records['XXX1']);
 
         // Other fields are prefixed (just try a couple).
-        $this->assertObjectHasAttribute('u_firstname', $records['XXX1']);
-        $this->assertObjectHasAttribute('u_imagealt', $records['XXX1']);
+        $this->assertObjectHasProperty('u_firstname', $records['XXX1']);
+        $this->assertObjectHasProperty('u_imagealt', $records['XXX1']);
 
         // Without a leading comma.
         ['selects' => $selects, 'joins' => $joins, 'params' => $joinparams] =
@@ -402,8 +437,8 @@ class fields_testcase extends \advanced_testcase {
             // ID should be the first field used by get_records_sql.
             $this->assertEquals($key, $record->id);
             // Check 2 other sample properties.
-            $this->assertObjectHasAttribute('firstname', $record);
-            $this->assertObjectHasAttribute('imagealt', $record);
+            $this->assertObjectHasProperty('firstname', $record);
+            $this->assertObjectHasProperty('imagealt', $record);
         }
     }
 
@@ -411,7 +446,7 @@ class fields_testcase extends \advanced_testcase {
      * Tests what happens if you use the SQL multiple times in a query (i.e. that it correctly
      * creates the different identifiers).
      */
-    public function test_get_sql_multiple() {
+    public function test_get_sql_multiple(): void {
         global $DB;
         $this->resetAfterTest();
 
@@ -451,7 +486,7 @@ class fields_testcase extends \advanced_testcase {
     /**
      * Tests the get_sql function when there are no fields to retrieve.
      */
-    public function test_get_sql_nothing() {
+    public function test_get_sql_nothing(): void {
         $fields = fields::empty();
         ['selects' => $selects, 'joins' => $joins, 'params' => $joinparams] = (array)$fields->get_sql();
         $this->assertEquals('', $selects);
@@ -463,7 +498,7 @@ class fields_testcase extends \advanced_testcase {
      * Tests get_sql when there are no custom fields; in this scenario, the joins and joinparams
      * are always blank.
      */
-    public function test_get_sql_no_custom_fields() {
+    public function test_get_sql_no_custom_fields(): void {
         $fields = fields::empty()->including('city', 'country');
         ['selects' => $selects, 'joins' => $joins, 'params' => $joinparams, 'mappings' => $mappings] =
                 (array)$fields->get_sql('u');
@@ -477,7 +512,7 @@ class fields_testcase extends \advanced_testcase {
      * Tests the format of the $selects string, which is important particularly for backward
      * compatibility.
      */
-    public function test_get_sql_selects_format() {
+    public function test_get_sql_selects_format(): void {
         global $DB;
 
         $this->resetAfterTest();
@@ -508,4 +543,77 @@ class fields_testcase extends \advanced_testcase {
         $selects = $fields->get_sql()->selects;
         $this->assertEquals(', id, city', $selects);
     }
+
+    /**
+     * Data provider for {@see test_get_sql_fullname}
+     *
+     * @return array
+     */
+    public static function get_sql_fullname_provider(): array {
+        return [
+            ['firstname lastname', 'FN LN'],
+            ['lastname, firstname', 'LN, FN'],
+            ['alternatename \'middlename\' lastname!', 'AN \'MN\' LN!'],
+            ['[firstname lastname alternatename]', '[FN LN AN]'],
+            ['firstnamephonetic lastnamephonetic', 'FNP LNP'],
+            ['firstname alternatename lastname', 'FN AN LN'],
+        ];
+    }
+
+    /**
+     * Test sql_fullname_display method with various fullname formats
+     *
+     * @param string $fullnamedisplay
+     * @param string $expectedfullname
+     *
+     * @dataProvider get_sql_fullname_provider
+     */
+    public function test_get_sql_fullname(string $fullnamedisplay, string $expectedfullname): void {
+        global $DB;
+
+        $this->resetAfterTest();
+
+        set_config('fullnamedisplay', $fullnamedisplay);
+        $user = $this->getDataGenerator()->create_user([
+            'firstname' => 'FN',
+            'lastname' => 'LN',
+            'firstnamephonetic' => 'FNP',
+            'lastnamephonetic' => 'LNP',
+            'middlename' => 'MN',
+            'alternatename' => 'AN',
+        ]);
+
+        [$sqlfullname, $params] = fields::get_sql_fullname('u');
+        $fullname = $DB->get_field_sql("SELECT {$sqlfullname} FROM {user} u WHERE u.id = :id", $params + [
+            'id' => $user->id,
+        ]);
+
+        $this->assertEquals($expectedfullname, $fullname);
+    }
+
+    /**
+     * Test sql_fullname_display when one of the configured name fields is null
+     */
+    public function test_get_sql_fullname_null_field(): void {
+        global $DB;
+
+        $this->resetAfterTest();
+
+        set_config('fullnamedisplay', 'firstname lastname alternatename');
+        $user = $this->getDataGenerator()->create_user([
+            'firstname' => 'FN',
+            'lastname' => 'LN',
+        ]);
+
+        // Set alternatename field to null, ensure we still get result in later assertion.
+        $user->alternatename = null;
+        user_update_user($user, false);
+
+        [$sqlfullname, $params] = fields::get_sql_fullname('u');
+        $fullname = $DB->get_field_sql("SELECT {$sqlfullname} FROM {user} u WHERE u.id = :id", $params + [
+            'id' => $user->id,
+        ]);
+
+        $this->assertEquals('FN LN ', $fullname);
+    }
 }
